@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -12,33 +13,39 @@
  * the Software, and to permit persons who receive copies from any such
  * party to do so, with the only requirement being that the above copyright
  * and this permission notice remain intact.
+ *
  */
 
 /********************** DECLARATIONS ********************/
 
 /* IMPORTS */
 
-#include "fig.h"
-#include "resources.h"
-#include "mode.h"
-#include "object.h"
-#include "paintop.h"
 #include "d_arc.h"
+
+#include <stddef.h>
+#include <math.h>
+#include <X11/Xlib.h>
+
+#include "resources.h"
+#include "object.h"
+#include "mode.h"
+#include "paintop.h"
+#include "f_util.h"
 #include "u_create.h"
+#include "u_draw.h"
 #include "u_elastic.h"
+#include "u_free.h"
 #include "u_list.h"
+#include "u_markers.h"
+#include "u_redraw.h"
 #include "w_canvas.h"
 #include "w_cursor.h"
 #include "w_drawprim.h"
 #include "w_msgpanel.h"
 #include "w_mousefun.h"
 #include "u_geom.h"
+#include "xfig_math.h"
 
-#include "f_util.h"
-#include "u_draw.h"
-#include "u_free.h"
-#include "u_markers.h"
-#include "u_redraw.h"
 
 /* EXPORT */
 
