@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,8 +16,16 @@
  *
  */
 
+#ifndef D_ARC_H
+#define D_ARC_H
+
+#include <X11/Intrinsic.h>
+#include "object.h"
+
 extern F_pos	point[3];
 extern F_pos	center_point;
 extern Boolean	center_marked;
 extern void	elastic_arc(void);
 extern void arc_drawing_selected (void);
+
+#endif
