@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,20 +16,23 @@
  *
  */
 
-#include "fig.h"
+#include "d_ellipse.h"
+
+#include <stdlib.h>
+#include <math.h>
+
 #include "resources.h"
 #include "mode.h"
 #include "object.h"
-#include "paintop.h"
 #include "u_create.h"
 #include "u_elastic.h"
 #include "u_list.h"
-#include "w_canvas.h"
-#include "w_mousefun.h"
-
 #include "u_markers.h"
 #include "u_redraw.h"
+#include "w_canvas.h"
 #include "w_cursor.h"
+#include "w_mousefun.h"
+#include "xfig_math.h"
 
 /*************************  local procedures  ********************/
 
