@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,40 +16,40 @@
  *
  */
 
-#include "fig.h"
+#include "d_line.h"
+
+#include <stdlib.h>
+#include <X11/Xlib.h>
+
 #include "resources.h"
 #include "mode.h"
 #include "object.h"
 #include "paintop.h"
-#include "d_line.h"
 #include "e_edit.h"
-#include "e_scale.h"
 #include "u_create.h"
 #include "u_elastic.h"
-#include "u_list.h"
-#include "w_canvas.h"
-#include "w_mousefun.h"
-#include "w_setup.h"
-
 #include "u_free.h"
+#include "u_list.h"
 #include "u_redraw.h"
+#include "w_canvas.h"
 #include "w_cursor.h"
 #include "w_drawprim.h"
+#include "w_mousefun.h"
 #include "w_msgpanel.h"
 
-Boolean	freehand_line;
-Boolean	dimension_line;
 
+Boolean	freehand_line;
 
 /* LOCAL */
 
+static Boolean	dimension_line;
+
 static void	    init_line_drawing(int x, int y, int shift);
 static void	    init_line_freehand_drawing(int x, int y);
 
 /**********************	 polyline and polygon section  **********************/
 
 
-
 void
 line_drawing_selected(void)
 {
@@ -299,10 +300,11 @@ create_lineobject(int x, int y)
 	elastic_moveline(first_point);	/* erase temporary outline */
     }
     if (dimension_line) {
-	comp = create_dimension_line(line, True);
-	reset_action_on(); /* this signals redisplay_curobj() not to refresh */
-	/* draw it and anything on top of it */
-	redisplay_compound(comp);
+	if ((comp = create_dimension_line(line, True)) != NULL) {
+		reset_action_on();/* signal redisplay_curobj() not to refresh */
+		/* draw it and anything on top of it */
+		redisplay_compound(comp);
+	}
     } else {
 	add_line(line);
 	reset_action_on(); /* this signals redisplay_curobj() not to refresh */
