@@ -1,8 +1,11 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1992 by James Tough
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 1992 by James Tough
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,15 +18,18 @@
  *
  */
 
-#include "fig.h"
-#include "resources.h"
-#include "object.h"
 #include "d_subspline.h"
+
+#include <stddef.h>
+#include <X11/Intrinsic.h>
+#include <X11/Xlib.h>
+
+#include "object.h"
 #include "u_create.h"
 #include "u_draw.h"
+#include "u_free.h"
 #include "u_list.h"
 
-#include "u_free.h"
 
 /*************************** local declarations *********************/
 
