@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,26 +16,26 @@
  *
  */
 
-extern void	char_handler(XKeyEvent *kpe, unsigned char c, KeySym keysym);
-extern void	draw_char_string(void);
-extern void	erase_char_string(void);
+#ifndef D_TEXT_H
+#define D_TEXT_H
+
+#if defined HAVE_CONFIG_H && !defined VERSION
+#include "config.h"
+#endif
+
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include "object.h"
+
+extern int		work_font;
+extern XFontStruct	*canvas_font;
+extern void	char_handler(unsigned char *c, int clen, KeySym keysym);
 extern void	finish_text_input(int x, int y, int shift);
-extern void	reload_text_fstruct(F_text *t);
-extern void	reload_text_fstructs(void);
-extern Boolean	text_selection_active;
-extern Boolean	ConvertSelection();
-extern void	LoseSelection(), TransferSelectionDone();
-extern int	work_font;
 
-#ifdef I18N
 extern XIC	xim_ic;
 extern Boolean	xim_active;
 extern Boolean	xim_initialize(Widget w);
-extern void	i18n_char_handler(unsigned char *str);
-extern void	prefix_append_char(unsigned char ch);
 extern void	xim_set_ic_geometry(XIC ic, int width, int height);
-#ifdef I18N_USE_PREEDIT
 extern void	kill_preedit();
-#endif  /* I18N_USE_PREEDIT */
-#endif  /* I18N */
-extern void text_drawing_selected (void);
+extern void	text_drawing_selected (void);
+
+#endif
