@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,8 +16,16 @@
  *
  */
 
-extern void	delete_linearrow(F_line *line, F_point *prev_point, F_point *selected_point);
-extern void	delete_arcarrow(F_arc *arc, int point_num);
-extern void	delete_splinearrow(F_spline *spline, F_point *prev_point, F_point *selected_point);
+#ifndef E_ARROW_H
+#define E_ARROW_H
 
-extern void arrow_head_selected (void);
+#include "object.h"
+
+extern void	delete_linearrow(F_line *line, F_point *prev_point,
+				F_point *selected_point);
+extern void	delete_arcarrow(F_arc *arc, int point_num);
+extern void	delete_splinearrow(F_spline *spline, F_point *prev_point,
+				F_point *selected_point);
+extern void	arrow_head_selected (void);
+
+#endif
