@@ -1,9 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,34 +17,39 @@
  *
  */
 
-#include "fig.h"
+#include "e_convert.h"
+
+#include <stdlib.h>
+
 #include "resources.h"
 #include "mode.h"
 #include "object.h"
 #include "paintop.h"
-#include "e_convert.h"
+#include "d_spline.h"
+#include "f_util.h"
 #include "u_create.h"
 #include "u_draw.h"
+#include "u_free.h"
 #include "u_list.h"
+#include "u_markers.h"
+#include "u_redraw.h"
 #include "u_search.h"
 #include "u_undo.h"
 #include "w_canvas.h"
+#include "w_cursor.h"
 #include "w_mousefun.h"
 #include "w_msgpanel.h"
-#include "d_spline.h"
 
-#include "f_util.h"
-#include "u_free.h"
-#include "u_markers.h"
-#include "u_redraw.h"
-#include "w_cursor.h"
 
-static void	init_convert_line_spline(F_line *p, int type, int x, int y, int px, int py);
-static void	init_convert_open_closed(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static void	init_convert_line_spline(F_line *p, int type, int x, int y,
+					int px, int py);
+static void	init_convert_open_closed(F_line *obj, int type, int x, int y,
+					F_point *p, F_point *q);
 
 
 
-void convert_selected(void)
+void
+convert_selected(void)
 {
     set_mousefun("spline<->line", "", "open<->closed", LOC_OBJ, LOC_OBJ, LOC_OBJ);
     canvas_kbd_proc = null_proc;
@@ -59,8 +65,12 @@ void convert_selected(void)
 }
 
 static void
-init_convert_open_closed(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
+init_convert_open_closed(F_line *obj, int type, int x, int y, F_point *p,
+			F_point *q)
 {
+	(void)x;
+	(void)y;
+
     switch (type) {
     case O_POLYLINE:
       cur_l = (F_line *) obj;
@@ -78,6 +88,8 @@ init_convert_open_closed(F_line *obj, int type, int x, int y, F_point *p, F_poin
 static void
 init_convert_line_spline(F_line *p, int type, int x, int y, int px, int py)
 {
+	(void)x; (void)y; (void)px; (void)py;
+
     static int flag = 0;
 
     switch (type) {
