@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2021 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -19,6 +19,7 @@
 #if defined HAVE_CONFIG_H && !defined VERSION
 #include "config.h"
 #endif
+#include "f_util.h"
 
 #include <errno.h>
 #include <time.h>
@@ -30,15 +31,16 @@
 #endif
 #include <sys/stat.h>
 #include <unistd.h>
-#include <X11/Xlib.h>
+#include <X11/Xlib.h>		/* includes X11/X.h */
+#include <X11/Xft/Xft.h>
 
-#include "resources.h"
+#include "resources.h"		/* PATH_MAX */
 #include "object.h"
 #include "mode.h"
 #include "f_neuclrtab.h"
 #include "f_read.h"
 #include "f_save.h"		/* write_file() */
-#include "f_util.h"
+#include "u_colors.h"
 #include "u_create.h"		/* new_string() */
 #include "u_fonts.h"		/* psfontnum() */
 #include "w_file.h"		/* renamefile() */
@@ -50,24 +52,26 @@
 #include "xfig_math.h"
 
 
+XftColor	image_cells[MAX_COLORMAP_SIZE];
+
 /* LOCALS */
 
-int	count_colors(void);
-int	count_pixels(void);
+static int	count_colors(void);
+static int	count_pixels(void);
 
 
 /* PROCEDURES */
 
 void beep (void);
-void alloc_imagecolors (int num);
-void add_all_pixels (void);
-void remap_image_colormap (void);
-void extract_cmap (void);
-void readjust_cmap (void);
-void free_pixmaps (F_compound *obj);
-void add_recent_file (char *file);
-int strain_out (char *name);
-void finish_update_xfigrc (void);
+static void alloc_imagecolors (int num);
+static void add_all_pixels (void);
+static void remap_image_colormap (void);
+static void extract_cmap (void);
+static void readjust_cmap (void);
+static void free_pixmaps (F_compound *obj);
+static void add_recent_file (char *file);
+static int strain_out (char *name);
+static void finish_update_xfigrc (void);
 
 int
 emptyname(char *name)
@@ -126,10 +130,10 @@ change_directory(char *path)
 {
     static char	cwd[PATH_MAX];
 
-    if (!strcmp(path, cwd) || path == NULL || *path == '\0')
+    if (path == NULL || *path == '\0' || !strcmp(path, cwd))
 	return 0;
     if (chdir(path) == -1) {
-	file_msg("Can't go to directory %s, : %s", path, strerror(errno));
+	file_msg("Cannot go to directory %s: %s", path, strerror(errno));
 	return 1;
     }
     strcpy(cwd, path);
@@ -340,11 +344,11 @@ void remap_imagecolors(void)
 	/* now change the color cells with the new colors */
 	/* clrtab[][] is the colormap produced by neu_clrtab */
 	for (i=0; i<avail_image_cols; i++) {
-	    image_cells[i].red   = (unsigned short) clrtab[i][N_RED] << 8;
-	    image_cells[i].green = (unsigned short) clrtab[i][N_GRN] << 8;
-	    image_cells[i].blue  = (unsigned short) clrtab[i][N_BLU] << 8;
+	    image_cells[i].color.red   = (unsigned short) clrtab[i][N_RED] << 8;
+	    image_cells[i].color.green = (unsigned short) clrtab[i][N_GRN] << 8;
+	    image_cells[i].color.blue  = (unsigned short) clrtab[i][N_BLU] << 8;
 	}
-	YStoreColors(tool_cm, image_cells, avail_image_cols);
+	alloc_or_store_colors(image_cells, avail_image_cols);
 	reset_cursor();
 
 	/* check if user pressed cancel button */
@@ -361,10 +365,7 @@ void remap_imagecolors(void)
 	scol = 0;	/* global color counter */
 	set_temp_cursor(wait_cursor);
 	extract_cmap();
-	for (i=0; i<scol; i++) {
-	    image_cells[i].flags = DoRed|DoGreen|DoBlue;
-	}
-	YStoreColors(tool_cm, image_cells, scol);
+	alloc_or_store_colors(image_cells, scol);
 	scol = 0;	/* global color counter */
 	readjust_cmap();
 	if (appres.DEBUG)
@@ -384,7 +385,6 @@ void alloc_imagecolors(int num)
     /* see if we can get all user wants */
     avail_image_cols = num;
     for (i=0; i<avail_image_cols; i++) {
-	image_cells[i].flags = DoRed|DoGreen|DoBlue;
 	if (!alloc_color_cells(&image_cells[i].pixel, 1)) {
 	    break;
 	}
@@ -470,19 +470,19 @@ extract_cmap(void)
 	struct _pics	*pics;
 	int		i;
 
-	/* extract the colormaps in the repository */
-	for (pics = pictures; pics; pics = pics->next)
-		if (pics->bitmap != NULL && pics->numcols > 0) {
-			for (i = 0; i < pics->numcols; ++i) {
-				image_cells[scol].red = pics->cmap[i].red << 8;
-				image_cells[scol].green=pics->cmap[i].green <<8;
-				image_cells[scol].blue = pics->cmap[i].blue <<8;
-				pics->cmap[i].pixel = scol;
-				++scol;
-			}
-		}
-	/* now free up the pixmaps */
-	free_pixmaps(&objects);
+    /* extract the colormaps in the repository */
+    for (pics = pictures; pics; pics = pics->next)
+	if (pics->bitmap != NULL && pics->numcols > 0) {
+	    for (i=0; i<pics->numcols; i++) {
+		image_cells[scol].color.red   = pics->cmap[i].red << 8;
+		image_cells[scol].color.green = pics->cmap[i].green << 8;
+		image_cells[scol].color.blue  = pics->cmap[i].blue << 8;
+		pics->cmap[i].pixel = scol;
+		scol++;
+	    }
+	}
+    /* now free up the pixmaps */
+    free_pixmaps(&objects);
 }
 
 void add_all_pixels(void)
@@ -683,77 +683,6 @@ safe_strcpy(char *p1, char *p2)
     return c1;
 }
 
-/* gunzip file if necessary */
-
-Boolean
-uncompress_file(char *name)
-{
-    char	    plainname[PATH_MAX];
-    char	    dirname[PATH_MAX];
-    char	    tmpfile[PATH_MAX];
-    char	    unc[PATH_MAX+20];	/* temp buffer for uncompress/gunzip command */
-    char	   *c;
-    struct stat	    status;
-
-    strcpy(tmpfile, name);		/* save original name */
-    strcpy(plainname, name);
-    c = strrchr(plainname, '.');
-    if (c) {
-      if (strcmp(c, ".gz") == 0 || strcmp(c, ".Z") == 0 || strcmp(c, ".z") == 0)
-	*c = '\0';
-    }
-
-    if (stat(name, &status)) {    /* first see if file exists AS NAMED */
-      /* no, try without .gz etc suffix */
-      strcpy(name, plainname);
-      if (stat(name, &status)) {
-	/* no, try with a .z */
-	sprintf(name, "%s.z", plainname);
-	if (stat(name, &status)) {
-	  /* no, try with .Z suffix */
-	  sprintf(name, "%s.Z", plainname);
-	  if (stat(name, &status)) {
-	    /* no, try .gz */
-	    sprintf(name, "%s.gz", plainname);
-	    if (stat(name, &status)) {
-	      /* none of the above, return original name and False status */
-	      strcpy(name, tmpfile);
-	      return False;
-	    }
-	  }
-	}
-      }
-    }
-    /* file doesn't have .gz etc suffix anymore, return modified name */
-    if (strcmp(name, plainname) == 0) return True;
-
-    strcpy(dirname, name);
-    c = strrchr(dirname, '/');
-    if (c) *c = '\0';
-    else strcpy(dirname, ".");
-
-    if (access(dirname, W_OK) == 0) {  /* OK - the directory is writable */
-      sprintf(unc, "gunzip -q %s", name);
-      if (system(unc) != 0)
-	file_msg("Couldn't uncompress the file: \"%s\"", unc);
-      strcpy(name, plainname);
-    } else {  /* the directory is not writable */
-      /* create a path to TMPDIR in case we need to uncompress a read-only file to there */
-      c = strrchr(plainname, '/');
-      if (c)
-	  sprintf(tmpfile, "%s%s", TMPDIR, c);
-      else
-	  sprintf(tmpfile, "%s/%s", TMPDIR, plainname);
-      sprintf(unc, "gunzip -q -c %s > %s", name, tmpfile);
-      if (system(unc) != 0)
-	  file_msg("Couldn't uncompress the file: \"%s\"", unc);
-      file_msg ("Uncompressing file %s in %s because it is in a read-only directory",
-		name, TMPDIR);
-      strcpy(name, tmpfile);
-    }
-    return True;
-}
-
 /*************************************************************/
 /* Read the user's .xfigrc file in his home directory to get */
 /* settings from previous session.                           */
@@ -810,6 +739,8 @@ void add_recent_file(char *file)
 	return;
     name = new_string(strlen(file)+3);	/* allow for file number (1), blank (1) and NUL */
     sprintf(name,"%1d %s",num_recent_files+1,file);
+    if (recent_files[num_recent_files].name)
+	    free(recent_files[num_recent_files].name);
     recent_files[num_recent_files].name = name;
     num_recent_files++;
 }
@@ -857,8 +788,9 @@ void update_xfigrc(char *name, char *string)
 
 int strain_out(char *name)
 {
-    char    line[RC_BUFSIZ+1], *tok;
-    int fd;
+	char	line[RC_BUFSIZ+1];
+	int	fd;
+	size_t	len;
 
     /* make a temp filename in the user's home directory so we
        can just rename it to .xfigrc after creating it */
@@ -888,16 +820,17 @@ int strain_out(char *name)
 	xfigrc = (FILE *) 0;
 	return 0;
     }
+
+    len = strlen(name);
     while (fgets(line, RC_BUFSIZ, xfigrc) != NULL) {
+	char	*c;
 	/* look for sane input */
 	if (line[0] == '\0')
 	    break;
-	/* make copy of line to look for token because strtok modifies the line */
-	tok = strdup(line);
-	if (strcasecmp(strtok(tok, ": \t"), name) == 0)
+	if ((c = strchr(line, ':')) && (size_t)(c - line) == len &&
+			!strncmp(name, line, len))
 	    continue;			/* match, skip */
 	fputs(line, tmpf);
-	free(tok);
     }
     return 0;
 }
@@ -1072,9 +1005,7 @@ update_fig_files(int argc, char **argv)
 	appres.magnification = 100.0f;
 	appres.multiple = False;
 	appres.transparent = -2;
-	/* and use scalable fonts, if available */
-	appres.scalablefonts = True;
-	/* but do not set correct_font_size; Originally, font sizes were given
+	/* Do not set correct_font_size. Originally, font sizes were given
 	   in pixel, and xfig displayed with 80 pixels to the inch. */
 
 	/* read Fig file but don't import any images */
@@ -1098,9 +1029,9 @@ update_fig_files(int argc, char **argv)
 	    /* copy user colors */
 	    for (col=0; col<MAX_USR_COLS; col++) {
 		colorUsed[col] = !n_colorFree[col];
-		user_colors[col].red = n_user_colors[col].red;
-		user_colors[col].green = n_user_colors[col].green;
-		user_colors[col].blue = n_user_colors[col].blue;
+		user_color[col].color.red = n_user_colors[col].color.red;
+		user_color[col].color.green = n_user_colors[col].color.green;
+		user_color[col].color.blue = n_user_colors[col].color.blue;
 	    }
 	    /* now write out the new one */
 	    num_usr_cols = MAX_USR_COLS;
