@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,6 +16,9 @@
  *
  */
 
+#include <X11/Intrinsic.h>	/* Boolean */
+#include <X11/X.h>		/* Atom */
+
 extern Atom	wm_protocols[];
 extern Boolean	geomspec;
 extern void	toggle_refresh_mode(void);
