@@ -1,35 +1,40 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2017 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and documentation
  * files (the "Software"), including without limitation the rights to use,
- * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies of
  * the Software, and to permit persons who receive copies from any such
  * party to do so, with the only requirement being that the above copyright
  * and this permission notice remain intact.
  *
  */
 
-#include "fig.h"
+#include "u_bound.h"
+
+#include <limits.h>		/* INT_MIN, INT_MAX */
+#include <math.h>
+#include <stdlib.h>
+
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
 #include "paintop.h"
-#include "u_bound.h"
+#include "u_colors.h"
+#include "u_draw.h"
+#include "u_fonts.h"
 #include "w_canvas.h"		/* round_coords() */
 #include "w_drawprim.h"
 #include "w_file.h"
 #include "w_layers.h"
-#include "w_setup.h"
-#include "w_zoom.h"
+#include "xfig_math.h"
 
-#include "u_draw.h"
 
 #define		Ninety_deg		M_PI_2
 #define		One_eighty_deg		M_PI
@@ -276,11 +281,9 @@ void active_compound_bound(F_compound *compound, int *xmin, int *ymin, int *xmax
     }
 
     for (t = compound->texts; t != NULL; t = t->next) {
-	int    dum;
 	if (active_only && !active_layer(t->depth))
 	    continue;
-	text_bound(t, &sx, &sy, &bx, &by,
-		  &dum,&dum,&dum,&dum,&dum,&dum,&dum,&dum);
+	text_bound(t, &sx, &sy, &bx, &by);
 	if (first) {
 	    first = 0;
 	    llx = sx;
@@ -296,10 +299,8 @@ void active_compound_bound(F_compound *compound, int *xmin, int *ymin, int *xmax
     }
 
     /* round the corners to the current positioning grid */
-    llx = floor_coords_x(llx, lly);
-    lly = floor_coords_y(lly, lly);
-    urx = ceil_coords_x(urx, ury);
-    ury = ceil_coords_y(ury, ury);
+    floor_coords(&llx, &lly);
+    ceil_coords(&urx, &ury);
     *xmin = llx;
     *ymin = lly;
     *xmax = urx;
@@ -518,59 +519,14 @@ approx_spline_bound(F_spline *s, int *xmin, int *ymin, int *xmax, int *ymax)
       }
 }
 
-/* This procedure calculates the bounding box for text.  It returns
-   the min/max x and y coords of the enclosing HORIZONTAL rectangle.
-   The actual corners of the rectangle are returned in (rx1,ry1)...(rx4,ry4)
- */
-
-void text_bound(F_text *t, int *xmin, int *ymin, int *xmax, int *ymax, int *rx1, int *ry1, int *rx2, int *ry2, int *rx3, int *ry3, int *rx4, int *ry4)
+/* Compute the bounding box for text */
+void
+text_bound(F_text *t, int *xmin, int *ymin, int *xmax, int *ymax)
 {
-    int		    h, l;
-    int		    x1,y1, x2,y2, x3,y3, x4,y4;
-    double	    cost, sint;
-    double	    dcost, dsint, lcost, lsint, hcost, hsint;
-
-    cost = cos((double)t->angle);
-    sint = sin((double)t->angle);
-    l = text_length(t);
-    h = t->ascent+t->descent;
-    lcost = round(l*cost);
-    lsint = round(l*sint);
-    hcost = round(h*cost);
-    hsint = round(h*sint);
-    dcost = round(t->descent*cost);
-    dsint = round(t->descent*sint);
-    x1 = t->base_x+dsint;
-    y1 = t->base_y+dcost;
-    if (t->type == T_CENTER_JUSTIFIED) {
-	x1 = t->base_x+dsint - round((l/2)*cost);
-	y1 = t->base_y+dcost + round((l/2)*sint);
-	x2 = x1 + lcost;
-	y2 = y1 - lsint;
-    }
-    else if (t->type == T_RIGHT_JUSTIFIED) {
-	x1 = t->base_x+dsint - lcost;
-	y1 = t->base_y+dcost + lsint;
-	x2 = t->base_x+dsint;
-	y2 = t->base_y+dcost;
-    }
-    else {
-	x2 = x1 + lcost;
-	y2 = y1 - lsint;
-    }
-    x4 = x1 - hsint;
-    y4 = y1 - hcost;
-    x3 = x2 - hsint;
-    y3 = y2 - hcost;
-
-    *xmin = min2(x1,min2(x2,min2(x3,x4)));
-    *xmax = max2(x1,max2(x2,max2(x3,x4)));
-    *ymin = min2(y1,min2(y2,min2(y3,y4)));
-    *ymax = max2(y1,max2(y2,max2(y3,y4)));
-    *rx1=x1; *ry1=y1;
-    *rx2=x2; *ry2=y2;
-    *rx3=x3; *ry3=y3;
-    *rx4=x4; *ry4=y4;
+	*xmin = t->bb[0].x;
+	*ymin = t->bb[0].y;
+	*xmax = t->bb[1].x;
+	*ymax = t->bb[1].y;
 }
 
 static void
@@ -695,112 +651,3 @@ void arrow_bound(int objtype, F_line *obj, int *xmin, int *ymin, int *xmax, int
 	}
     }
 }
-
-/* rounds DOWN the coordinates depending on point positioning mode */		// isometric grid
-
-int
-floor_coords_x( x, y )
-int x, y;
-{
-    int x_old = x, y_old = y;
-    int i = 0;
-
-    /* no grid */
-    if( cur_pointposn == P_ANY || x == INT_MIN ) return x;
-
-    /* smaller than zero inverts problem */
-    if( x < 0 ) return -ceil_coords_x( -x, y );
-
-    /* adjust x coordinate to be on grid */
-    do {
-	x = x_old - i++;
-	y = y_old;
-	round_coords( &x, &y );
-    } while( x > x_old );
-
-    /* assign result */
-    return x;
-}
-
-int
-floor_coords_y( x, y )
-int x, y;
-{
-    int x_old = x, y_old = y;
-    int i = 0;
-
-    /* no grid */
-    if( cur_pointposn == P_ANY || y == INT_MIN ) return y;
-
-    /* smaller than zero inverts problem */
-    if( y < 0 ) return -ceil_coords_y( x, -y );
-
-    /* adjust y coordinate to be on grid */
-    do {
-	x = x_old;
-	y = y_old - i++;
-	round_coords( &x, &y );
-    } while( y > y_old );
-
-    /* assign result */
-    return y;
-}
-
-
-/* rounds UP the coordinates depending on point positioning mode */
-
-int
-ceil_coords_x( x, y )
-int x, y;
-{
-    int x_old = x, y_old = y;
-    int i = 0;
-
-    /* no grid */
-    if( cur_pointposn == P_ANY ) return x;
-
-    /* smaller than zero inverts problem */
-    if( x < 0 ) {
-	if (x == INT_MIN)
-		x = -INT_MAX;
-	return -floor_coords_x( -x, y );
-    }
-
-    /* adjust x coordinate to be on grid */
-    do {
-	x = x_old + i++;
-	y = y_old;
-	round_coords( &x, &y );
-    } while( x < x_old && x < INT_MAX);
-
-    /* assign result */
-    return x;
-}
-
-int
-ceil_coords_y( x, y )
-int x, y;
-{
-    int x_old = x, y_old = y;
-    int i = 0;
-
-    /* no grid */
-    if( cur_pointposn == P_ANY ) return y;
-
-    /* smaller than zero inverts problem */
-    if( y < 0 ) {
-	if (y == INT_MIN)
-		y = -INT_MAX;
-	return -floor_coords_y( x, -y );
-    }
-
-    /* adjust y coordinate to be on grid */
-    do {
-	x = x_old;
-	y = y_old + i++;
-	round_coords( &x, &y );
-    } while( y < y_old );
-
-    /* assign result */
-    return y;
-}
