@@ -1,8 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
+ *
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
@@ -16,29 +18,36 @@
  *
  */
 
-#include "fig.h"
+
+#include "u_create.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <X11/Xlib.h>		/* includes X11/X.h */
+
 #include "resources.h"
 #include "mode.h"
 #include "object.h"
-#include "e_edit.h"
-#include "u_create.h"
-#include "w_indpanel.h"
-#include "w_layers.h"
-#include "w_msgpanel.h"
-#include "w_setup.h"
-#include "w_util.h"
+#include "paintop.h"
 
+#include "e_edit.h"
 #include "e_scale.h"
+#include "u_colors.h"
+#include "u_fonts.h"
 #include "u_free.h"
 #include "u_list.h"
 #include "w_cursor.h"
 #include "w_modepanel.h"
 #include "w_mousefun.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_util.h"
 
 static char	Err_mem[] = "Running out of memory.";
 
-/****************** ARROWS ****************/
 
+/****************** ARROWS ****************/
 
 
 F_arrow *
@@ -359,7 +368,7 @@ copy_arc(F_arc *a)
 
     if (a->for_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) arc);
+	    free_arc(&arc);
 	    return NULL;
 	}
 	arc->for_arrow = arrow;
@@ -367,7 +376,7 @@ copy_arc(F_arc *a)
     }
     if (a->back_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) arc);
+	    free_arc(&arc);
 	    return NULL;
 	}
 	arc->back_arrow = arrow;
@@ -457,14 +466,14 @@ create_picture_entry(void)
 
     picture = malloc(sizeof(struct _pics));
 
-    picture->file = picture->realname = NULL;
+    picture->file = NULL;
     picture->bitmap = NULL;
     picture->transp = TRANSP_NONE;
     picture->numcols = 0;
     picture->refcount = 0;
     picture->prev = picture->next = NULL;
     if (appres.DEBUG)
-	fprintf(stderr,"create picture entry %p\n", picture);
+	fprintf(stderr, "create picture entry %p\n", (void *)picture);
     return picture;
 }
 
@@ -488,7 +497,7 @@ copy_line(F_line *l)
 
     if (l->for_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) line);
+	    free_linestorage(line);
 	    return NULL;
 	}
 	line->for_arrow = arrow;
@@ -496,7 +505,7 @@ copy_line(F_line *l)
     }
     if (l->back_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) line);
+	    free_linestorage(line);
 	    return NULL;
 	}
 	line->back_arrow = arrow;
@@ -511,11 +520,11 @@ copy_line(F_line *l)
     /* copy picture information */
     if (l->pic) {
 	if ((line->pic = create_pic()) == NULL) {
-	    free((char *) line);
+	    free_linestorage(line);
 	    return NULL;
 	}
 	/* copy all the numbers and the pointer to the picture repository (pic->pic_cache) */
-	bcopy(l->pic, line->pic, PIC_SIZE);
+	memcpy(line->pic, l->pic, PIC_SIZE);
 	/* increase reference count for this picture */
 	if (line->pic->pic_cache)
 	    line->pic->pic_cache->refcount++;
@@ -577,7 +586,7 @@ copy_spline(F_spline *s)
 
     if (s->for_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) spline);
+	    free_splinestorage(spline);
 	    return NULL;
 	}
 	spline->for_arrow = arrow;
@@ -585,7 +594,7 @@ copy_spline(F_spline *s)
     }
     if (s->back_arrow) {
 	if ((arrow = create_arrow()) == NULL) {
-	    free((char *) spline);
+	    free_splinestorage(spline);
 	    return NULL;
 	}
 	spline->back_arrow = arrow;
@@ -622,7 +631,7 @@ create_text(void)
 	return NULL;
     }
     t->tagged = 0;
-    t->fontstruct = 0;
+    t->xf_font = NULL;
     t->comments = NULL;
     t->cstring = NULL;
     t->next = NULL;
@@ -652,12 +661,14 @@ copy_text(F_text *t)
     /* copy static items first */
     *text = *t;
     text->next = NULL;
+    if (t->xf_font)
+	   text->xf_font = font_xfcopy(t->xf_font);
 
     /* do comments next */
     copy_comments(&t->comments, &text->comments);
 
     if ((text->cstring = new_string(strlen(t->cstring))) == NULL) {
-	free((char *) text);
+	free_text(&text);
 	return NULL;
     }
     strcpy(text->cstring, t->cstring);
@@ -783,13 +794,15 @@ copy_compound(F_compound *c)
 F_compound*
 create_dimension_line(F_line *line, Boolean add_to_figure)
 {
+    int            i;
     F_compound	   *comp;
     F_line	   *box, *tick1, *tick2;
     F_text	   *text;
     F_point	   *pnt;
 
     /* make a new compound */
-    comp = create_compound();
+    if ((comp = create_compound()) == NULL)
+	return NULL;
     /* if fixed text, put in an unchanging comment for the compound */
     /* (rescale_dimension_line will create the comment if the text not fixed (=length)) */
     if (cur_dimline_fixed) {
@@ -827,7 +840,10 @@ create_dimension_line(F_line *line, Boolean add_to_figure)
     /* make the text object for the length */
     /***************************************/
 
-    text = create_text();
+    if ((text = create_text()) == NULL) {
+	free_compound(&comp);
+	return NULL;
+    }
     text->depth  = line->depth-2;
     text->cstring = (char *) NULL;	/* the string will be put in later */
     text->color = cur_dimline_textcolor;
@@ -844,7 +860,10 @@ create_dimension_line(F_line *line, Boolean add_to_figure)
     /* make the box for the text */
     /*****************************/
 
-    box = create_line();
+    if ((box = create_line()) == NULL) {
+	free_compound(&comp);
+	return NULL;
+    }
     box->comments = strdup("text box");
     box->depth = line->depth-1;
     box->type = T_POLYGON;
@@ -859,15 +878,18 @@ create_dimension_line(F_line *line, Boolean add_to_figure)
     box->cap_style = CAP_BUTT;
 
     /* make 5 points for the box */
-    pnt = create_point();
+    if ((pnt = create_point()) == NULL) {
+	free_compound(&comp);
+	return NULL;
+    }
     box->points = pnt;
-    pnt->next = create_point();
-    pnt = pnt->next;
-    pnt->next = create_point();
-    pnt = pnt->next;
-    pnt->next = create_point();
-    pnt = pnt->next;
-    pnt->next = create_point();
+    for (i = 0; i < 4; ++i) {
+	if ((pnt->next = create_point()) == NULL) {
+		free_compound(&comp);
+		return NULL;
+	}
+	pnt = pnt->next;
+    }
 
     /* add this to the lines in the compound */
     comp->lines->next = box;
@@ -925,52 +947,52 @@ create_dimension_line(F_line *line, Boolean add_to_figure)
     return comp;
 }
 
+
+F_line *
+create_tick(F_line *line)
+{
+	F_line	   *tick;
+
+	if ((tick = create_line()) == NULL)
+		return tick;
+	/* copy the attributes from the main line */
+	*tick = *line;
+	/* set thickness */
+	tick->thickness = cur_dimline_tickthick;
+	/* make solid */
+	tick->style = SOLID_LINE;
+	tick->comments = strdup("tick");
+	/* zero the arrows and next pointer */
+	tick->for_arrow = tick->back_arrow = NULL;
+	tick->next = NULL;
+	if ((tick->points = create_point()) == NULL) {
+		free_linestorage(tick);
+		return NULL;
+	}
+	if ((tick->points->next = create_point()) == NULL) {
+		free_linestorage(tick);
+		return NULL;
+	}
+
+	return tick;
+}
+
 /*
  * make the two ticks for the endpoints
  * the actual values of the ticks' points are computed in rescale_dimension_line()
+ * On error, return NULL in *tick1.
  */
 
 void
 create_dimline_ticks(F_line *line, F_line **tick1, F_line **tick2)
 {
-	F_point	   *pnt;
-	F_line	   *tick;
+	if ((*tick1 = create_tick(line)) == NULL)
+		return;
 
-	/* first tick */
-
-	tick = create_line();
-	/* copy the attributes from the main line */
-	*tick = *line;
-	/* set thickness */
-	tick->thickness = cur_dimline_tickthick;
-	/* make solid */
-	tick->style = SOLID_LINE;
-	tick->comments = strdup("tick");
-	/* zero the arrows and next pointer */
-	tick->for_arrow = tick->back_arrow = (F_arrow *) NULL;
-	tick->next = (F_line *) NULL;
-	pnt = create_point();
-	tick->points = pnt;
-	pnt->next = create_point();
-	*tick1 = tick;
-
-	/* now the other tick */
-
-	tick = create_line();
-	/* copy the attributes from the main line */
-	*tick = *line;
-	/* set thickness */
-	tick->thickness = cur_dimline_tickthick;
-	/* make solid */
-	tick->style = SOLID_LINE;
-	tick->comments = strdup("tick");
-	/* zero the arrows and next pointer */
-	tick->for_arrow = tick->back_arrow = (F_arrow *) NULL;
-	tick->next = (F_line *) NULL;
-	pnt = create_point();
-	tick->points = pnt;
-	pnt->next = create_point();
-	*tick2 = tick;
+	if ((*tick2 = create_tick(line)) == NULL) {
+		free_linestorage(*tick1);
+		*tick1 = NULL;
+	}
 }
 
 F_arrow*
