@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,9 +16,16 @@
  *
  */
 
+#ifndef U_DRAG_H
+#define U_DRAG_H
+
+#include "object.h"
+
 extern void	init_ellipsedragging(F_ellipse *e, int x, int y);
 extern void	init_arcdragging(F_arc *a, int x, int y);
 extern void	init_linedragging(F_line *l, int x, int y);
 extern void	init_textdragging(F_text *t, int x, int y);
 extern void	init_splinedragging(F_spline *s, int x, int y);
 extern void	init_compounddragging(F_compound *c, int x, int y);
+
+#endif
