@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2020 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Parts Copyright (c) 1992 by James Tough
  * Parts Copyright (c) 1998 by Georg Stemmer
@@ -20,151 +20,179 @@
  *
  */
 
+#include "u_draw.h"
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
-#include <X11/Intrinsic.h> /* includes X11/Xlib.h */	/* Boolean */
+#include <X11/Intrinsic.h> /* includes X11/Xlib.h */	/* Boolean, True */
 #include <X11/ImUtil.h>	/* must first include X11/Xlib.h */
 			/* _XInitImageFuncPtrs() */
 
 #include "resources.h"
-#include "mode.h"
 #include "object.h"
-#include "d_text.h"		/* reload_text_fstruct() */
+#include "paintop.h"
 #include "f_util.h"		/* xf_basename() */
+#include "f_picobj.h"		/* ABSOLUTE_PATHNAME */
 #include "u_bound.h"		/* <obj>_bound(), overlapping() */
-#include "u_draw.h"
-#include "u_geom.h"		/* compute_angle() */
+#include "u_convert.h"
 #include "u_error.h"		/* X_error_handler() */
+#include "u_fonts.h"
+#include "u_geom.h"		/* compute_angle() */
+#include "u_redraw.h"		/* redisplay_line() */
 #include "w_canvas.h"		/* clip_xmax, clip_xmin */
+#include "w_cursor.h"		/* reset_cursor() */
 #include "w_file.h"		/* check_cancel() */
 #include "w_layers.h"		/* active_layer() */
 #include "w_msgpanel.h"		/* put_msg() */
 #include "w_util.h"		/* NUM_ARROW_TYPES */
-#include "u_redraw.h"		/* redisplay_line() */
-#include "w_cursor.h"		/* reset_cursor() */
+#include "w_zoom.h"
 #include "xfig_math.h"
 
 static Boolean add_point(int x, int y);
 static void init_point_array(void);
 static Boolean add_closepoint(void);
 
+#define PICFILE_FONT	FONT_COURIER	/* defined in u_fonts.h */
+#define PICFONT_SIZE	12.0
+
 /* the spline definition stuff has been moved to u_draw_spline.c */
 #include "u_draw_spline.c"
 
 /************** ARRAY FOR ARROW SHAPES **************/
 
 struct _fpnt {
-		double x,y;
-	};
+	double x,y;
+};
 
 struct _arrow_shape {
-		int	numpts;		/* number of points in arrowhead */
-		int	tipno;		/* which point contains the tip */
-		int	numfillpts;	/* number of points to fill */
-		Boolean	simplefill;	/* if true, use points array to fill otherwise use fill_points array */
-		Boolean	clip;		/* if false, no clip area needed (e.g. for reverse triangle arrowhead) */
-		Boolean	half;		/* if true, arrowhead is half-wide and must be shifted to cover the line */
-		double	tipmv;		/* acuteness of tip (smaller angle, larger tipmv) */
-		struct	_fpnt points[6]; /* points in arrowhead */
-		struct	_fpnt fillpoints[6]; /* points to fill if not "simple" */
-	};
+	int	numpts;		/* number of points in arrowhead */
+	int	tipno;		/* which point contains the tip */
+	int	numfillpts;	/* number of points to fill */
+	Boolean	simplefill;	/* if true, use points array to fill otherwise
+				   use fill_points array */
+	Boolean	clip;		/* if false, no clip area needed
+				   (e.g. for reverse triangle arrowhead) */
+	Boolean	half;		/* if true, arrowhead is half-wide and must be
+				   shifted to cover the line */
+	double	tipmv;		/* acuteness of tip (smaller angle,
+						larger tipmv) */
+	struct	_fpnt points[6]; /* points in arrowhead */
+	struct	_fpnt fillpoints[6]; /* points to fill if not "simple" */
+};
 
 static struct _arrow_shape arrow_shapes[NUM_ARROW_TYPES] = {
 	/* number of points, index of tip, {datapairs} */
 	/* first point must be upper-left point of tail, then tip */
-
 	/* type 0 */
-	{ 3, 1, 0, True, True, False, 2.15, {{-1,0.5}, {0,0}, {-1,-0.5}}, {0}},
+	{ 3, 1, 0, True, True, False, 2.15,
+		{{-1,0.5}, {0.,0.}, {-1,-0.5}},
+		{{0.,0.}}},
 	/* place holder for what would be type 0 filled */
 	{ 0 },
 	/* type 1a simple triangle */
 	{ 4, 1, 0, True, True, False, 2.1,
-			{{-1.0,0.5}, {0.,0.}, {-1.0,-0.5}, {-1.0,0.5}}, {0}},
+		{{-1.0,0.5}, {0.,0.}, {-1.,-0.5}, {-1.0,0.5}},
+		{{0.,0.}}},
 	/* type 1b filled simple triangle*/
 	{ 4, 1, 0, True, True, False, 2.1,
-			{{-1.0,0.5}, {0.,0.}, {-1.0,-0.5}, {-1.0,0.5}}, {0}},
+		{{-1.0,0.5}, {0.,0.}, {-1.0,-0.5}, {-1.,0.5}},
+		{{0.,0.}}},
 	/* type 2a concave spearhead */
 	{ 5, 1, 0, True, True, False, 2.6,
-			{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
-			{0}},
+		{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
+		{{0.,0.}}},
 	/* type 2b filled concave spearhead */
 	{ 5, 1, 0, True, True, False, 2.6,
-			{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
-			{0}},
+		{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
+		{{0.,0.}}},
 	/* type 3a convex spearhead */
 	{ 5, 1, 0, True, True, False, 1.5,
-			{{-0.75,0.5},{0.,0.},{-0.75,-0.5},{-1.,0.},{-0.75,0.5}},
-			{0}},
+		{{-0.75,0.5},{0.,0.},{-0.75,-0.5},{-1.,0.},{-0.75,0.5}},
+		{{0.,0.}}},
 	/* type 3b filled convex spearhead */
 	{ 5, 1, 0, True, True, False, 1.5,
-			{{-0.75,0.5},{0.,0.},{-0.75,-0.5},{-1.,0.},{-0.75,0.5}},
-			{0}},
+		{{-0.75,0.5},{0.,0.},{-0.75,-0.5},{-1.,0.},{-0.75,0.5}},
+		{{0.,0.}}},
 	/* type 4a diamond */
 	{ 5, 1, 0, True, True, False, 1.15,
-			{{-0.5,0.5},{0.,0.},{-0.5,-0.5},{-1.0,0.},{-0.5,0.5}},
-			{0}},
+		{{-0.5,0.5},{0.,0.},{-0.5,-0.5},{-1.0,0.},{-0.5,0.5}},
+		{{0.,0.}}},
 	/* type 4b filled diamond */
 	{ 5, 1, 0, True, True, False, 1.15,
-			{{-0.5,0.5},{0.,0.},{-0.5,-0.5},{-1.0,0.},{-0.5,0.5}},
-			{0}},
+		{{-0.5,0.5},{0.,0.},{-0.5,-0.5},{-1.0,0.},{-0.5,0.5}},
+		{{0.,0.}}},
 	/* type 5a/b circle - handled in code */
-	{ 0, 0, 0, True, True, False, 0.0, {0}, {0}},
-	{ 0, 0, 0, True, True, False, 0.0, {0}, {0}},
+	{ 0, 0, 0, True, True, False, 0.0, {{0.,0.}}, {{0.,0.}}},
+	{ 0, 0, 0, True, True, False, 0.0, {{0.,0.}}, {{0.,0.}}},
 	/* type 6a/b half circle - handled in code */
-	{ 0, 0, 0, True, True, False, -1.0, {0}, {0}},
-	{ 0, 0, 0, True, True, False, -1.0, {0}, {0}},
+	{ 0, 0, 0, True, True, False, -1.0, {{0.,0.}}, {{0.,0.}}},
+	{ 0, 0, 0, True, True, False, -1.0, {{0.,0.}}, {{0.,0.}}},
 	/* type 7a square */
 	{ 5, 1, 0, True, True, False, 0.0,
-			{{-1.0,0.5},{0.,0.5},{0.,-0.5},{-1.0,-0.5},{-1.0,0.5}},
-			{0}},
+		{{-1.0,0.5},{0.,0.5},{0.,-0.5},{-1.0,-0.5},{-1.0,0.5}},
+		{{0.,0.}}},
 	/* type 7b filled square */
 	{ 5, 1, 0, True, True, False, 0.0,
-			{{-1.0,0.5},{0.,0.5},{0.,-0.5},{-1.0,-0.5},{-1.0,0.5}},
-			{0}},
+		{{-1.0,0.5},{0.,0.5},{0.,-0.5},{-1.0,-0.5},{-1.0,0.5}},
+		{{0.,0.}}},
 	/* type 8a reverse triangle */
 	{ 4, 1, 0, True, False, False, 0.0,
-			{{-1.0,0.},{0.,0.5},{0.,-0.5},{-1.0,0}}, {0}},
+		{{-1.0,0.}, {0.,0.5}, {0.,-0.5}, {-1.0,0}},
+		{{0.,0.}}},
 	/* type 8b filled reverse triangle */
 	{ 4, 1, 0, True, False, False, 0.0,
-			{{-1.0,0.},{0.,0.5},{0.,-0.5},{-1.0,0.}}, {0}},
+		{{-1.0,0.}, {0.,0.5}, {0.,-0.5}, {-1.0,0.}},
+		{{0.,0.}}},
 	/* type 9a top-half filled concave spearhead */
 	{ 5, 1, 3, False, True, False, 2.6,
-			{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
-			{{-1.25,-0.5},{0.,0.},{-1.,0.}}},
+		{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
+		{{-1.25,-0.5},{0.,0.},{-1.,0.}}},
 	/* type 9b bottom-half filled concave spearhead */
 	{ 5, 1, 3, False, True, False, 2.6,
-			{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
-			{{-1.25,0.5},{0.,0.},{-1,0}}},
+		{{-1.25,0.5},{0.,0.},{-1.25,-0.5},{-1.,0.},{-1.25,0.5}},
+		{{-1.25,0.5},{0.,0.},{-1,0}}},
 	/* type 10o top-half simple triangle */
 	{ 4, 1, 0, True, True, True, 2.5,
-			{{-1.0,0.5}, {0.,0.}, {-1.,0.0}, {-1.0,0.5}}, {0}},
+		{{-1.0,0.5}, {0.,0.}, {-1.,0.0}, {-1.0,0.5}},
+		{{0.,0.}}},
 	/* type 10f top-half filled simple triangle*/
 	{ 4, 1, 0, True, True, True, 2.5,
-			{{-1.0,0.5}, {0.,0.}, {-1.,0.0}, {-1.0,0.5}}, {0}},
+		{{-1.0,0.5}, {0.,0.}, {-1.,0.0}, {-1.0,0.5}},
+		{{0.,0.}}},
 	/* type 11o top-half concave spearhead */
 	{ 4, 1, 0, True, True, True, 3.5,
-			{{-1.25,0.5}, {0.,0.}, {-1.,0.}, {-1.25,0.5}}, {0}},
+		{{-1.25,0.5}, {0.,0.}, {-1.,0.}, {-1.25,0.5}},
+		{{0.,0.}}},
 	/* type 11f top-half filled concave spearhead */
 	{ 4, 1, 0, True, True, True, 3.5,
-			{{-1.25,0.5}, {0.,0.}, {-1.,0.}, {-1.25,0.5}}, {0}},
+		{{-1.25,0.5}, {0.,0.}, {-1.,0.}, {-1.25,0.5}},
+		{{0.,0.}}},
 	/* type 12o top-half convex spearhead */
 	{ 4, 1, 0, True, True, True, 2.5,
-			{{-0.75,0.5}, {0.,0.}, {-1.,0.}, {-0.75,0.5}}, {0}},
+		{{-0.75,0.5}, {0.,0.}, {-1.,0.}, {-0.75,0.5}},
+		{{0.,0.}}},
 	/* type 12f top-half filled convex spearhead */
 	{ 4, 1, 0, True, True, True, 2.5,
-			{{-0.75,0.5}, {0.,0.}, {-1.,0.}, {-0.75,0.5}}, {0}},
+		{{-0.75,0.5}, {0.,0.}, {-1.,0.}, {-0.75,0.5}},
+		{{0.,0.}}},
 	/* type 13a "wye" */
 	{ 3, 0, 0, True, True, False, -1.0,
-			{{0,0.5},{-1.0,0.},{0.,-0.5}}, {0}},
+		{{0,0.5},{-1.0,0.},{0.,-0.5}},
+		{{0.,0.}}},
 	/* type 13b bar */
-	{ 2, 1, 0, True, True, False, 0.0, {{0.,0.5},{0.,-0.5}}, {0}},
+	{ 2, 1, 0, True, True, False, 0.0,
+		{{0.,0.5},{0.,-0.5}},
+		{{0.,0.}}},
 	/* type 14a two-prong fork */
 	{ 4, 0, 0, True, True, False, -1.0,
-			{{0,0.5},{-1.0,0.5},{-1.0,-0.5},{0.,-0.5}}, {0}},
+		{{0,0.5},{-1.0,0.5},{-1.0,-0.5},{0.,-0.5}},
+		{{0.,0.}}},
 	/* type 14b backward two-prong fork */
 	{ 4, 1, 0, True, True, False, 0.0,
-			{{-1.0,0.5,},{0.,0.5},{0.,-0.5},{-1.0,-0.5}}, {0}},
+		{{-1.0,0.5,},{0.,0.5},{0.,-0.5},{-1.0,-0.5}},
+		{{0.,0.}}},
 };
 
 /************** POLYGON/CURVE DRAWING FACILITIES ****************/
@@ -274,22 +302,25 @@ void draw_arc(F_arc *a, int op)
 	scx = ZOOMX(cx);
 	scy = ZOOMY(cy);
 	if (scx < clip_xmin+10 || scx > clip_xmax-10 ||
-	    scy < clip_ymin+10 || scy > clip_ymax-10)
+	    scy < clip_ymin+10 || scy > clip_ymax-10) {
+		xmin = clip_xmin;  xmax = clip_xmax;
+		ymin = clip_ymin;  ymax = clip_ymax;
 		set_clip_window(min2(scx-10,clip_xmin), min2(scy-10,clip_ymin),
 				max2(scx+10,clip_xmax), max2(scy+10,clip_ymax));
+	}
 	sprintf(bufx,"c");
 	pw_text(canvas_win, cx, round(cy-3.0/zoomscale),
-		op, a->depth, roman_font, 0.0, bufx, RED, COLOR_NONE);
+		op, a->depth, roman_font, bufx, RED, COLOR_NONE);
 	pw_point(canvas_win, cx, cy, op, a->depth, 4, RED, CAP_ROUND);
 	for (i=1; i<=3; i++) {
 	    /* label the point number above the point */
 	    sprintf(bufx,"%d",i);
 	    pw_text(canvas_win, a->point[i-1].x, round(a->point[i-1].y-3.0/zoomscale),
-		op, a->depth, roman_font, 0.0, bufx, RED, COLOR_NONE);
+		op, a->depth, roman_font, bufx, RED, COLOR_NONE);
 	    pw_point(canvas_win, a->point[i-1].x, a->point[i-1].y, op, a->depth, 4, RED, CAP_ROUND);
 	}
 	/* restore original clip window */
-	set_clip_window(clip_xmin, clip_ymin, clip_xmax, clip_ymax);
+	set_clip_window(xmin, ymin, xmax, ymax);
     }
     /* fill points array but don't display the points yet */
 
@@ -612,9 +643,7 @@ void draw_line(F_line *line, int op)
     F_point	   *point;
     int		    i, x, y;
     int		    xmin, ymin, xmax, ymax;
-    char	   *string;
     F_point	   *p0, *p1, *p2;
-    PR_SIZE	    txt;
 
     line_bound(line, &xmin, &ymin, &xmax, &ymax);
     if (!overlapping(ZOOMX(xmin), ZOOMY(ymin), ZOOMX(xmax), ZOOMY(ymax),
@@ -628,6 +657,11 @@ void draw_line(F_line *line, int op)
     }
     /* is it a picture object or a Fig figure? */
     if (line->type == T_PICTURE) {
+	    char		*string;
+	    int			length, asc, desc;
+	    F_pos		offset;
+	    struct xfig_font	*mono_font;
+
 	if (line->pic->pic_cache) {
 	    if ((line->pic->pic_cache->bitmap != NULL) && active_layer(line->depth)) {
 		/* only draw the picture if there is a pixmap AND this layer is active */
@@ -645,11 +679,14 @@ void draw_line(F_line *line, int op)
 
 	/* either there is no pixmap or this layer is grayed out,
 	   label empty pic bounding box */
-	if (!line->pic->pic_cache || line->pic->pic_cache->file[0] == '\0')
+	mono_font = font_open(True, PICFILE_FONT, PICFONT_SIZE, 0.0);
+	if (!line->pic->pic_cache || !line->pic->pic_cache->file ||
+					line->pic->pic_cache->file[0] == '\0')
 	    string = EMPTY_PIC;
-	else {
-	    string = xf_basename(line->pic->pic_cache->file);
-	}
+	else
+		/* xf_basename anyhow walks over possibly prepended chars */
+	    string = conv_utf8strdup(xf_basename(line->pic->pic_cache->file));
+
 	p0 = line->points;
 	p1 = p0->next;
 	p2 = p1->next;
@@ -657,33 +694,54 @@ void draw_line(F_line *line, int op)
 	ymin = min3(p0->y, p1->y, p2->y);
 	xmax = max3(p0->x, p1->x, p2->x);
 	ymax = max3(p0->y, p1->y, p2->y);
-	canvas_font = lookfont(0, 12);	/* get a size 12 font */
-	txt = textsize(canvas_font, strlen(string), string);
-	/* if the box is large enough, put the filename in the four corners */
-	if (xmax - xmin > 2.5*txt.length) {
-	    int u,d,marg;
-	    u = txt.ascent;
-	    d = txt.descent;
-	    marg = 6 * ZOOM_FACTOR;	/* margin space around text */
 
-	    pw_text(canvas_win, xmin+marg, ymin+u+marg, op, line->depth,
-			canvas_font, 0.0, string, DEFAULT, GREEN);
-	    pw_text(canvas_win, xmax-txt.length-marg, ymin+u+marg, op, line->depth,
-			canvas_font, 0.0, string, DEFAULT, GREEN);
+	/* compute the text extents */
+	offset = font_textlen(mono_font, PICFILE_FONT, string,
+				(int)strlen(string));
+	font_ascentdescent(True, PICFILE_FONT, PICFONT_SIZE, &asc, &desc);
+	length = offset.x;
+	asc /= ZOOM_FACTOR;
+	desc /= ZOOM_FACTOR;
+
+	/* if the box is large enough, put the filename in the four corners */
+	if (xmax - xmin > 2.5 * length / zoomscale) {
+	    int	left = 0; //extents.x / zoomscale;		/* distance from left */
+	    int	top = asc / zoomscale;		/* distance from top */
+	    int	right = length / zoomscale;	/* distance from right*/
+	    int bottom = desc / zoomscale;
+	    int	marg = 6 * ZOOM_FACTOR;		/* margin space around text */
+
+	    pw_xfttext(canvas_draw,
+			    xmin + marg + left,
+			    ymin + marg + top,
+			    line->depth, mono_font->xft_font, 0, string, GREEN4);
+	    pw_xfttext(canvas_draw,
+			    xmax - marg - right,
+			    ymin + marg + top,
+			    line->depth, mono_font->xft_font, 0, string, GREEN4);
 	    /* do bottom two corners if tall enough */
-	    if (ymax - ymin > 3*(u+d)) {
-		pw_text(canvas_win, xmin+marg, ymax-d-marg, op, line->depth,
-			canvas_font, 0.0, string, DEFAULT, GREEN);
-		pw_text(canvas_win, xmax-txt.length-marg, ymax-d-marg, op, line->depth,
-			canvas_font, 0.0, string, DEFAULT, GREEN);
+	    if (ymax - ymin > 3. * (asc + desc) / zoomscale) {
+		    pw_xfttext(canvas_draw,
+				    xmin + marg + left,
+				    ymax - marg - bottom,
+				    line->depth, mono_font->xft_font, 0, string, GREEN4);
+		    pw_xfttext(canvas_draw,
+				    xmax - marg - right,
+				    ymax - marg - bottom,
+				    line->depth, mono_font->xft_font, 0, string, GREEN4);
 	    }
 	} else {
 	    /* only room for one label - center it */
-	    x = (xmin + xmax) / 2 - txt.length/display_zoomscale / 2;
-	    y = (ymin + ymax) / 2;
-	    pw_text(canvas_win, x, y, op, line->depth, canvas_font, 0.0, string, DEFAULT, GREEN);
+	    x = (xmin + xmax) / 2 - length / (2. * zoomscale);
+	    y = (ymin + ymax) / 2 + (0.5*(asc - desc))/zoomscale;
+	    pw_xfttext(canvas_draw, x, y, line->depth, mono_font->xft_font, 0, string,
+			    GREEN4);
 	}
+	if (string != EMPTY_PIC)
+		free(string);
+	font_close(mono_font);
     }
+
     /* get first point and coordinates */
     point = line->points;
     x = point->x;
@@ -697,7 +755,7 @@ void draw_line(F_line *line, int op)
 	/* label the point number above the point */
 	if (appres.shownums && active_layer(line->depth)) {
 	    pw_text(canvas_win, x, round(y-3.0/zoomscale), PAINT, line->depth,
-			roman_font, 0.0, "0", RED, COLOR_NONE);
+			roman_font, "0", RED, COLOR_NONE);
 	}
 	return;
     }
@@ -716,7 +774,7 @@ void draw_line(F_line *line, int op)
 		(line->type != T_BOX && line->type != T_POLYGON)) {
 		sprintf(bufx,"%d",i++);
 		pw_text(canvas_win, x, round(y-3.0/zoomscale), PAINT, line->depth,
-			roman_font, 0.0, bufx, RED, COLOR_NONE);
+			roman_font, bufx, RED, COLOR_NONE);
 	    }
 	}
 	if (!add_point(x, y)) {
@@ -771,7 +829,7 @@ void draw_arcbox(F_line *line, int op)
 	if (appres.shownums && active_layer(line->depth)) {
 	    sprintf(bufx,"%d",i++);
 	    pw_text(canvas_win, point->x, round(point->y-3.0/zoomscale), PAINT, line->depth,
-			roman_font, 0.0, bufx, RED, COLOR_NONE);
+			roman_font, bufx, RED, COLOR_NONE);
 	}
     }
     pw_arcbox(canvas_win, xmin, ymin, xmax, ymax, round(line->radius*ZOOM_FACTOR),
@@ -936,11 +994,13 @@ void create_pic_pixmap(F_line *box, int rotation, int width, int height, int fli
 	    nbytes = (width + 7) / 8;
 	    bbytes = (cwidth + 7) / 8;
 	    if ((data = (unsigned char *) malloc(nbytes * height)) == NULL) {
-		file_msg(ALLOC_PIC_ERR, box->pic->pic_cache->file);
+		file_msg(ALLOC_PIC_ERR,
+				ABSOLUTE_PATH(box->pic->pic_cache->file));
 		return;
 	    }
 	    if ((tdata = (unsigned char *) malloc(nbytes)) == NULL) {
-		file_msg(ALLOC_PIC_ERR, box->pic->pic_cache->file);
+		file_msg(ALLOC_PIC_ERR,
+				ABSOLUTE_PATH(box->pic->pic_cache->file));
 		free(data);
 		return;
 	    }
@@ -1000,15 +1060,14 @@ void create_pic_pixmap(F_line *box, int rotation, int width, int height, int fli
 	    }
 
 	    if (box->pic->pic_cache->subtype == T_PIC_XBM) {
-		fg = x_color(box->pen_color);		/* xbm, use object pen color */
-		bg = x_bg_color.pixel;
-	    } else if (box->pic->pic_cache->subtype == T_PIC_EPS ||
-			box->pic->pic_cache->subtype == T_PIC_PDF) {
-		fg = black_color.pixel;			/* pbm from gs is inverted */
-		bg = white_color.pixel;
+		fg = getpixel(box->pen_color);	/* xbm, use object pen color */
+		bg = getpixel(CANVAS_BG);
+	    } else if (box->pic->pic_cache->subtype == T_PIC_EPS) {
+		fg = getpixel(BLACK);		/* pbm from gs is inverted */
+		bg = getpixel(WHITE);
 	    } else {
-		fg = white_color.pixel;			/* gif, xpm after map_to_mono */
-		bg = black_color.pixel;
+		fg = getpixel(WHITE);		/* gif, xpm after map_to_mono */
+		bg = getpixel(BLACK);
 	    }
 
 	    box->pic->pixmap = XCreatePixmapFromBitmapData(tool_d, canvas_win,
@@ -1054,14 +1113,16 @@ void create_pic_pixmap(F_line *box, int rotation, int width, int height, int fli
 	    cbpl = cwidth * cbpp;
 	    bpl = width * image_bpp;
 	    if ((data = malloc(bpl * height)) == NULL) {
-		file_msg(ALLOC_PIC_ERR,box->pic->pic_cache->file);
+		file_msg(ALLOC_PIC_ERR,
+				ABSOLUTE_PATH(box->pic->pic_cache->file));
 		return;
 	    }
 	    /* allocate mask for any transparency information */
 	    if (box->pic->pic_cache->subtype == T_PIC_GIF &&
 	        box->pic->pic_cache->transp != TRANSP_NONE) {
 		    if ((mask = (unsigned char *) malloc((width+7)/8 * height)) == NULL) {
-			file_msg(ALLOC_PIC_ERR,box->pic->pic_cache->file);
+			file_msg(ALLOC_PIC_ERR,
+				    ABSOLUTE_PATH(box->pic->pic_cache->file));
 			free(data);
 			return;
 		    }
@@ -1261,66 +1322,84 @@ static char    *hidden_text_string = "<<>>";
 
 void draw_text(F_text *text, int op)
 {
-    PR_SIZE	    size;
-    int		    x,y;
+    int		    ox, oy;
     int		    xmin, ymin, xmax, ymax;
-    int		    x1,y1, x2,y2, x3,y3, x4,y4;
-    double	    cost, sint;
-
-    if (text->zoom != zoomscale || text->fontstruct == (XFontStruct*) 0)
-	reload_text_fstruct(text);
-    text_bound(text, &xmin, &ymin, &xmax, &ymax,
-	       &x1,&y1, &x2,&y2, &x3,&y3, &x4,&y4);
 
+    text_bound(text, &xmin, &ymin, &xmax, &ymax);
     if (!overlapping(ZOOMX(xmin), ZOOMY(ymin), ZOOMX(xmax), ZOOMY(ymax),
 		     clip_xmin, clip_ymin, clip_xmax, clip_ymax))
 	return;
 
-    /* outline the text bounds in red if debug resource is set */
-    if (appres.DEBUG) {
-	pw_vector(canvas_win, x1, y1, x2, y2, op, 1, RUBBER_LINE, 0.0, RED);
-	pw_vector(canvas_win, x2, y2, x3, y3, op, 1, RUBBER_LINE, 0.0, RED);
-	pw_vector(canvas_win, x3, y3, x4, y4, op, 1, RUBBER_LINE, 0.0, RED);
-	pw_vector(canvas_win, x4, y4, x1, y1, op, 1, RUBBER_LINE, 0.0, RED);
-    }
+    text_origin(&ox, &oy, text->base_x, text->base_y, text->type, text->offset);
 
-    x = text->base_x;
-    y = text->base_y;
-    cost = cos(text->angle);
-    sint = sin(text->angle);
-    if (text->type == T_CENTER_JUSTIFIED || text->type == T_RIGHT_JUSTIFIED) {
-	size = textsize(text->fontstruct, strlen(text->cstring),
-			    text->cstring);
-	size.length = size.length/display_zoomscale;
-	if (text->type == T_CENTER_JUSTIFIED) {
-	    x = round(x-cost*size.length/2);
-	    y = round(y+sint*size.length/2);
-	} else {	/* T_RIGHT_JUSTIFIED */
-	    x = round(x-cost*size.length);
-	    y = round(y+sint*size.length);
-	}
-    }
-    if (hidden_text(text)) {
-	pw_text(canvas_win, x, y, op, text->depth, lookfont(0,12),
-		text->angle, hidden_text_string, DEFAULT, COLOR_NONE);
-    } else {
-	/* if size is less than the displayable size, Greek it by drawing a DARK gray line,
-	   UNLESS the depth is inactive in which case draw it in MED_GRAY */
-	if (text->size*display_zoomscale < MIN_X_FONT_SIZE) {
-	    x1 = (x1+x4)/2;
-	    x2 = (x2+x3)/2;
-	    y1 = (y1+y4)/2;
-	    y2 = (y2+y3)/2;
-	    greek_text(text, x1, y1, x2, y2);
-	} else {
-	    /* Otherwise, draw the text normally */
-	    pw_text(canvas_win, x, y, op, text->depth, text->fontstruct,
-		text->angle, text->cstring, text->color, COLOR_NONE);
-	}
-    }
+    /* if size is less than the displayable size,
+       Greek it by drawing a DARK gray line,
+       UNLESS the depth is inactive in which case draw it in MED_GRAY */
+    if (text->size * display_zoomscale > MIN_X_FONT_SIZE) {
+	    Color c;
+	    if (op == PAINT)
+		    c = text->color;
+	    else
+		    c = CANVAS_BG;
+
+	    if (text->zoom != zoomscale)
+		    font_reload(text);
+
+	    pw_xfttext(canvas_draw, ox, oy, text->depth,
+			    text->xf_font->xft_font, text->font,
+			    hidden_text(text) ? hidden_text_string :
+			    text->cstring, c);
+
+	    /* outline the text bounds in red if debug resource is set */
+	    if (appres.DEBUG && op == PAINT && !preview_in_progress) {
+		    pw_vector(canvas_win, xmin, ymin, xmax, ymin,
+				    op, 1, RUBBER_LINE, 0.0, GREEN);
+		    pw_vector(canvas_win, xmax, ymin, xmax, ymax,
+				    op, 1, RUBBER_LINE, 0.0, GREEN);
+		    pw_vector(canvas_win, xmax, ymax, xmin, ymax,
+				    op, 1, RUBBER_LINE, 0.0, GREEN);
+		    pw_vector(canvas_win, xmin, ymax, xmin, ymin,
+				    op, 1, RUBBER_LINE, 0.0, GREEN);
+		    if (text->offset.x != 0 && text->offset.y != 0) {
+			    /* oblique text */
+			    pw_vector(canvas_win, ox, oy,
+					    ox + text->offset.x,
+					    oy + text->offset.y,
+					    op, 1, RUBBER_LINE, 0.0, RED);
+			    pw_vector(canvas_win,
+					    ox + text->offset.x,
+					    oy + text->offset.y,
+					    ox + text->offset.x +
+					    text->offset.y * text->asc_len,
+					    oy + text->offset.y -
+					    text->offset.x * text->asc_len,
+					    op, 1, RUBBER_LINE, 0.0, RED);
+			    pw_vector(canvas_win,
+					    ox + text->offset.x +
+					    text->offset.y * text->asc_len,
+					    oy + text->offset.y -
+					    text->offset.x * text->asc_len,
+					    ox + text->offset.y * text->asc_len,
+					    oy - text->offset.x * text->asc_len,
+					    op, 1, RUBBER_LINE, 0.0, RED);
+			    pw_vector(canvas_win,
+					    ox + text->offset.y * text->asc_len,
+					    oy - text->offset.x * text->asc_len,
+					    ox, oy,
+					    op, 1, RUBBER_LINE, 0.0, RED);
+		    }
+	    }
+
+    } else if (text->size * display_zoomscale > MIN_GREEK_TEXT &&
+		    !hidden_text(text)) {
+	    xmin = ox + text->offset.y * text->asc_len * 0.5;
+	    ymin = oy - text->offset.x * text->asc_len * 0.5;
+	    greek_text(text, xmin, ymin,
+			    xmin + text->offset.x, ymin + text->offset.y);
+    } /* text->size * display_zoomscale <= MIN_GREEK_TEXT ... do nothing */
 
     /* write the depth on the object */
-    debug_depth(text->depth,x,y);
+    debug_depth(text->depth, ox, oy);
 }
 
 /*
@@ -1465,6 +1544,8 @@ void compute_arcarrow_angle(float x1, float y1, int x2, int y2, int direction, F
 
 int tempXErrorHandler (Display *display, XErrorEvent *event)
 {
+	(void)display;
+	(void)event;
 	return 0;
 }
 
@@ -1533,7 +1614,7 @@ void clip_arrows(F_line *obj, int objtype, int op, int skip)
 		}
 		n = i;
 		/* draw the clipping area for debugging */
-		if (appres.DEBUG) {
+		if (appres.DEBUG && !preview_in_progress) {
 		  for (i=0; i<n; i++) {
 		    if (i==n-1)
 			j=0;
@@ -1572,7 +1653,7 @@ void clip_arrows(F_line *obj, int objtype, int op, int skip)
 		}
 		n = i;
 		/* draw the clipping area for debugging */
-		if (appres.DEBUG) {
+		if (appres.DEBUG && !preview_in_progress) {
 		  int j;
 		  for (i=0; i<n; i++) {
 		    if (i==n-1)
@@ -2067,11 +2148,11 @@ void debug_depth(int depth, int x, int y)
     char	str[10];
     PR_SIZE	size;
 
-    if (appres.DEBUG) {
+    if (appres.DEBUG && !preview_in_progress) {
 	sprintf(str,"%d",depth);
 	size = textsize(roman_font, strlen(str), str);
 	pw_text(canvas_win, x-size.length-round(3.0/zoomscale), round(y-3.0/zoomscale),
-		PAINT, depth, roman_font, 0.0, str, RED, COLOR_NONE);
+		PAINT, depth, roman_font, str, RED, COLOR_NONE);
     }
 }
 
@@ -2103,7 +2184,7 @@ draw_spline(F_spline *spline, int op)
 	    /* label the point number above the point */
 	    sprintf(bufx,"%d",i++);
 	    pw_text(canvas_win, p->x, round(p->y-3.0/zoomscale), PAINT, spline->depth,
-		roman_font, 0.0, bufx, RED, COLOR_NONE);
+		roman_font, bufx, RED, COLOR_NONE);
 	}
     }
     if (open_spline(spline))
