@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2021 by Thomas Loimer
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -17,8 +17,9 @@
  */
 
 /*
- * Call ghostscript to get the /MediaBox, and convert eps/pdf to bitmaps.
- * Autor: Thomas Loimer, 2020.
+ * Routines to (i) get the bounding box of a pdf, (ii) convert eps/pdf files
+ * to bitmaps.
+ * Author: Thomas Loimer, 2020-2023.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,6 +54,53 @@
 #define BITMAP_PPI	160	/* the resolution for rendering bitmaps */
 #define GS_ERROR	(-2)
 
+/*
+ * Ghostscript command lines to get the mediabox of a pdf
+ *
+ * gs < 9.50 (rev.revision < 950)
+ *     gs -q -dNODISPLAY -dNOSAFER -c "(in.pdf) (r) file "
+ *         "runpdfbegin 1 pdfgetpage /MediaBox pget pop == runpdfend quit"
+ * gs >= 9.50
+ *    gs -q -dNODISPLAY --permit-file-read=in.pdf -c "(in.pdf) (r) file "
+ *         "runpdfbegin 1 pdfgetpage /MediaBox pget pop == runpdfend quit"
+ *
+ * Beginning with gs 9.50, "-dSAFER" is the default, and permission to access
+ * files must be explicitly given with the --permit-file-{read,write,..}
+ * options. Before gs 9.50, "-dNOSAFER" is the default.
+ *
+ * rev.revision: As of version 9.53, the patch level is included into the
+ * version number <https://ghostscript.readthedocs.io/en/gs10.01.0/News.html>,
+ * therefore: 952, next: 9530
+ *
+ * The commands above write the coordinates of the lower left and upper right
+ * corners to stdout, enclosed by square braces: "[lx lly ..]".
+ *
+ * The command line was found, and modified a bit, at
+ *https://stackoverflow.com/questions/2943281/using-ghostscript-to-get-page-size
+ * With commit [a9181d] "runpdfend" was inserted, see also
+ * <https://bugs.ghostscript.com/show_bug.cgi?id=705855> and
+ * <https://bugs.ghostscript.com/show_bug.cgi?id=705836>
+ *
+ * The command line calls the pdf interpreter, see
+ * <https://ghostscript.readthedocs.io/en/gs10.01.0/Language.html#scripting-the-pdf-interpreter>
+ * (in.pdf) (r) file	open in.pdf for reading, return a file object
+ * <file> runpdfbegin	set up the environment to process the file as pdf
+ * 1 pdfgetpage		read page 1, return a dict
+ * <dict> /MediaBox pget  if /MediaBox is found, return it's value and true
+ * pop			pop the "true" from the stack
+ * ==			print the value on the stack
+ * runpdfend		terminate processing of the pdf
+ *
+ * Instead of "/Mediabox pget pop ==" one could use "/Mediabox get ==". pget is
+ * defined in /usr/share/ghostscript/Resource/Init/pdf_main.ps and retrieves a
+ * (possibly inherited) key value from a page dict.
+ *
+ * A different option is to use
+ *    gs -q -dNODISPLAY -dNOPAUSE -dBATCH -sPageList=1 -dPDFINFO 'in.pdf'
+ * which writes "MediaBox: [llx lly ..]" besides much information to stderr.
+ *
+ */
+
 /*
  * Link into the ghostscript library.
  * If invoked via the library, ghostscript calls three callback functions when
@@ -62,16 +110,13 @@
  * necessary.
  * Here, set up calls to ghostscript and callback functions to read a pdf
  * file, scanning the output for the /MediaBox specification.
- *
- * If linking into the library by dlopen() fails, call the ghostscript
- * executable via popen("gs..", "r").
  */
 
 #ifdef HAVE_GSLIB
 /*
  * callback data struct
  * A pointer to this struct can be passed to ghostscript, which
- * is then passed to the callback functions.
+ * is then passed to the stdio-callback functions.
  */
 struct _callback_data {
 	int	*bb;		/* for stdout_mediabox() callback function */
@@ -115,8 +160,17 @@ stdout_mediabox(void *caller_handle, const char *str, int len)
 	int	err;
 	double	fbb[4];
 
-	/* This rests on the assumption that ghostscript writes the required
-	   information all at once to str. Should use a buffer, instead. */
+	if (data->bb[2] >= 0 || data->bb[2] == GS_ERROR) {
+		/* bounding box already determined */
+		return len;
+	} else if (!strchr(str, ']')) {
+		/* return a short character read for incomplete information */
+		char	*p;
+		if ((p = strchr(str, '[')))
+			return p - str;
+		else
+			return 0;
+	}
 
 	/* gs always uses full stops as decimal character. The locale is already
 	   set to C in read_pdf() */
@@ -128,7 +182,7 @@ stdout_mediabox(void *caller_handle, const char *str, int len)
 		data->bb[3] = (int)ceil(fbb[3]);
 	} else {
 		/* Either the pdf is corrupt, which yields a matching failure,
-		 * or a read error occured or EOF is reached.
+		 * or a read error occurred or EOF is reached.
 		 * Ghostscript returns with zero from a corrupt pdf, and writes
 		 * error information to stdout.
 		 * Use the bounding box to report a, most likely, corrupt pdf.
@@ -314,18 +368,8 @@ gsexe(FILE **out, bool *isnew, char *exenew, char *exeold)
 }
 
 /*
- * Call ghostscript to extract the /MediaBox from the pdf given in file.
- * Command line, for gs >= 9.50,
- *    gs -q -dNODISPLAY --permit-file-read=in.pdf -c \
- *	"(in.pdf) (r) file runpdfbegin 1 pdfgetpage /MediaBox pget pop == quit"
- * gs < 9.50:
- *    gs -q -dNODISPLAY -dNOSAFER -c \
- *	"(in.pdf) (r) file runpdfbegin 1 pdfgetpage /MediaBox pget pop == quit"
- * The command line was found, and modified a bit, at
- *https://stackoverflow.com/questions/2943281/using-ghostscript-to-get-page-size
- * Beginning with gs 9.50, "-dSAFER" is the default, and permission to access
- * files must be explicitly given with the --permit-file-{read,write,..}
- * options. Before gs 9.50, "-dNOSAFER" is the default.
+ * Call ghostscript to extract the bounding box from the pdf given in file.
+ * See above for an description of the command line.
  *
  * Return 0 on success, -1 on failure, -2 (GS_ERROR) for a ghostscript-error,
  * -3 if the path to the ghostscript executable is not given.
@@ -349,9 +393,11 @@ gsexe_mediabox(char *file, int *llx, int *lly, int *urx, int *ury)
 		return -3;
 
 	exenew = "%s -q -dNODISPLAY \"--permit-file-read=%s\" -c \"(%s) (r) "
-		"file runpdfbegin 1 pdfgetpage /MediaBox pget pop == quit\"";
+		"file runpdfbegin 1 pdfgetpage /MediaBox pget pop == "
+		"runpdfend quit\"";
 	exeold = "%s -q -dNODISPLAY -c \"(%s) (r) "
-		"file runpdfbegin 1 pdfgetpage /MediaBox pget pop == quit\"";
+		"file runpdfbegin 1 pdfgetpage /MediaBox pget pop == "
+		"runpdfend quit\"";
 
 	/* malloc() buffers for the command line, if necessary */
 	fmt = exenew;
@@ -416,7 +462,7 @@ gsexe_mediabox(char *file, int *llx, int *lly, int *urx, int *ury)
 
 #ifdef HAVE_GSLIB
 /*
- * Return codes: 0..success, -1..within the function, an error occured,
+ * Return codes: 0..success, -1..within the function, an error occurred,
  *  GS_ERROR..the ghostscript interpreter returned an error
  */
 static int
@@ -445,7 +491,8 @@ gslib_mediabox(char *file, int *llx, int *lly, int *urx, int *ury)
 	argnew[3] = "--permit-file-read=%s";	/* file */
 	argnew[4] = "-c";
 	argnew[5] =
-	    "(%s) (r) file runpdfbegin 1 pdfgetpage /MediaBox pget pop == quit";
+	    "(%s) (r) file runpdfbegin 1 pdfgetpage /MediaBox pget pop == "
+	    "runpdfend quit";
 
 	argold[0] = argnew[0];
 	argold[1] = argnew[1];
@@ -762,7 +809,7 @@ bitmap_stdout(void *data_handle, const char *str, int len)
 	return len;
 }
 
-/* Data provided to the callback functions */
+/* Data provided to the display callback functions */
 struct calldata {
 	int		width;
 	int		height;
@@ -805,12 +852,12 @@ display_presize(void *handle, void *device, int width, int height,
 }
 
 static void *
-display_memalloc(void *handle, void *device, unsigned long size)
+display_memalloc(void *handle, void *device, size_t size)
 {
 	(void) device;
 	struct calldata	*data = (struct calldata *)handle;
 
-	data->img = malloc((size_t)size);
+	data->img = malloc(size);
 
 	if (appres.DEBUG && data->img == NULL)
 		fputs("gslib_bitmap() - display_memalloc(): Out of memory.\n",
@@ -889,14 +936,58 @@ display_close(void *handle, void *device)
 	return 0;
 }
 
+/*
+ * All the display callback functions above are provided via a struct.
+ */
+static struct display_callback_s callback_functions = {
+	sizeof(struct display_callback_s),
+	DISPLAY_VERSION_MAJOR,
+	DISPLAY_VERSION_MINOR,
+	display_open,
+	display_preclose,
+	display_close,
+	display_presize,
+	display_size,
+	display_sync,
+	display_page,
+	NULL,		/* display_update */
+	display_memalloc,
+	display_memfree,
+#if DISPLAY_VERSION_MAJOR > 1
+	NULL,		/* display_separation */
+#if DISPLAY_VERSION_MAJOR > 2
+	NULL,		/* display_adjust_band_height */
+	NULL		/* display_rectangle_request */
+#endif
+#endif
+};
+
+#ifdef DISPLAY_CALLOUT_GET_CALLBACK
+static int
+callout_function(void *inst, void *callout_handle, const char *device,
+		int id, int size, void *data)
+{
+	(void)inst;
+	(void)size;
+	gs_display_get_callback_t	*cb;
+
+	if (device == NULL || strcmp(device, "display") ||
+			id != DISPLAY_CALLOUT_GET_CALLBACK)
+		return gs_error_unknownerror;
+
+	cb = (gs_display_get_callback_t *)data;
+	cb->callback = &callback_functions;
+	cb->caller_handle = callout_handle;
+	return gs_error_ok;
+}
+#endif
+
 /*
  * Link into the ghostscript library, writing to the display device.
  * Return 0 on success, -1 on failure, or GS_ERROR for a ghostscript error.
  * TODO: With a corrupt pdf, ghostscript will write messages to stdout and
  * return 0. A stdout callback should be used to catch these messages.
  * Warning messages seem to be directed to stderr.
- * TODO: After gs 9.52, ghostscript uses a callout function for registering the
- * callback functions. Implement that interface.
  */
 static int
 gslib_bitmap(char *file, F_pic *pic, int llx, int lly, int urx, int ury)
@@ -934,24 +1025,6 @@ gslib_bitmap(char *file, F_pic *pic, int llx, int lly, int urx, int ury)
 		errbuf
 	};
 
-	/* callback structure for "display" device */
-	struct display_callback_s callback_functions = {
-		sizeof(struct display_callback_s),
-		DISPLAY_VERSION_MAJOR,
-		DISPLAY_VERSION_MINOR,
-		display_open,
-		display_preclose,
-		display_close,
-		display_presize,
-		display_size,
-		display_sync,
-		display_page,
-		NULL,	/* display_update */
-		display_memalloc,
-		display_memfree,
-		NULL	/* display_separation */
-	};
-
 
 	/* This should be the size to which a pixmap is rendered for display
 	   on the canvas, at a magnification of 2.
@@ -1011,7 +1084,6 @@ gslib_bitmap(char *file, F_pic *pic, int llx, int lly, int urx, int ury)
 	arg[3] = "-dBATCH";
 	arg[4] = "-dNOPAUSE";
 	arg[5] = "-sDEVICE=display";
-	arg[6] = "-sDisplayHandle=16#%" PRIxPTR;
 	arg[7] = "-dDisplayFormat=%d";
 	arg[8] = "-r%d";	/* resolution */
 	arg[9] = "-g%dx%d";	/* width x height */
@@ -1020,8 +1092,14 @@ gslib_bitmap(char *file, F_pic *pic, int llx, int lly, int urx, int ury)
 	arg[12] = "-f";		/* terminate list of tokens for the -c switch */
 	arg[13] = file;
 
+#ifdef DISPLAY_CALLOUT_GET_CALLBACK
+	arg[6] = arg[1];	/* an empty argument ("") does not work */
+	(void)arg6;
+#else
+	arg[6] = "-sDisplayHandle=16#%" PRIxPTR;
 	sprintf(arg6, arg[6], (uintptr_t)&handle);
 	arg[6] = arg6;
+#endif
 	sprintf(arg7, arg[7], format);
 	arg[7] = arg7;
 	sprintf(arg8, arg[8], BITMAP_PPI);
@@ -1049,7 +1127,11 @@ gslib_bitmap(char *file, F_pic *pic, int llx, int lly, int urx, int ury)
 		return gsexe_bitmap(file, pic, llx, lly, urx, ury);
 
 	if (code == 0)
+#ifdef DISPLAY_CALLOUT_GET_CALLBACK
+		code = gsapi_register_callout(minst, callout_function, &handle);
+#else
 		code = gsapi_set_display_callback(minst, &callback_functions);
+#endif
 	if (code == 0)
 		code = gsapi_set_arg_encoding(minst, GS_ARG_ENCODING_UTF8);
 	if (code == 0)
