@@ -1,9 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Copyright (c) 1991 by Henning Spruth
+ * Parts Copyright (c) 1991 by Henning Spruth
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -17,7 +18,7 @@
  */
 
 extern void	pan_origin(void);
-extern int pan_down (int shift);
-extern int pan_left (int shift);
-extern int pan_right (int shift);
-extern int pan_up (int shift);
+extern void	pan_down (int shift);
+extern void	pan_left (int shift);
+extern void	pan_right (int shift);
+extern void	pan_up (int shift);
