@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2021 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,20 +16,29 @@
  *
  */
 
-#include "fig.h"
-#include "resources.h"
+#include "u_search.h"
+
+#include <stdlib.h>
+#include <math.h>
+#include <X11/Xlib.h>
+
+#include "object.h"
+
 #include "object.h"
 #include "mode.h"
-#include "u_list.h"
-#include "u_search.h"
-#include "w_drawprim.h"
-#include "w_layers.h"
-#include "w_setup.h"
-#include "w_zoom.h"
-#include "w_snap.h"
-
+#include "u_bound.h"
+#include "u_fonts.h"
 #include "u_geom.h"
+#include "u_list.h"
 #include "u_markers.h"
+#include "u_search.h"
+#include "w_layers.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_snap.h"
+#include "w_zoom.h"
+#include "xfig_math.h"
+
 
 #define TOLERANCE ((int)((display_zoomscale < 20.0? 10: 14) * \
 			PIX_PER_INCH/DISPLAY_PIX_PER_INCH/display_zoomscale))
@@ -306,7 +315,7 @@ next_spline_found(int x, int y, int tolerance, int *px, int *py, unsigned int sh
 Boolean
 next_text_found(int x, int y, int tolerance, int *px, int *py, unsigned int shift)
 {
-    int		    dum;
+	(void)tolerance;
 
     if (!anytext_in_mask())
 	return False;
@@ -325,7 +334,7 @@ next_text_found(int x, int y, int tolerance, int *px, int *py, unsigned int shif
 	}
 	if (validtext_in_mask(t)) {
 	    ++n;
-	    if (in_text_bound(t, x, y, &dum, False)) {
+	    if (in_text_bound(t, x, y)) {
 		*px = x;
 		*py = y;
 		return True;
@@ -869,84 +878,69 @@ point_search_right(int x, int y, unsigned int shift)
 }
 
 F_text	       *
-text_search(int x, int y, int *posn)
+text_search(int x, int y)
 {
     F_text	   *t;
 
     for (t = objects.texts; t != NULL; t = t->next) {
-	if (active_layer(t->depth) && in_text_bound(t, x, y, posn, False))
+	if (active_layer(t->depth) && in_text_bound(t, x, y))
 		return(t);
     }
     return (NULL);
 }
 
-/* return true if (x,y) is in the text rectangle by rotating the point (x,y)
-   around the text base point by it's negative angle and seeing if that is
-   in the rectangle.
-   Additionally, set posn to the pixel position of the mouse from the beginning
-   of the string
-   Call with extra = True to consider small distance on either end of string as
-   "inside" the bounds.  This is used by the text select tracker (track_text_select).
+/*
+ * Return true if (x,y) is inside the text rectangle.
+ *
+ * Check if inside by computing the scalar products with the two vectors
+ * defining the rectangle,
+ *
+ *  x A               0 <= dot(OA,OP) <= dot(OA,OA) &&
+ *  |                 0 <= dot(OB,OP) <= dot(OB,OB)
+ *  |  x P
+ *  | /               Acknowledgements to:
+ *  |/                https://stackoverflow.com/a/2763387/12428326
+ *  O---------x B
+ *
+ * OB = (offset.x, offset.y);  OA = (offset.y * asc_len, -offset.x * asc_len)
+ * An extra parameter to enlarge the rectangle by a small amount was previously
+ * used by the text select tracker, track_text_select().
  */
-
 Boolean
-in_text_bound(F_text *t, int x, int y, int *posn, Boolean extra)
+in_text_bound(F_text *t, int x, int y)
 {
-    double	    cost, sint;
-    int		    xo,yo, xr,yr;
-    int		    x0, x1,y1, x2,y2;
-    int		    l, h;
+	int	xmin, ymin, xmax, ymax;
+	F_pos	op;
+	int	oa_dot_op;
+	int	ob_dot_op;
+	int	len2;
 
-    cost = cos((double) -t->angle);
-    sint = sin((double) -t->angle);
-    xo = t->base_x;
-    yo = t->base_y;
+	text_bound(t, &xmin, &ymin, &xmax, &ymax);
+	if (x < xmin || x > xmax || y < ymin || y > ymax)
+		return False;
+	else if (t->offset.x == 0 || t->offset.y == 0)
+		return True;
 
-    /* rotate the point (x,y) about (xo,yo) giving (xr,yr) */
-    xr = xo + (x-xo)*cost - (yo-y)*sint;
-    yr = yo - (yo-y)*cost - (x-xo)*sint;
-    /* now see if that point is in the text bounds of the unrotated text */
-    l = text_length(t);
-    /* add 5 char widths to length if extra wanted */
-    if (extra)
-	l += 5*char_width(t->fontstruct);
-    h = t->ascent+t->descent;
-    x1 = t->base_x;
-    /* use x0 for left bound comparison but use x1 for char position in string of x,y */
-    x0 = x1;
-    /* subtract 4 char widths before start of string */
-    if (extra)
-	x0 -= 4*char_width(t->fontstruct);
-    y1 = t->base_y+t->descent;
-    if (t->type == T_CENTER_JUSTIFIED) {
-	x2 = x1 + l/2;
-	x1 = x1 - l/2;
-	x0 = x0 - l/2;
-	y2 = y1 - h;
-    }
-    else if (t->type == T_RIGHT_JUSTIFIED) {
-	x2 = x1;
-	x1 = x1 - l;
-	x0 = x0 - l;
-	y2 = y1 - h;
-    }
-    else {
-	x2 = x1 + l;
-	y2 = y1 - h;
-    }
-    if (xr >= x0 && xr <= x2 && yr <= y1 && yr >= y2) {
-	/* return the pixel position from the beginning of the string */
-	*posn = xr-x1;
-	if (*posn < 0)
-	    *posn = 0;
-	else if (*posn > text_length(t))
-	    *posn = text_length(t);
-	return True;
-    }
-    return False;
+	/* for oblique text, continue */
+	/* see text_bound() for draw_x, draw_y */
+	text_origin(&op.x, &op.y, t->base_x, t->base_y, t->type, t->offset);
+	op.x = x - op.x;
+	op.y = y - op.y;
+	/* 0 <= dot(OA,OP) <= dot(OA,OA) && 0 <= dot(OB,OP) <= dot(OB,OB) */
+	oa_dot_op = t->offset.y * t->asc_len * op.x -
+					t->offset.x * t->asc_len * op.y;
+	ob_dot_op = t->offset.x * op.x + t->offset.y * op.y;
+
+	len2 = t->length * t->length;
+	if (0 > oa_dot_op || oa_dot_op > t->asc_len * t->asc_len * len2 ||
+			0 > ob_dot_op || ob_dot_op > len2)
+		return False;
+	else
+		return True;
 }
 
-F_compound     *
+
+F_compound *
 compound_search(int x, int y, int tolerance, int *px, int *py)
 {
     F_compound	   *c;
