@@ -1,7 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Copyright (c) 1995 Jim Daley (jdaley@cix.compulink.co.uk)
- * Parts Copyright (c) 1995-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,23 +17,29 @@
  *
  */
 
-#include "fig.h"
+#include "w_browse.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>		/* XA_STRING */
+
 #include "figx.h"
 #include "resources.h"
-#include "object.h"
-#include "mode.h"
 #include "e_edit.h"
-#include "u_create.h"
-#include "w_drawprim.h"
-#include "w_dir.h"
-#include "w_indpanel.h"
-#include "w_msgpanel.h"
-#include "w_util.h"
-#include "w_setup.h"
-
+#include "f_picobj.h"
 #include "f_util.h"
+#include "u_create.h"
 #include "w_color.h"
 #include "w_cursor.h"
+#include "w_dir.h"
+#include "w_drawprim.h"
+#include "w_setup.h"
+#include "w_util.h"
+
 
 /* EXPORTS */
 
@@ -44,7 +53,7 @@ Widget		browse_selfile,	/* selected file widget */
 Boolean		browse_up;
 
 
-DeclareStaticArgs(12);
+DeclareStaticArgs(13);
 static Widget	cfile_lab;
 static Widget	closebtn, apply;
 static Widget	browse_parent;
@@ -60,7 +69,7 @@ static String	file_list_translations =
 static String	file_name_translations =
 	"<Key>Return: ApplyBrowseAndClose()\n";
 static void	browse_panel_close(Widget w, XButtonEvent *ev);
-void		got_browse(Widget w, XButtonEvent *ev);
+static void	got_browse(Widget w, XButtonEvent *ev);
 
 static XtActionsRec	file_name_actions[] =
 {
@@ -107,9 +116,14 @@ browse_panel_dismiss(void)
 void
 got_browse(Widget w, XButtonEvent *ev)
 {
-    char	   *fval, *dval, *path;
+	(void)w;
+	(void)ev;
+	char	*abs_path = NULL;
+	char	*fval, *dval;
+
+	if (!browse_popup)
+		return;
 
-    if (browse_popup) {
 	FirstArg(XtNstring, &dval);
 	GetValues(browse_dir);
 	FirstArg(XtNstring, &fval);
@@ -122,21 +136,47 @@ got_browse(Widget w, XButtonEvent *ev)
 	if (emptyname_msg(fval, "Apply"))
 	    return;
 
-	path = new_string( strlen(dval) + 1 + strlen(fval) + 1);
-	if ( path ) {
-	    strcpy( path,dval );
-	    strcat( path, "/");
-	    strcat( path, fval );
-	    panel_set_value( pic_name_panel, path );
-	    free(path);
+	if (fval[0] == '/' || fval[0] == '~') {
+		/* the user entered an absolute path, or relative to $HOME */
+		abs_path = internal_path(fval);
+	} else {
+		char	full_path_buf[128];
+		char	*full_path = full_path_buf;
+		size_t	dval_len, fval_len;
+		dval_len = strlen(dval);
+		fval_len = strlen(fval);
+		if (dval_len + 1 + fval_len + 1 >= sizeof full_path_buf)
+			full_path = new_string(dval_len + fval_len + 1);
+		if (full_path) {
+			/* Create an absolute file name.. */
+			memcpy(full_path, dval, dval_len);
+			full_path[dval_len] = '/';
+			memcpy(full_path + dval_len + 1, fval, fval_len + 1);
+			abs_path = realpath(full_path, NULL);
+		}
+		if (full_path != full_path_buf)
+			free(full_path);
 	}
+	if (abs_path) {
+		char	path_buf[128];
+		char	*path = path_buf;
+		if (external_path(&path, sizeof path_buf, abs_path) >= 0)
+			panel_set_value(pic_name_panel, path);
+		else
+			panel_set_value(pic_name_panel, "");
+		free(abs_path);
+		if (path != path_buf)
+			free(path);
+	}
+	/* inter alia, writes the path name to the picture object */
 	push_apply_button();  /* slightly iffy - assumes called from edit picture */
-    }
 }
 
 static void
 browse_panel_close(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     browse_panel_dismiss();
 }
 
@@ -168,6 +208,8 @@ void popup_browse_panel(Widget w)
       strcpy(browse_filename, fval+1);
       local_dir[strlen(pval) - strlen(fval)] = '\0';
       (void) change_directory(local_dir);
+      /* and get the canonical path */
+      get_directory(local_dir);
     }
 
     if (!browse_popup) {
@@ -200,6 +242,7 @@ void create_browse_panel(Widget w)
 	FirstArg(XtNx, xposn);
 	NextArg(XtNy, yposn + 50);
 	NextArg(XtNtitle, "Xfig: Browse files for picture import");
+	NextArg(XtNtitleEncoding, XA_STRING);
 	browse_popup = XtCreatePopupShell("xfig_browse_menu",
 					transientShellWidgetClass,
 					tool, Args, ArgCount);
@@ -210,6 +253,7 @@ void create_browse_panel(Widget w)
 					   browse_popup, NULL, ZERO);
 
 	FirstArg(XtNlabel, "     Filename");
+	NextArg(XtNinternational, False);
 	NextArg(XtNvertDistance, 15);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -223,6 +267,7 @@ void create_browse_panel(Widget w)
 	NextArg(XtNheight, max_char_height(temp_font) * 2 + 4);
 	NextArg(XtNeditType, "edit");
 	NextArg(XtNstring, browse_filename);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNinsertPosition, strlen(browse_filename));
 	NextArg(XtNfromHoriz, file);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -254,6 +299,7 @@ void create_browse_panel(Widget w)
 	XtAugmentTranslations(browse_flist,
 			   XtParseTranslationTable(file_list_translations));
 	FirstArg(XtNlabel, " Close ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNvertDistance, 15);
 	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNheight, 25);
@@ -269,6 +315,7 @@ void create_browse_panel(Widget w)
 
 
 	FirstArg(XtNlabel,  " Apply ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNfromHoriz, closebtn);
 	NextArg(XtNfromVert, below);
