@@ -1,7 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1995 Jim Daley (jdaley@cix.compulink.co.uk)
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 1995 Jim Daley (jdaley@cix.compulink.co.uk)
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,5 +17,14 @@
  *
  */
 
-Boolean	captureImage(Widget window, char *filename);		/* returns True on success */
-Boolean	canHandleCapture(Display *d);	/* returns True if image capture will works */
+#ifndef W_CAPTURE_H
+#define W_CAPTURE_H
+
+#include <X11/Intrinsic.h>	/* includes X11/Xlib.h */
+
+/* returns True on success */
+extern Boolean	captureImage(Widget window, char *filename);
+/* returns True if image capture will work */
+extern Boolean	canHandleCapture(Display *d);
+
+#endif
