@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,11 +16,16 @@
  *
  */
 
+#ifndef W_CMDPANEL_H
+#define W_CMDPANEL_H
+
+#include <X11/IntrinsicP.h>
+//#include <X11/Intrinsic.h>	/* includes X11/Xlib.h */
+
 /* width of a command button */
 #define CMD_BUT_WD 60
 #define CMD_BUT_HT 22
 
-
 /* def for menu */
 
 typedef struct {
@@ -59,10 +65,12 @@ extern int	num_main_menus(void);
 extern Widget	create_menu_item(main_menu_info *menup);
 extern void	refresh_view_menu(void);
 extern void	popup_character_map(void);
-extern void	refresh_character_panel(void);
+extern void	refresh_character_panel(int ps_sel, int font_sel);
 extern void     init_main_menus(Widget tool, char *filename);
 
 extern void goodbye (Boolean abortflag);
 extern void update_cur_filename (char *newname);
 extern void rebuild_file_menu (Widget menu);
 void setup_main_menus(void);
+
+#endif
