@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,9 +16,15 @@
  *
  */
 
+#ifndef W_CURSOR_H
+#define W_CURSOR_H
+
+#include <X11/X.h>
+
 extern void	set_cursor(Cursor cursor);
 extern void	set_temp_cursor(Cursor cursor);
 extern void	init_cursor(void);
 extern void     recolor_cursors(void);
+extern void	reset_cursor (void);
 
-extern void reset_cursor (void);
+#endif
