@@ -1,6 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -13,17 +16,25 @@
  *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"		/* intptr_t */
+#endif
+#include "w_digitize.h"
+
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+
 #include "figx.h"
 #include "resources.h"
-#include "object.h"
-#include "mode.h"
-#include "w_digitize.h"
-#include "w_indpanel.h"
+#include "w_canvas.h"
 #include "w_setup.h"
 #include "w_util.h"
 
-#include "w_canvas.h"
 
 /* LOCAL */
 
@@ -64,6 +75,7 @@ static	Boolean	digitize_append_save;
 void
 popup_digitize_panel(Widget w)
 {
+	(void)w;
 	DeclareArgs(2);
 	char	   *tmpstr;
 
@@ -98,6 +110,7 @@ create_digitize_panel(void)
 	NextArg(XtNy, yposn+50);
 	NextArg(XtNtitle, "Xfig: Export menu");
 	NextArg(XtNcolormap, tool_cm);
+	NextArg(XtNinput, True);
 	digitize_popup = XtCreatePopupShell("digitize_popup",
 					  transientShellWidgetClass,
 					  tool, Args, ArgCount);
@@ -111,6 +124,7 @@ create_digitize_panel(void)
 	/* Title */
 
 	FirstArg(XtNlabel, "Digitize to file(s)");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
@@ -122,6 +136,7 @@ create_digitize_panel(void)
 	/* File prefix */
 
 	FirstArg(XtNlabel, "Filename prefix");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNvertDistance, 8);
 	NextArg(XtNborderWidth, 0);
@@ -147,13 +162,15 @@ create_digitize_panel(void)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainRight);
-	below = digitize_file_prefix = XtCreateManagedWidget("file_prefix_entry",
-					    asciiTextWidgetClass, digitize_panel,
-					    Args, ArgCount);
+	NextArg(XtNinternational, appres.international);
+	below = digitize_file_prefix =
+		XtCreateManagedWidget("file_prefix_entry", asciiTextWidgetClass,
+			digitize_panel, Args, ArgCount);
         XtOverrideTranslations(digitize_file_prefix,
                            XtParseTranslationTable(text_translations));
 
 	FirstArg(XtNlabel, "     Starting #");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -178,6 +195,7 @@ create_digitize_panel(void)
 	/* label for suffix */
 
 	FirstArg(XtNlabel, "Filename suffix");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -202,13 +220,15 @@ create_digitize_panel(void)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainRight);
-	below = digitize_file_suffix = XtCreateManagedWidget("file_suffix_entry",
-					    asciiTextWidgetClass, digitize_panel,
-					    Args, ArgCount);
+	NextArg(XtNinternational, appres.international);
+	below = digitize_file_suffix =
+		XtCreateManagedWidget("file_suffix_entry", asciiTextWidgetClass,
+				digitize_panel, Args, ArgCount);
         XtOverrideTranslations(digitize_file_suffix,
                            XtParseTranslationTable(text_translations));
 
 	FirstArg(XtNlabel, "        Example");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -219,6 +239,7 @@ create_digitize_panel(void)
 					digitize_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, "                        ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNborderWidth, 0);
@@ -230,6 +251,7 @@ create_digitize_panel(void)
 					digitize_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, "  For each line");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -265,6 +287,7 @@ create_digitize_panel(void)
 	/* label - "Append to file" */
 
 	FirstArg(XtNlabel, "Append to file ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, appendtoggle);
@@ -300,6 +323,7 @@ create_digitize_panel(void)
 	/* label - "Create new file" */
 
 	FirstArg(XtNlabel, "Create new file");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, newfiletoggle);
@@ -313,6 +337,7 @@ create_digitize_panel(void)
 	/* Done button */
 
 	FirstArg(XtNlabel, "Done");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -327,6 +352,7 @@ create_digitize_panel(void)
 	/* Cancel button */
 
 	FirstArg(XtNlabel, "Cancel");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNfromHoriz, done_but);
@@ -385,6 +411,7 @@ digitize_panel_cancel(void)
 static void
 update_example(Widget w, XtPointer info, XtPointer dum)
 {
+    (void)w; (void)info; (void)dum;
     DeclareArgs(2);
     char	*prefix, *suffix;
     char	 example[100];
@@ -405,6 +432,7 @@ update_example(Widget w, XtPointer info, XtPointer dum)
 static void
 switch_file_mode(Widget w, XtPointer closure, XtPointer call_data)
 {
+    (void)closure; (void)call_data;
     DeclareArgs(5);
     Boolean	    state;
     intptr_t	    which;
