@@ -1,9 +1,11 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1990 by Digital Equipment Corporation. All Rights Reserved.
- * Parts Copyright (c) 1991 by Paul King
  * Parts Copyright (c) 1990 by Digital Equipment Corporation
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,6 +16,11 @@
  * party to do so, with the only requirement being that the above copyright
  * and this permission notice remain intact.
  *
+ *
+ * Part of the code in this file is taken from xdir,
+ * an X-based directory browser.
+ *
+ *
  * Original xdir code:
  *
  *	Created: 13 Aug 88
@@ -46,33 +53,40 @@
  * used in advertising or publicity pertaining to distribution of the
  * software without specific, written prior permission.
  *
- *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"		/* restrict */
+#endif
+#include "w_dir.h"
+
+#include <X11/StringDefs.h>
+#include <X11/Xlib.h>
+
+#include <errno.h>
+#include <pwd.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include "dirstruct.h"
+
 #include "figx.h"
 #include "resources.h"
 #include "mode.h"
+#include "f_util.h"
+#include "u_error.h"
 #include "w_browse.h"
-#include "w_dir.h"
+#include "w_cursor.h"
 #include "w_drawprim.h"		/* for max_char_height */
 #include "w_export.h"
 #include "w_file.h"
-#include "w_indpanel.h"
 #include "w_listwidget.h"
 #include "w_msgpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
 
-#include "object.h"
-#include "f_util.h"
-#include "w_cursor.h"
-#include "dirstruct.h"
 
 static char	CurrentSelectionName[PATH_MAX];
-static int	file_entry_cnt, dir_entry_cnt;
-static char   **file_list, **dir_list;
-static char   **filelist, **dirlist;
 static char    *dirmask;
 
 static Widget	hidden;
@@ -85,10 +99,13 @@ static void	SetDir(Widget widget, XEvent *event, String *params,
 				Cardinal *num_params);
 static void	ParentDir(Widget w, XEvent *event, String *params,
 				Cardinal *num_params);
-static void	CallbackRescan(Widget widget, XtPointer closure,
-				XtPointer call_data);
-static Boolean	MakeFileList(char *dir_name, char *mask, char ***_dirs,
-				char ***files);
+static Boolean	MakeFileList(char *dir_name, char *mask, char ***dirlist,
+				char ***filelist);
+static void	parseuserpath(const char *restrict path,
+				char *restrict longpath);
+
+static int	wild_match (char *string, char *pattern);
+static void	NewList (Widget listwidget, String *list);
 
 /* Static variables */
 
@@ -122,12 +139,13 @@ static XtActionsRec actionTable[] = {
  */
 
 
-int wild_match (char *string, char *pattern);
-void NewList (Widget listwidget, String *list);
 
 void
 FileSelected(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)client_data;
+
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
     strcpy(CurrentSelectionName, ret_struct->string);
@@ -140,7 +158,7 @@ FileSelected(Widget w, XtPointer client_data, XtPointer call_data)
 	XawTextSetInsertionPoint(file_selfile, strlen(CurrentSelectionName));
 	/* and show a preview of the figure in the preview canvas */
 	preview_figure(CurrentSelectionName, file_popup, preview_widget,
-			preview_size, preview_port_pixmap, preview_land_pixmap);
+			preview_size);
     } else if (export_up) {
 	SetValues(exp_selfile);
 	XawTextSetInsertionPoint(exp_selfile, strlen(CurrentSelectionName));
@@ -159,6 +177,9 @@ FileSelected(Widget w, XtPointer client_data, XtPointer call_data)
 void
 DirSelected(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)client_data;
+
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
     strcpy(CurrentSelectionName, ret_struct->string);
@@ -168,6 +189,8 @@ DirSelected(Widget w, XtPointer client_data, XtPointer call_data)
 void
 ShowHidden(Widget w, XtPointer client_data, XtPointer ret_val)
 {
+	(void)w; (void)client_data; (void)ret_val;
+
     show_hidden = !show_hidden;
     FirstArg(XtNlabel, show_hidden? "Hide Hidden": "Show Hidden");
     SetValues(hidden);
@@ -188,6 +211,7 @@ update_file_export_dir(const char *restrict dir)
 void
 GoHome(Widget w, XtPointer client_data, XtPointer ret_val)
 {
+	(void)w; (void)client_data; (void)ret_val;
     char	    dir[PATH_MAX];
 
     parseuserpath("~",dir);
@@ -215,6 +239,8 @@ GoHome(Widget w, XtPointer client_data, XtPointer ret_val)
 void
 SetDir(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 {
+	(void)widget; (void)event; (void)params; (void)num_params;
+    char	longdir[PATH_MAX];
     char	   *ndir;
 
     /* get the string from the widget */
@@ -231,42 +257,27 @@ SetDir(Widget widget, XEvent *event, String *params, Cardinal *num_params)
     }
     /* if there is a ~ in the directory, parse the username */
     if (ndir[0]=='~') {
-	char longdir[PATH_MAX];
 	parseuserpath(ndir,longdir);
 	ndir=longdir;
     }
     DoChangeDir(ndir);
 }
 
-/* make the full path from ~/partialpath */
-void parseuserpath(char *path, char *longpath)
+/*
+ * make the full path from ~/partialpath
+ * Parse ~somewhere as ~/somewhere, do not go to other user's home directories.
+ */
+static void
+parseuserpath(const char *restrict path, char *restrict longpath)
 {
-    char	  *p;
-    struct passwd *who;
-
-    /* this user's home */
-    if (strlen(path)==1 || path[1]=='/') {
-	strcpy(longpath,getenv("HOME"));
-	if (strlen(path)==1)		/* nothing after the ~, we have the full path */
-		return;
-	strcat(longpath,&path[1]);	/* append the rest of the path */
+	/* parseuserpath is called with  path[0] == '~' */
+	strcpy(longpath, getenv("HOME"));
+	if (strlen(path) == 1)
+		return;		/* nothing after the ~, we have the full path */
+	if (path[1] != '/')
+		strcat(longpath, "/");
+	strcat(longpath, &path[1]);	/* append the rest of the path */
 	return;
-    }
-    /* another user name after ~ */
-    strcpy(longpath,&path[1]);
-    p=strchr(longpath,'/');
-    if (p)
-	    *p='\0';
-    who = getpwnam(longpath);
-    if (!who) {
-	file_msg("No such user: %s",longpath);
-	strcpy(longpath,path);
-    } else {
-	strcpy(longpath,who->pw_dir);
-	p=strchr(path,'/');
-	if (p)
-		strcat(longpath,p);	/* attach stuff after the / */
-    }
 }
 
 
@@ -276,17 +287,17 @@ void parseuserpath(char *path, char *longpath)
 void
 create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside, Widget *ret_below, Widget *mask_w, Widget *dir_w, Widget *flist_w, Widget *dlist_w, int file_width, Boolean file_panel)
 {
+	(void)file_exp;
     Widget	    w,dir_alt,home;
     Widget	    file_viewport;
     Widget	    dir_viewport;
     XFontStruct	   *temp_font;
     int		    char_ht,char_wd;
     char	   *dir;
+    char	  **dirlist, **filelist;
 
-    dir_entry_cnt = NENTRIES;
-    file_entry_cnt = NENTRIES;
-    filelist = (char **) calloc(file_entry_cnt, sizeof(char *));
-    dirlist = (char **) calloc(dir_entry_cnt, sizeof(char *));
+    /* Initialize _filelist and _dirlist */
+    MakeFileList(NULL, NULL, NULL, NULL);
 
     if (browse_up) {
 	get_directory(cur_browse_dir);
@@ -304,6 +315,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     } else {
 	FirstArg(XtNlabel, "     Existing");
     }
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNtop, XtChainTop);
@@ -346,6 +358,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     /* label for filename mask */
 
     FirstArg(XtNlabel, "Filename Mask");
+    NextArg(XtNinternational, False);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNfromVert, file_viewport);
     NextArg(XtNtop, XtChainBottom);
@@ -358,6 +371,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     /* text widget for the filename mask */
 
     FirstArg(XtNeditType, XawtextEdit);
+    NextArg(XtNinternational, appres.international);
     NextArg(XtNleftMargin, 4);
     NextArg(XtNheight, char_ht * 2);
     NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
@@ -377,10 +391,11 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
 
     FirstArg(XtNstring, &dirmask);
     GetValues(*mask_w);
-    if (MakeFileList(dir, dirmask, &dir_list, &file_list) == False)
+    if (MakeFileList(dir, dirmask, &dirlist, &filelist) == False)
 	file_msg("No files in directory?");
 
     FirstArg(XtNlabel, "  Current Dir");
+    NextArg(XtNinternational, False);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNfromVert, *mask_w);
     NextArg(XtNtop, XtChainBottom);
@@ -391,6 +406,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
 			      parent, Args, ArgCount);
 
     FirstArg(XtNstring, dir);
+    NextArg(XtNinternational, appres.international);
     NextArg(XtNleftMargin, 4);
     NextArg(XtNinsertPosition, strlen(dir));
     NextArg(XtNheight, char_ht * 2);
@@ -412,6 +428,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     /* directory alternatives */
 
     FirstArg(XtNlabel, "  Directories");
+    NextArg(XtNinternational, False);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNfromVert, *dir_w);
     NextArg(XtNtop, XtChainBottom);
@@ -423,6 +440,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
 
     /* put a Home button to the left of the list of directories */
     FirstArg(XtNlabel, "Home");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, dir_alt);
     NextArg(XtNfromHoriz, dir_alt);
     NextArg(XtNhorizDistance, -(char_wd * 5));
@@ -437,6 +455,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     /* put a button for showing/hiding hidden files below the Home button */
 
     FirstArg(XtNlabel, show_hidden? "Hide Hidden": "Show Hidden");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, home);
     NextArg(XtNfromHoriz, dir_alt);
     NextArg(XtNhorizDistance, (int) -(char_wd * 10.5));
@@ -463,7 +482,8 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     dir_viewport = XtCreateManagedWidget("dirvport", viewportWidgetClass,
 					 parent, Args, ArgCount);
 
-    FirstArg(XtNlist, file_list);
+    FirstArg(XtNlist, filelist);
+    NextArg(XtNinternational, appres.international);
     /* for file panel use only one column */
     if (file_panel) {
 	NextArg(XtNdefaultColumns, 1);
@@ -475,7 +495,8 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     XtOverrideTranslations(*flist_w,
 			   XtParseTranslationTable(list_panel_translations));
 
-    FirstArg(XtNlist, dir_list);
+    FirstArg(XtNlist, dirlist);
+    NextArg(XtNinternational, appres.international);
     *dlist_w = XtCreateManagedWidget("dir_list_panel", figListWidgetClass,
 				     dir_viewport, Args, ArgCount);
     XtOverrideTranslations(*dlist_w,
@@ -489,6 +510,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     }
 
     FirstArg(XtNlabel, "Rescan");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, dir_viewport);
     NextArg(XtNvertDistance, 15);
     NextArg(XtNborderWidth, INTERNAL_BW);
@@ -500,7 +522,7 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     NextArg(XtNright, XtChainLeft);
     w = XtCreateManagedWidget("rescan", commandWidgetClass, parent,
 			      Args, ArgCount);
-    XtAddCallback(w, XtNcallback, CallbackRescan, (XtPointer) NULL);
+    XtAddCallback(w, XtNcallback, (XtCallbackProc)Rescan, NULL);
 
     /* install accelerators so they can be used from each window */
     XtInstallAccelerators(parent, w);
@@ -513,46 +535,132 @@ create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside
     return;
 }
 
+#ifdef FREEMEM
+
+static void
+free_list(char **restrict list, const int count)
+{
+	char	**entry;
+	for (entry = list; entry - list < count; ++entry)
+		if (*entry)
+			free(*entry);
+	free(list);
+}
+
+static void
+freelists(void)
+{
+	char	**dummy;
+	(void)MakeFileList(NULL, NULL, &dummy, NULL);
+}
+
+#endif /* FREEMEM */
+
 /* Function:	SPComp() compares two string pointers for qsort().
  * Arguments:	s1, s2: strings to be compared.
  * Returns:	Value of strcmp().
- * Notes:
+ * Notes:	man qsort(3) shows the cast in the EXAMPLES section.
  */
 
 static int
-SPComp(char **s1, char **s2)
+SPComp(const void *p1, const void *p2)
 {
-    return (strcmp(*s1, *s2));
+    return (strcmp(*(const char **)p1, *(const char **)p2));
 }
 
+static void
+replace_entry(char **restrict entry, const char *new)
+{
+	if (!strcmp(*entry, new))
+		return;
+	if (*entry)
+		free(*entry);
+	*entry = strdup(new);
+}
+
+/*
+ * Double the size of the file/directory list.
+ * Return the newly allocated list and the new size of the list.
+ */
+static void
+extend_list(char ***restrict list, char ***restrict current, int *num)
+{
+	char		**c;
+	const int	fac = 2;
+
+	*list = realloc(*list, *num * fac * sizeof(char *));
+
+	/* zero the string pointers to know when free() is possible */
+	for (c = *list + *num; c < *list + fac * *num; ++c)
+		*c = NULL;
+	*current = *list + *num - 1;
+	*num *= fac;
+}
+
+
 #define MAX_MASKS	20
 #define MAX_MASK_LEN	64
+#define _STRINGIFY(X)	#X
+#define STRINGIFY(X)	_STRINGIFY(X)
 
-Boolean
-MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
+/*
+ * Use MakeFileList(NULL, NULL, NULL, NULL) to allocate memory for the
+ * statically allocated string arrays _dirlist and _filelist.
+ * MakeFileList(NULL, NULL, x, NULL) will free _dirlist and _filelist.
+ */
+static Boolean
+MakeFileList(char *dir_name, char *mask, char ***dirlist, char ***filelist)
 {
+    static char	**_filelist = NULL;
+    static char	**_dirlist = NULL;
+    static int	file_entry_cnt = NENTRIES;
+    static int	dir_entry_cnt = NENTRIES;
     DIR		  *dirp;
     DIRSTRUCT	  *dp;
     char	 **cur_file, **cur_directory;
-    char	 **last_file, **last_dir;
     int		   nmasks,i;
     char	  *wild[MAX_MASKS],*cmask;
     char	buf[MAX_MASK_LEN];
+    const char	delims[] = " \t";
     Boolean	   match;
 
+    if (dir_name == NULL) {
+#ifdef FREEMEM
+	/* at exit, call with MakeFileList(NULL, NULL, x, NULL) */
+	if (dirlist != NULL) {
+		if (_filelist)
+			free_list(_filelist, file_entry_cnt);
+		if(_dirlist)
+			free_list(_dirlist, dir_entry_cnt);
+		return True;
+	}
+#endif
+	if (_filelist)	/* already initialized */
+	    return True;
+
+	_filelist = calloc(file_entry_cnt, sizeof(char *));
+	_dirlist = calloc(dir_entry_cnt, sizeof(char *));
+	if (!_filelist || !_dirlist) {
+		put_msg("No more memory - QUITTING!");
+		fputs("Available memory exceeded - quitting.\n", stderr);
+		emergency_quit(False);
+	}
+#ifdef FREEMEM
+	atexit(freelists);
+#endif
+	return True;
+    }
+
     set_temp_cursor(wait_cursor);
-    cur_file = filelist;
-    cur_directory = dirlist;
-    last_file = filelist + file_entry_cnt - 1;
-    last_dir = dirlist + dir_entry_cnt - 1;
+    cur_file = _filelist;
+    cur_directory = _dirlist;
 
     dirp = opendir(dir_name);
     if (dirp == NULL) {
 	reset_cursor();
-	*file_list = filelist;
-	*file_list[0]="";
-	*dir_list = dirlist;
-	*dir_list[0]="..";
+	replace_entry(_filelist, "");
+	replace_entry(_dirlist, "..");
+	*filelist = _filelist;
 	return False;
     }
     /* process any events to ensure cursor is set to wait_cursor */
@@ -571,10 +679,13 @@ MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
     }
     if (*cmask == '\0')
 	strcpy(cmask,"*");
-    wild[0] = strtok(cmask," \t");
+    wild[0] = strtok(cmask, delims);
     nmasks = 1;
-    while ((wild[nmasks]=strtok((char*) NULL, " \t")) && nmasks < MAX_MASKS)
+    while (nmasks < MAX_MASKS && (wild[nmasks]=strtok((char*)NULL, delims)))
 	nmasks++;
+    if (nmasks == MAX_MASKS && strtok(NULL, delims))
+	file_msg("Maximum number of filename masks exceeded, only first "
+				STRINGIFY(MAX_MASKS) " considered.");
     for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 	/* skip over '.' (current dir) */
 	if (!strcmp(dp->d_name, "."))
@@ -584,14 +695,11 @@ MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
 	    continue;
 
 	if (IsDirectory(dir_name, dp->d_name)) {
+	    if (*cur_directory)
+		    free(*cur_directory);
 	    *cur_directory++ = strdup(dp->d_name);
-	    if (cur_directory == last_dir) {	/* out of space, make more */
-		dirlist = (char **) realloc(dirlist,
-					2 * dir_entry_cnt * sizeof(char *));
-		cur_directory = dirlist + dir_entry_cnt - 1;
-		dir_entry_cnt = 2 * dir_entry_cnt;
-		last_dir = dirlist + dir_entry_cnt - 1;
-	    }
+	    if (cur_directory == _dirlist + dir_entry_cnt - 1)
+		extend_list(&_dirlist, &cur_directory, &dir_entry_cnt);
 	} else {
 	    /* check if matches regular expression */
 	    match=False;
@@ -606,28 +714,34 @@ MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
 		continue;	/* no, do next */
 	    if (wild[i][0] == '*' && dp->d_name[0] == '.')
 		continue;	/* skip files with leading . */
+	    if (*cur_file)
+		    free(*cur_file);
 	    *cur_file++ = strdup(dp->d_name);
-	    if (cur_file == last_file) {	/* out of space, make more */
-		filelist = (char **) realloc(filelist,
-				       2 * file_entry_cnt * sizeof(char *));
-		cur_file = filelist + file_entry_cnt - 1;
-		file_entry_cnt = 2 * file_entry_cnt;
-		last_file = filelist + file_entry_cnt - 1;
+	    if (cur_file == _filelist + file_entry_cnt - 1) {
+		extend_list(&_filelist, &cur_file, &file_entry_cnt);
 	    }
 	}
     }
-    *cur_file = NULL;
-    *cur_directory = NULL;
-    if (cur_file != filelist)
-	qsort(filelist, cur_file - filelist, sizeof(char *), (int(*)())SPComp);
-    if (cur_directory != dirlist)
-	qsort(dirlist, cur_directory - dirlist, sizeof(char *), (int(*)())SPComp);
-    *file_list = filelist;
-    *dir_list = dirlist;
-    reset_cursor();
-    closedir(dirp);
+    if (*cur_file) {
+	    free(*cur_file);
+	    *cur_file = NULL;
+    }
+    if (*cur_directory) {
+	    free(*cur_directory);
+	    *cur_directory = NULL;
+    }
     if (cmask != buf)
-	    free(cmask);	/* free copy of mask */
+	    free(cmask);
+    closedir(dirp);
+
+    *filelist = _filelist;
+    *dirlist = _dirlist;
+
+    if (cur_file != _filelist)
+	qsort(_filelist, cur_file - _filelist, sizeof(char *), SPComp);
+    if (cur_directory != _dirlist)
+	qsort(_dirlist, cur_directory - _dirlist, sizeof(char *), SPComp);
+    reset_cursor();
     return True;
 }
 
@@ -640,6 +754,7 @@ MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
 static void
 ParentDir(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+	(void)w; (void)event; (void)params; (void)num_params;
     DoChangeDir("..");
 }
 
@@ -655,77 +770,61 @@ ParentDir(Widget w, XEvent *event, String *params, Cardinal *num_params)
 void
 DoChangeDir(char *dir)
 {
-    char	   *p;
-    char	    ndir[PATH_MAX];
+	char	*abs_path = NULL;
+	char	**filelist, **dirlist;
 
-
-    if (browse_up) {
-	strcpy(ndir, cur_browse_dir);
-    } else if (file_up) {
-	strcpy(ndir, cur_file_dir);
-    } else if (export_up) {
-	strcpy(ndir, cur_export_dir);
-    }
-    if (dir != NULL && dir[0] != '/') { /* relative path, prepend current dir */
-	if (dir[strlen(dir) - 1] == '/')
-	    dir[strlen(dir) - 1] = '\0';
-	if (strcmp(dir, "..")==0) {	/* Parent directory. */
-	    if (*ndir == '\0')
-		return;			/* no current directory, */
-					/* can't do anything unless absolute path */
-	    p = strrchr(ndir, '/');
-	    *p = EOS;
-	    if (ndir[0] == EOS)
-		strcpy(ndir, "/");
-	} else if (strcmp(dir, ".")!=0) {
-	    if (strcmp(ndir, "/"))	/* At the root already */
-		strcat(ndir, "/");
-	    strcat(ndir, dir);
+	/* resolve the path given in dir to abs_path */
+	if (!(abs_path = realpath(dir && *dir ? dir : ".", NULL))) {
+		/* realpath knows about the current directory and
+		   correctly resolves relative paths. */
+		file_msg("Cannot infer absolute path for %s: %s",
+				dir && *dir ? dir : "." , strerror(errno));
+		return;
 	}
-    } else {
-	strcpy(ndir, dir);		/* abs path copy to ndir */
-    }
-    if (change_directory(ndir) != 0 ) {
-	return;				/* some problem, return */
-    } else if (MakeFileList(ndir, dirmask, &dirlist, &filelist) == False) {
-	file_msg("Unable to list directory %s", ndir);
-	return;
-    }
 
-    FirstArg(XtNstring, ndir);
-    /* update the current directory and file/dir list widgets */
-    if (browse_up) {
-	SetValues(browse_dir);
-	strcpy(cur_browse_dir,ndir);	/* update global var */
-	XawTextSetInsertionPoint(browse_dir, strlen(ndir));
-	NewList(browse_flist, filelist);
-	NewList(browse_dlist, dirlist);
-    } else if (file_up) {
-	SetValues(file_dir);
-	update_file_export_dir(ndir);
-	XawTextSetInsertionPoint(file_dir, strlen(ndir));
-	NewList(file_flist,filelist);
-	NewList(file_dlist,dirlist);
-    } else if (export_up) {
-	SetValues(exp_dir);
-	strcpy(cur_export_dir,ndir);	/* update global var */
-	XawTextSetInsertionPoint(exp_dir, strlen(ndir));
-	NewList(exp_flist, filelist);
-	NewList(exp_dlist, dirlist);
-    }
-    CurrentSelectionName[0] = '\0';
+	/* change to abs_path */
+	if (abs_path && change_directory(abs_path)) {
+		free(abs_path);
+		/* change_directory already delivered an error message */
+		return;
+	}
+
+	if (MakeFileList(abs_path, dirmask, &dirlist, &filelist) == False) {
+		file_msg("Unable to list directory %s", abs_path);
+		free(abs_path);
+		return;
+	}
+
+	/* update the current directory and file/dir list widgets */
+	FirstArg(XtNstring, abs_path);
+	if (browse_up) {
+		SetValues(browse_dir);
+		strcpy(cur_browse_dir, abs_path);	/* update global var */
+		XawTextSetInsertionPoint(browse_dir, strlen(abs_path));
+		NewList(browse_flist, filelist);
+		NewList(browse_dlist, dirlist);
+	} else if (file_up) {
+		SetValues(file_dir);
+		update_file_export_dir(abs_path);
+		XawTextSetInsertionPoint(file_dir, strlen(abs_path));
+		NewList(file_flist, filelist);
+		NewList(file_dlist, dirlist);
+	} else if (export_up) {
+		SetValues(exp_dir);
+		strcpy(cur_export_dir, abs_path);	/* update global var */
+		XawTextSetInsertionPoint(exp_dir, strlen(abs_path));
+		NewList(exp_flist, filelist);
+		NewList(exp_dlist, dirlist);
+	}
+	free(abs_path);
+	CurrentSelectionName[0] = '\0';
 }
 
 void
-CallbackRescan(Widget widget, XtPointer closure, XtPointer call_data)
-{
-     Rescan(0, 0, 0, 0);
-}
-
-void
-Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params)
+Rescan(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     char	*dir;
+    char	**filelist, **dirlist;
 
     /*
      * get the mask string from the File or Export mask widget and put in
@@ -739,9 +838,10 @@ Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
 	strcpy(cur_browse_dir,dir);	/* save in global var */
-	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
-	NewList(browse_flist, file_list);
-	NewList(browse_dlist, dir_list);
+	(void) MakeFileList(dir, dirmask, &dirlist, &filelist);
+	/* List Widgets need to have the string array remain available. */
+	NewList(browse_flist, filelist);
+	NewList(browse_dlist, dirlist);
     } else if (file_up) {
 	FirstArg(XtNstring, &dirmask);
 	GetValues(file_mask);
@@ -750,9 +850,9 @@ Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
 	update_file_export_dir(dir);
-	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
-	NewList(file_flist,file_list);
-	NewList(file_dlist,dir_list);
+	(void) MakeFileList(dir, dirmask, &dirlist, &filelist);
+	NewList(file_flist,filelist);
+	NewList(file_dlist,dirlist);
     } else if (export_up) {
 	FirstArg(XtNstring, &dirmask);
 	GetValues(exp_mask);
@@ -761,13 +861,14 @@ Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
 	strcpy(cur_export_dir,dir);	/* save in global var */
-	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
-	NewList(exp_flist, file_list);
-	NewList(exp_dlist, dir_list);
+	(void) MakeFileList(dir, dirmask, &dirlist, &filelist);
+	NewList(exp_flist, filelist);
+	NewList(exp_dlist, dirlist);
     }
 }
 
-void NewList(Widget listwidget, String *list)
+static void
+NewList(Widget listwidget, String *list)
 {
 	/* install the new list */
 	XawListChange(listwidget, list, 0, 0, True);
@@ -868,8 +969,9 @@ wild_match(char *string, char *pattern)
     for (; *pattern; string++, pattern++)
 	switch (*pattern) {
 	case '\\':
-	    /* Literal match with following character; fall through. */
+	    /* Literal match with following character */
 	    pattern++;
+	    /* intentionally fall through. */
 	default:
 	    if (*string != *pattern)
 		return 0;
