@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2018 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,31 +16,41 @@
  *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"		/* intptr_t */
+#endif
+#include "w_export.h"
+
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <X11/StringDefs.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Xatom.h>		/* XA_STRING */
+
 #include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
-#include "e_edit.h"
 #include "f_util.h"
-#include "u_print.h"
-#include "w_dir.h"
-#include "w_drawprim.h"		/* for max_char_height */
-#include "w_export.h"
-#include "w_file.h"
-#include "w_indpanel.h"
-#include "w_msgpanel.h"
-#include "w_print.h"
-#include "w_setup.h"
-#include "w_util.h"
-#include "w_icons.h"
-
 #include "u_bound.h"
-#include "u_redraw.h"
+#include "u_colors.h"
+#include "u_print.h"
 #include "w_canvas.h"
 #include "w_cmdpanel.h"
 #include "w_color.h"
 #include "w_cursor.h"
+#include "w_dir.h"
+#include "w_drawprim.h"		/* for max_char_height */
+#include "w_file.h"
+#include "w_msgpanel.h"
+#include "w_print.h"
+#include "u_redraw.h"
+#include "w_setup.h"
+#include "w_util.h"
+#include "xfig_math.h"
+
 
 /* EXPORTS */
 
@@ -200,6 +210,8 @@ export_panel_dismiss(void)
 static void
 export_panel_cancel(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     export_panel_dismiss();
 }
 
@@ -233,7 +245,7 @@ do_export(Widget w)
 	F_line     *l;
 	char	    transparent[10], backgrnd[10], grid[80];
 	char	    msg[200];
-	int	    transp;
+	int	    transp = TRANSP_NONE;
 	int	    border;
 	Boolean	    use_transp_backg;
 
@@ -332,17 +344,12 @@ do_export(Widget w)
 	    /* make background transparent */
 	    if (transp == TRANSP_BACKGROUND) {
 		use_transp_backg = True;
-		sprintf(transparent,"#%02x%02x%02x",
-				x_bg_color.red>>8,
-				x_bg_color.green>>8,
-				x_bg_color.blue>>8);
+		sprintf(transparent,"#%02x%02x%02x", getred(CANVAS_BG)>>8,
+				getgreen(CANVAS_BG)>>8, getblue(CANVAS_BG)>>8);
 	    /* make other color transp */
 	    } else {
-		XColor  col;
-		col.pixel = x_color(transp);
-		XQueryColor(tool_d, tool_cm, &col);
-		sprintf(transparent,"#%02x%02x%02x",
-				col.red>>8, col.green>>8, col.blue>>8);
+		sprintf(transparent,"#%02x%02x%02x", getred(transp)>>8,
+				getgreen(transp)>>8, getblue(transp)>>8);
 		use_transp_backg = False;
 	    }
 	}
@@ -355,7 +362,7 @@ do_export(Widget w)
 	appres.export_margin = border;
 
 	/* call fig2dev to export the file */
-	if (print_to_file(fval, xoff, yoff, backgrnd,
+	if (print_export(fval, xoff, yoff, backgrnd,
 				transp == TRANSP_NONE? NULL: transparent,
 				use_transp_backg, border, grid) == 0) {
 		FirstArg(XtNlabel, fval);
@@ -372,6 +379,7 @@ do_export(Widget w)
 static void
 preview_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
 	ptr_int		data = {client_data};
 
 	if (preview_type != data.val) {
@@ -384,6 +392,8 @@ preview_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 orient_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	ptr_int		data = {client_data};
 
 	if (appres.landscape != data.val) {
@@ -398,6 +408,7 @@ orient_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 just_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
 	FirstArg(XtNlabel, XtName(w));
 	SetValues(export_just_panel);
 	/* change print justification if it exists */
@@ -409,6 +420,8 @@ just_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 papersize_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	intptr_t papersize = (intptr_t) client_data;
 
 	FirstArg(XtNlabel, paper_sizes[papersize].fname);
@@ -424,6 +437,8 @@ papersize_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 multiple_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	intptr_t multiple = (intptr_t) client_data;
 
 	FirstArg(XtNlabel, multiple_pages[multiple]);
@@ -458,6 +473,8 @@ multiple_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 overlap_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	intptr_t overlap = (intptr_t) client_data;
 
 	FirstArg(XtNlabel, overlap_pages[overlap]);
@@ -503,6 +520,7 @@ set_postscript_pdf_options(void)
 static void
 lang_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
 	ptr_int		new_lang = {client_data};
 
 	FirstArg(XtNlabel, XtName(w));
@@ -558,6 +576,8 @@ set_export_mask(int lang)
 static void
 smooth_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	intptr_t new_smooth = (intptr_t) client_data;
 	appres.smooth_factor = new_smooth == 0 ? 1 : new_smooth*2;
 	FirstArg(XtNlabel, smooth_choices[new_smooth]);
@@ -569,6 +589,7 @@ smooth_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 background_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
 	ptr_int	data = {client_data};
 	Pixel	    bgcolor, fgcolor;
 
@@ -596,6 +617,7 @@ background_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 transp_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
 	Pixel	    bgcolor, fgcolor;
 	ptr_int	data = {client_data};
 
@@ -621,6 +643,8 @@ void
 export_grid_minor_select(Widget w,
 			XtPointer new_grid_choice, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	char	buf[MAX_GRID_STRLEN];
 	char	*val;
 	ptr_int	grid = {new_grid_choice};
@@ -645,6 +669,8 @@ void
 export_grid_major_select(Widget w,
 			XtPointer new_grid_choice, XtPointer call_data)
 {
+	(void)w;
+	(void)call_data;
 	char	buf[MAX_GRID_STRLEN];
 	char	*val;
 	ptr_int	grid = {new_grid_choice};
@@ -914,6 +940,7 @@ get_magnif(void)
 static void
 update_mag(Widget widget, Widget *item, int *event)
 {
+	(void)widget; (void)item; (void)event;
     char	   *buf;
 
     buf = panel_get_value(export_mag_text);
@@ -1036,6 +1063,7 @@ popup_export_panel(Widget w)
 static void
 exp_xoff_unit_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
     FirstArg(XtNlabel, XtName(w));
     SetValues(exp_xoff_unit_panel);
     xoff_unit_setting = (intptr_t) client_data;
@@ -1044,6 +1072,7 @@ exp_xoff_unit_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 exp_yoff_unit_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
     FirstArg(XtNlabel, XtName(w));
     SetValues(exp_yoff_unit_panel);
     yoff_unit_setting = (intptr_t) client_data;
@@ -1053,6 +1082,8 @@ exp_yoff_unit_select(Widget w, XtPointer client_data, XtPointer call_data)
 static void /* XtCallbackProc */
 toggle_hpgl_pcl_switch(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure;
+	(void)call_data;
     Boolean	    state;
 
     /* check state of the toggle and set/remove checkmark */
@@ -1074,6 +1105,8 @@ toggle_hpgl_pcl_switch(Widget w, XtPointer closure, XtPointer call_data)
 static void /* XtCallbackProc */
 toggle_hpgl_font(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure;
+	(void)call_data;
     Boolean	    state;
 
     /* check state of the toggle and set/remove checkmark */
@@ -1095,6 +1128,8 @@ toggle_hpgl_font(Widget w, XtPointer closure, XtPointer call_data)
 static void /* XtCallbackProc */
 toggle_pdf_pagemode(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure;
+	(void)call_data;
 	Boolean state;
 
 	/* check state of the toggle and set/remove checkmark */
@@ -1115,6 +1150,7 @@ toggle_pdf_pagemode(Widget w, XtPointer closure, XtPointer call_data)
 
 void create_export_panel(Widget w)
 {
+	(void)w;
 	Widget		 beside, below;
 	Widget		 entry, papersize_menu;
 	XFontStruct	*temp_font;
@@ -1130,8 +1166,10 @@ void create_export_panel(Widget w)
 	FirstArg(XtNx, xposn+50);
 	NextArg(XtNy, yposn+50);
 	NextArg(XtNtitle, "Xfig: Export menu");
+	NextArg(XtNtitleEncoding, XA_STRING);
 	NextArg(XtNcolormap, tool_cm);
 	NextArg(XtNallowShellResize, True);
+	NextArg(XtNinput, True);
 	export_popup = XtCreatePopupShell("export_popup",
 					  transientShellWidgetClass,
 					  tool, Args, ArgCount);
@@ -1145,6 +1183,7 @@ void create_export_panel(Widget w)
 	/* The export language is first */
 
 	FirstArg(XtNlabel, "     Language");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
@@ -1154,6 +1193,7 @@ void create_export_panel(Widget w)
 					 export_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, lang_texts[cur_exp_lang]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, lang_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNtop, XtChainTop);
@@ -1172,6 +1212,7 @@ void create_export_panel(Widget w)
 	/* Magnification */
 
 	FirstArg(XtNlabel, "Magnification %");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, lang_panel);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1211,6 +1252,7 @@ void create_export_panel(Widget w)
 	   strncat(buf, "                                        ",39-i);
 
 	FirstArg(XtNlabel, buf);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, lang_panel);
 	NextArg(XtNfromHoriz, mag_spinner);
 	NextArg(XtNhorizDistance, 5);
@@ -1230,6 +1272,7 @@ void create_export_panel(Widget w)
 	/* the border margin and background color will appear depending on the export language */
 
 	FirstArg(XtNlabel, "Border Margin");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, layer_choice);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1253,6 +1296,7 @@ void create_export_panel(Widget w)
 	/* background color option for all bitmap export and PS/EPS */
 
 	FirstArg(XtNlabel, "Background");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, layer_choice);
 	NextArg(XtNfromHoriz, border_spinner);
 	NextArg(XtNhorizDistance, 8);
@@ -1265,6 +1309,7 @@ void create_export_panel(Widget w)
 					 export_panel, Args, ArgCount);
 
 	FirstArg(XtNfromVert, layer_choice);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, background_lab);
 	NextArg(XtNresize, False);
 	NextArg(XtNwidth, 80);
@@ -1286,6 +1331,7 @@ void create_export_panel(Widget w)
 
 	/* grid options */
 	FirstArg(XtNlabel, "Grid");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, border_lab);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1310,6 +1356,7 @@ void create_export_panel(Widget w)
 
 	/* label pagemode */
 	FirstArg(XtNlabel, "  Page mode");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_grid_label);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1337,6 +1384,7 @@ void create_export_panel(Widget w)
 
 	/* description to the right of the checkbox pagemode */
 	FirstArg(XtNlabel, "Crop to Bounding Box");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_grid_label);
 	NextArg(XtNfromHoriz, pdf_pagemode_checkbox);
 	NextArg(XtNborderWidth, 0);
@@ -1351,6 +1399,7 @@ void create_export_panel(Widget w)
 	/* choose a preview: none, ASCII, color tiff or monochrome tiff */
 	/* label preview*/
 	FirstArg(XtNlabel, "Add Preview");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_grid_label);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1362,6 +1411,7 @@ void create_export_panel(Widget w)
 
 	/* drop-down menu preview */
 	FirstArg(XtNlabel, preview_items[preview_type]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_grid_label);
 	NextArg(XtNfromHoriz, preview_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1391,6 +1441,7 @@ void create_export_panel(Widget w)
 					     export_panel, Args, ArgCount);
 	/* now a label */
 	FirstArg(XtNlabel, "Bitmap Options");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
@@ -1401,6 +1452,7 @@ void create_export_panel(Widget w)
 
 	/* Label for smoothing options */
 	FirstArg(XtNlabel, "Smoothing");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1412,6 +1464,7 @@ void create_export_panel(Widget w)
 
 	/* make a pulldown menu for smooth (No smoothing=1, Some=2, More=4) */
 	FirstArg(XtNlabel, smooth_choices[appres.smooth_factor==1? 0: appres.smooth_factor/2]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, smooth_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1429,6 +1482,7 @@ void create_export_panel(Widget w)
 	/* transparent color option for GIF export */
 
 	FirstArg(XtNlabel, "GIF Transparent color");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, smooth_menu_button);
 	NextArg(XtNborderWidth, 0);
@@ -1440,6 +1494,7 @@ void create_export_panel(Widget w)
 					 bitmap_form, Args, ArgCount);
 
 	FirstArg(XtNfromVert, below);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, transp_lab);
 	NextArg(XtNresize, False);
 	NextArg(XtNwidth, 80);
@@ -1461,6 +1516,7 @@ void create_export_panel(Widget w)
 
 	/* first label */
 	FirstArg(XtNlabel, "JPEG Image quality (%)");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, smooth_menu_button);
 	NextArg(XtNborderWidth, 0);
@@ -1496,19 +1552,9 @@ void create_export_panel(Widget w)
 	NextArg(XtNright, XtChainLeft);
 	postscript_form = XtCreateWidget("postscript_form", formWidgetClass,
 					     export_panel, Args, ArgCount);
-	/* now a label */
-/*	FirstArg(XtNlabel, "PostScript Options");
-	NextArg(XtNborderWidth, 0);
-	NextArg(XtNtop, XtChainTop);
-	NextArg(XtNbottom, XtChainTop);
-	NextArg(XtNleft, XtChainLeft);
-	NextArg(XtNright, XtChainLeft);
-	below = ps_form_label = XtCreateManagedWidget("postscript_label", labelWidgetClass,
-					postscript_form, Args, ArgCount);
-*/
 	/* paper size */
 	FirstArg(XtNlabel, " Paper Size");
-//	NextArg(XtNfromVert, below);
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
@@ -1518,7 +1564,7 @@ void create_export_panel(Widget w)
 					 postscript_form, Args, ArgCount);
 
 	FirstArg(XtNlabel, paper_sizes[appres.papersize].fname);
-//	NextArg(XtNfromVert, below);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, papersize_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNresizable, True);
@@ -1543,7 +1589,7 @@ void create_export_panel(Widget w)
 	/* Fit Page */
 
 	FirstArg(XtNlabel, "Fit to Page");
-//	NextArg(XtNfromVert, below);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, export_papersize_panel);
 	NextArg(XtNhorizDistance, 9);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1559,6 +1605,7 @@ void create_export_panel(Widget w)
 	/* Landscape/Portrait Orientation */
 
 	FirstArg(XtNlabel, "Orientation");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_papersize_panel);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1568,7 +1615,8 @@ void create_export_panel(Widget w)
 	orient_lab = XtCreateManagedWidget("orient_label", labelWidgetClass,
 					 postscript_form, Args, ArgCount);
 
-	FirstArg(XtNlabel, orient_items[appres.landscape]);
+	FirstArg(XtNlabel, orient_items[(int)appres.landscape]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_papersize_panel);
 	NextArg(XtNfromHoriz, orient_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1585,6 +1633,7 @@ void create_export_panel(Widget w)
 	/* Justification */
 
 	FirstArg(XtNlabel, "Justification");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_papersize_panel);
 	NextArg(XtNfromHoriz, export_orient_panel);
 	NextArg(XtNborderWidth, 0);
@@ -1596,6 +1645,7 @@ void create_export_panel(Widget w)
 					 postscript_form, Args, ArgCount);
 
 	FirstArg(XtNlabel, just_items[appres.flushleft? 1 : 0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_papersize_panel);
 	NextArg(XtNfromHoriz, just_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1613,6 +1663,7 @@ void create_export_panel(Widget w)
 	/* multiple/single page */
 
 	FirstArg(XtNlabel, "      Pages");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_orient_panel);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1623,6 +1674,7 @@ void create_export_panel(Widget w)
 					 postscript_form, Args, ArgCount);
 
 	FirstArg(XtNlabel, multiple_pages[appres.multiple? 1:0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_orient_panel);
 	NextArg(XtNfromHoriz, multiple_lab);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1640,6 +1692,7 @@ void create_export_panel(Widget w)
 	/* overlap/no-overlap for multiple page selection */
 
 	FirstArg(XtNlabel, overlap_pages[appres.overlap? 1:0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_orient_panel);
 	NextArg(XtNfromHoriz, export_multiple_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1657,6 +1710,7 @@ void create_export_panel(Widget w)
 	/* X/Y offset choices */
 
 	FirstArg(XtNlabel, "     Offset");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1665,11 +1719,12 @@ void create_export_panel(Widget w)
 	NextArg(XtNright, XtChainLeft);
 	exp_off_lab = XtCreateManagedWidget("export_offset_label", labelWidgetClass,
 				     postscript_form, Args, ArgCount);
-	FirstArg(XtNlabel, "X");
+	FirstArg(XtNlabel, "X = ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNfromHoriz, exp_off_lab);
 	NextArg(XtNhorizDistance, 5);
-	NextArg(XtNborderWidth, 1);
+	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
@@ -1677,6 +1732,7 @@ void create_export_panel(Widget w)
 	exp_xoff_lab = XtCreateManagedWidget("export_offset_lbl_x", labelWidgetClass,
 				     postscript_form, Args, ArgCount);
 	FirstArg(XtNwidth, 50);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNleftMargin, 4);
 	NextArg(XtNeditType, XawtextEdit);
 	NextArg(XtNstring, "0.0");
@@ -1692,6 +1748,7 @@ void create_export_panel(Widget w)
 	export_offset_x = XtCreateManagedWidget("export_offset_x", asciiTextWidgetClass,
 					     postscript_form, Args, ArgCount);
 	FirstArg(XtNfromHoriz, export_offset_x);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNtop, XtChainTop);
@@ -1704,11 +1761,12 @@ void create_export_panel(Widget w)
 	make_pulldown_menu(offset_unit_items, XtNumber(offset_unit_items),
 				-1, "", exp_xoff_unit_panel, exp_xoff_unit_select);
 
-	FirstArg(XtNlabel, "Y");
+	FirstArg(XtNlabel, "Y = ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, exp_xoff_unit_panel);
 	NextArg(XtNhorizDistance, 10);
 	NextArg(XtNfromVert, export_multiple_panel);
-	NextArg(XtNborderWidth, 1);
+	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
@@ -1716,6 +1774,7 @@ void create_export_panel(Widget w)
 	exp_yoff_lab = XtCreateManagedWidget("export_offset_lbl_y", labelWidgetClass,
 				     postscript_form, Args, ArgCount);
 	FirstArg(XtNwidth, 50);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNleftMargin, 4);
 	NextArg(XtNeditType, XawtextEdit);
 	NextArg(XtNstring, "0.0");
@@ -1731,6 +1790,7 @@ void create_export_panel(Widget w)
 	export_offset_y = XtCreateManagedWidget("export_offset_y", asciiTextWidgetClass,
 					     postscript_form, Args, ArgCount);
 	FirstArg(XtNfromHoriz, export_offset_y);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNtop, XtChainTop);
@@ -1746,6 +1806,7 @@ void create_export_panel(Widget w)
 
 	/* (HPGL) PCL output option */
 	FirstArg(XtNlabel, "Issue PCL Command to use HPGL");  /* Label */
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, exp_off_lab);
 	NextArg(XtNtop, XtChainTop);
@@ -1768,6 +1829,7 @@ void create_export_panel(Widget w)
 	NextArg(XtNinternalWidth, 1);
 	NextArg(XtNinternalHeight, 1);
 	NextArg(XtNlabel, "  ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNstate, print_hpgl_pcl_switch);	/* initial state */
 	pcl_cmd_checkbox = XtCreateManagedWidget("pcl_cmd_checkbox",
 			toggleWidgetClass, postscript_form, Args, ArgCount);
@@ -1776,6 +1838,7 @@ void create_export_panel(Widget w)
 
 	/* (HPGL) Use 'SD' font command */
 	FirstArg(XtNlabel, "Use Specified Font");  /* Label */
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, exp_off_lab);
 	NextArg(XtNfromHoriz, pcl_cmd_checkbox);
@@ -1798,6 +1861,7 @@ void create_export_panel(Widget w)
 	NextArg(XtNinternalWidth, 1);
 	NextArg(XtNinternalHeight, 1);
 	NextArg(XtNlabel, "  ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNstate, hpgl_specified_font);	/* initial state */
 	hpgl_font_checkbox = XtCreateManagedWidget("hpgl_font_checkbox",
 			toggleWidgetClass, postscript_form, Args, ArgCount);
@@ -1838,6 +1902,7 @@ void create_export_panel(Widget w)
 	/* next the default file name */
 
 	FirstArg(XtNlabel, " Default File");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
 	NextArg(XtNbottom, XtChainTop);
@@ -1847,6 +1912,7 @@ void create_export_panel(Widget w)
 					  bottom_section, Args, ArgCount);
 
 	FirstArg(XtNlabel, default_export_file);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNfromHoriz, dfile_lab);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1857,6 +1923,7 @@ void create_export_panel(Widget w)
 					   bottom_section, Args, ArgCount);
 
 	FirstArg(XtNlabel, "  Output File");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, dfile_text);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1883,6 +1950,7 @@ void create_export_panel(Widget w)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainRight);
+	NextArg(XtNinternational, appres.international);
 	exp_selfile = XtCreateManagedWidget("file", asciiTextWidgetClass,
 					    bottom_section, Args, ArgCount);
 	XtOverrideTranslations(exp_selfile,
@@ -1906,6 +1974,7 @@ void create_export_panel(Widget w)
 
 	/* cancel button */
 	FirstArg(XtNlabel, "Cancel");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNfromVert, below);
@@ -1922,6 +1991,7 @@ void create_export_panel(Widget w)
 			  (XtEventHandler) export_panel_cancel, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Export");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, cancel_but);
 	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNfromVert, below);
