@@ -1,7 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,6 +16,12 @@
  *
  */
 
+
+#ifndef W_EXPORT_H
+#define W_EXPORT_H
+
+#include <X11/Intrinsic.h>
+
 extern Widget	export_popup;	/* the main export popup */
 extern Widget	export_panel;	/* the form it's in */
 extern Widget	exp_selfile,	/* output (selected) file widget */
@@ -45,3 +53,5 @@ extern void	export_grid_major_select(Widget w, XtPointer new_grid_choice, XtPoin
 extern void	popup_export_panel(Widget w);
 extern void	do_export(Widget w);
 extern void update_def_filename (void);
+
+#endif
