@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2021 by Thomas Loimer
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,37 +16,51 @@
  *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"		/* intptr_t */
+#endif
+#include "w_file.h"
+
+#include <errno.h>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Xatom.h>		/* XA_STRING */
+#include <X11/Xft/Xft.h>
+
 #include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
-#include "e_edit.h"
-#include "f_read.h"
-#include "f_util.h"
-#include "u_create.h"
-#include "u_redraw.h"
-#include "w_drawprim.h"		/* for max_char_height */
-#include "w_dir.h"
-#include "w_export.h"
-#include "w_file.h"
-#include "w_indpanel.h"
-#include "w_layers.h"
-#include "w_msgpanel.h"
-#include "w_util.h"
-#include "w_setup.h"
-#include "w_icons.h"
-#include "w_zoom.h"
-
+#include "paintop.h"
 #include "e_compound.h"
 #include "f_load.h"
+#include "f_read.h"
 #include "f_save.h"
+#include "f_util.h"
+#include "u_colors.h"
+#include "u_create.h"
 #include "u_free.h"
 #include "u_list.h"
+#include "u_redraw.h"
 #include "w_canvas.h"
 #include "w_cmdpanel.h"
 #include "w_color.h"
 #include "w_cursor.h"
+#include "w_drawprim.h"		/* for max_char_height */
+#include "w_dir.h"
+#include "w_export.h"
+#include "w_layers.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_util.h"
+#include "w_zoom.h"
+#include "xfig_math.h"
 
 #define	FILE_WID	157	/* width of Current file etc to show preview to the right */
 #define FILE_ALT_WID	260	/* width of file alternatives panel */
@@ -76,30 +90,45 @@ Boolean		colors_are_swapped = False;
 Widget		preview_size, preview_widget, preview_widget_form, preview_name;
 Widget		preview_stop, preview_label, dummy_label;
 Widget		comments_widget;
-Pixmap		preview_land_pixmap, preview_port_pixmap;
 Boolean		cancel_preview = False;
 Boolean		preview_in_progress = False;
-void		load_request(Widget w, XButtonEvent *ev);		/* this is needed by main() */
+void		load_request(Widget w, XButtonEvent *ev); /* needed by main() */
 
 /* LOCALS */
 
-static void	file_preview_stop(Widget w, XButtonEvent *ev);
 static Boolean	file_cancel_request = False;
 static Boolean	file_load_request = False;
 static Boolean	file_save_request = False;
 static Boolean	file_merge_request = False;
 static char	save_file_dir[PATH_MAX];
 
-/* these are in fig units */
+static Boolean	user_colors_saved = False;
+static XftColor	saved_user_colors[MAX_USR_COLS];
+static Boolean	saved_userFree[MAX_USR_COLS];
+static int	saved_user_num;
 
-static float	offset_unit_conv[] = { (float)PIX_PER_INCH, (float)PIX_PER_CM, 1.0 };
+static Boolean	nuser_colors_saved = False;
+static XftColor	saved_nuser_colors[MAX_USR_COLS];
+static Boolean	saved_nuserFree[MAX_USR_COLS];
+static int	saved_nuser_num;
+static Pixmap	preview_land_pixmap, preview_port_pixmap;
+
+
+static void	file_preview_stop(Widget w, XButtonEvent *ev);
+
+/* these are in fig units */
+static float	offset_unit_conv[] = {
+	(float)PIX_PER_INCH,
+	(float)PIX_PER_CM,
+	1.0
+};
 
 static char	*load_msg = "The current figure is modified.\nDo you want to discard it and load the new file?";
 static char	buf[40];
 
 /* to save image colors when doing a figure preview */
 
-static XColor	save_image_cells[MAX_COLORMAP_SIZE];
+static XftColor	save_image_cells[MAX_COLORMAP_SIZE];
 static int	save_avail_image_cols;
 static Boolean	image_colors_are_saved = False;
 
@@ -164,7 +193,7 @@ static XtActionsRec	file_actions[] =
 
 
 int renamefile (char *file);
-void create_file_panel (void);
+static void create_file_panel (void);
 
 void
 file_panel_dismiss(void)
@@ -181,12 +210,11 @@ file_panel_dismiss(void)
 	/* restore image colors on canvas */
 	avail_image_cols = save_avail_image_cols;
 	for (i=0; i<avail_image_cols; i++) {
-	    image_cells[i].red   = save_image_cells[i].red;
-	    image_cells[i].green = save_image_cells[i].green;
-	    image_cells[i].blue  = save_image_cells[i].blue;
-	    image_cells[i].flags  = DoRed|DoGreen|DoBlue;
+	    image_cells[i].color.red   = save_image_cells[i].color.red;
+	    image_cells[i].color.green = save_image_cells[i].color.green;
+	    image_cells[i].color.blue  = save_image_cells[i].color.blue;
         }
-	YStoreColors(tool_cm, image_cells, avail_image_cols);
+	alloc_or_store_colors(image_cells, avail_image_cols);
 	image_colors_are_saved = False;
     }
     FirstArg(XtNstring, "\0");
@@ -231,6 +259,8 @@ merge_request(Widget w, XButtonEvent *ev)
 static void
 do_merge(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     char	    path[PATH_MAX], fname[PATH_MAX];
     char	   *fval, *dval;
     int		    xoff, yoff;
@@ -300,6 +330,8 @@ load_request(Widget w, XButtonEvent *ev)
 static void
 do_load(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     char	    fname[PATH_MAX];
     char	   *fval, *dval;
     int		    xoff, yoff;
@@ -369,6 +401,8 @@ do_load(Widget w, XButtonEvent *ev)
 void
 new_xfig_request(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     pid_t pid;
     char *fval, fname[PATH_MAX];
     char **xxargv;
@@ -433,9 +467,65 @@ save_request(Widget w, XButtonEvent *ev)
     }
 }
 
+/*
+ * The user might enter a file path containing directory components relative to
+ * cur_file_dir, an absolute path, or a path relative to the home dir.
+ * Set cur_file_dir to the directory containing the basename of the file.
+ * If something goes wrong, continue with a probably wrong cur_file_dir.
+ */
+static void
+set_curfiledir_from_savepath(const char *restrict fname)
+{
+	char	*b;	/* point to slash before basename component in fname */
+
+	b = strrchr(fname, '/');
+	if (!b && !(fname[0] == '~' && fname[1] == '/'))
+		/* fname does not contain a directory component */
+		return;
+
+	if (fname[0] == '~' && fname[1] == '/') {
+		char	*home = getenv("HOME");
+		size_t	home_len = strlen(home);
+
+		if (b - fname + home_len < PATH_MAX) {
+			memcpy(cur_file_dir, home, home_len);
+			memcpy(cur_file_dir + home_len, fname + 1,
+					b - fname - 1);/* omit the last slash */
+			cur_file_dir[b - fname + home_len - 1] = '\0';
+		} else {
+			goto warning;
+		}
+		return;
+	} else if (fname[0] == '/') {
+		if (b - fname + 1 < PATH_MAX) {
+			memcpy(cur_file_dir, fname, b - fname);
+			cur_file_dir[b - fname] = '\0';
+		} else {
+			goto warning;
+		}
+	} else { /*  to/rel/path/f.fig */
+		size_t	cur_len = strlen(cur_file_dir);
+		if (cur_len + b - fname + 2 < PATH_MAX) {
+			cur_file_dir[cur_len] = '/';
+			memcpy(cur_file_dir + cur_len + 1, fname, b - fname);
+			cur_file_dir[cur_len + b - fname + 1] = '\0';
+		} else {
+			goto warning;
+		}
+	}
+	return;
+
+warning:
+	file_msg("Could not change to directory of saved file. "
+			"If %s contains pictures, incorrect picture file paths "
+			"might be written to the saved file.");
+}
+
 void
 do_save(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     char	   *fval, *dval, fname[PATH_MAX];
     int		    qresult;
 
@@ -476,11 +566,14 @@ do_save(Widget w, XButtonEvent *ev)
 	GetValues(file_selfile);    /* check the ascii widget for a filename */
 	strcpy(fname, fval);
 	if (emptyname(fname)) {
-	    strcpy(fname, cur_filename); /* "Filename" widget empty, use current filename */
+		/* "Filename" widget empty, use current filename */
+	    strcpy(fname, cur_filename);
 	    warnexist = False;		/* don't warn if this file exists */
 	/* copy the name from the file_name widget to the current filename */
 	} else {
-	    if (!strchr(fname,'.'))	/* if no suffix, add .fig */
+		char	*b = strrchr(fname, '/');
+		/* if no suffix in the basename part, add .fig */
+	    if ((b && !strchr(b, '.')) || !strchr(fname, '.'))
 		strcat(fname,".fig");
 	    if (strcmp(cur_filename, fname) != 0)
 		warnexist = True;	/* warn if this file exists */
@@ -496,14 +589,36 @@ do_save(Widget w, XButtonEvent *ev)
 	    strcpy(save_file_dir, dval);
 	}
 
+	/*
+	 * Image paths are written to the .fig file as relative to cur_file_dir.
+	 * The user might enter a file path containing directory components into
+	 * the file_selfile widget. Update cur_file_dir accordingly.
+	 */
+
 	/* go to the file directory */
 	if (change_directory(save_file_dir) == 0) {
+		char	tmp_save_dir[PATH_MAX];
+		char	abs_path_buf[1024] = "";
+		char	*abs_path = abs_path_buf;
+
 	    if (!ok_to_write(fname, "SAVE"))
 		return;
 	    XtSetSensitive(save_button, False);
 	    if (appres.write_bak == True)
 		(void) renamefile(fname);
-	    if (write_file(fname, True) == 0) {
+	    strcpy(tmp_save_dir, cur_file_dir);
+	    set_curfiledir_from_savepath(fname);
+	    if (fname[0] == '~' && fname[1] == '/') {
+		    char	*home = getenv("HOME");
+		    size_t	home_len = strlen(home);
+		    size_t	fname_len = strlen(fname) - 1;
+		    if (home_len + fname_len + 1 > sizeof abs_path_buf &&
+			    !(abs_path = new_string(home_len + fname_len)))
+			    return;
+		    memcpy(abs_path, home, home_len);
+		    memcpy(abs_path + home_len, fname + 1, fname_len + 1);
+	    }
+	    if (write_file(*abs_path ? abs_path : fname, True) == 0) {
 		FirstArg(XtNlabel, fname);
 		SetValues(cfile_text);
 		if (strcmp(fname, cur_filename) != 0) {
@@ -513,12 +628,17 @@ do_save(Widget w, XButtonEvent *ev)
 		if (file_up)
 		    file_panel_dismiss();
 	    }
+	    if (abs_path != abs_path_buf)
+		    free(abs_path);
 	    XtSetSensitive(save_button, True);
+	    strcpy(cur_file_dir, tmp_save_dir);
 	}
     } else { /* !file_popup */
+	    char	tmp_save_dir[PATH_MAX];
 	/*
 	 * !file_popup, so the file panel was never created and probably
-	 * save_file_dir not set.
+	 * save_file_dir not set. Also, the user did not have opportunity to
+	 * enter "~/some.fig" as file path, no need to parse for "~".
 	 */
 	if (*save_file_dir == '\0') {
 		strcpy(save_file_dir, cur_file_dir);
@@ -530,8 +650,11 @@ do_save(Widget w, XButtonEvent *ev)
 	/* not using popup => filename not changed so ok to write existing file */
 	warnexist = False;
 	(void) renamefile(cur_filename);
+	strcpy(tmp_save_dir, cur_file_dir);
+	set_curfiledir_from_savepath(fname);
 	if (write_file(cur_filename, True) == 0)
 	    reset_modifiedflag();
+	strcpy(cur_file_dir, tmp_save_dir);
     }
 }
 
@@ -586,6 +709,8 @@ cancel_request(Widget w, XButtonEvent *ev)
 static void
 file_panel_cancel(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     if (user_colors_saved) {
 	restore_user_colors();
 	restore_nuser_colors();
@@ -725,6 +850,7 @@ popup_file_panel(int mode)
 static void
 file_xoff_unit_select(Widget w, XtPointer new_unit, XtPointer garbage)
 {
+	(void)garbage;
     FirstArg(XtNlabel, XtName(w));
     SetValues(file_xoff_unit_panel);
     xoff_unit_setting = (intptr_t) new_unit;
@@ -733,6 +859,7 @@ file_xoff_unit_select(Widget w, XtPointer new_unit, XtPointer garbage)
 static void
 file_yoff_unit_select(Widget w, XtPointer new_unit, XtPointer garbage)
 {
+	(void)garbage;
     FirstArg(XtNlabel, XtName(w));
     SetValues(file_yoff_unit_panel);
     yoff_unit_setting = (intptr_t) new_unit;
@@ -753,7 +880,9 @@ void create_file_panel(void)
 	FirstArg(XtNx, xposn+50);
 	NextArg(XtNy, yposn+50);
 	NextArg(XtNtitle, "Xfig: File menu");
+	NextArg(XtNtitleEncoding, XA_STRING);
 	NextArg(XtNcolormap, tool_cm);
+	NextArg(XtNinput, True);
 	file_popup = XtCreatePopupShell("file_popup",
 					transientShellWidgetClass,
 					tool, Args, ArgCount);
@@ -764,6 +893,7 @@ void create_file_panel(void)
 					   file_popup, NULL, ZERO);
 
 	FirstArg(XtNlabel, "  File Status");
+	NextArg(XtNinternational, False);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNresize, False);
@@ -775,6 +905,7 @@ void create_file_panel(void)
 					    file_panel, Args, ArgCount);
 	/* start with long label so length is enough - it will be reset by calling proc */
 	FirstArg(XtNlabel, "Not modified ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, file_stat_label);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
@@ -787,6 +918,7 @@ void create_file_panel(void)
 					    file_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, " # of Objects");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, file_status);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
@@ -798,6 +930,7 @@ void create_file_panel(void)
 					  file_panel, Args, ArgCount);
 
 	FirstArg(XtNwidth, 50);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, file_status);
 	NextArg(XtNfromHoriz, num_obj_label);
 	NextArg(XtNjustify, XtJustifyLeft);
@@ -811,6 +944,7 @@ void create_file_panel(void)
 					    file_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, "Figure Offset");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, num_objects);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -820,6 +954,7 @@ void create_file_panel(void)
 	figure_off = XtCreateManagedWidget("fig_offset_label", labelWidgetClass,
 				     file_panel, Args, ArgCount);
 	FirstArg(XtNlabel, "X");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, figure_off);
 	NextArg(XtNfromVert, num_objects);
 	NextArg(XtNborderWidth, 0);
@@ -843,6 +978,7 @@ void create_file_panel(void)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, appres.international);
 	fig_offset_x = XtCreateManagedWidget("fig_offset_x", asciiTextWidgetClass,
 					     file_panel, Args, ArgCount);
 	FirstArg(XtNfromHoriz, fig_offset_x);
@@ -852,6 +988,7 @@ void create_file_panel(void)
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
 	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
+	NextArg(XtNinternational, False);
 	file_xoff_unit_panel = XtCreateManagedWidget(offset_unit_items[appres.INCHES? 0: 1],
 				menuButtonWidgetClass, file_panel, Args, ArgCount);
 	make_pulldown_menu(offset_unit_items, XtNumber(offset_unit_items),
@@ -860,6 +997,7 @@ void create_file_panel(void)
 	/* put the Y offset below the X */
 
 	FirstArg(XtNlabel, "Y");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, figure_off);
 	NextArg(XtNfromVert, file_xoff_unit_panel);
 	NextArg(XtNborderWidth, 0);
@@ -883,6 +1021,7 @@ void create_file_panel(void)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, appres.international);
 	fig_offset_y = XtCreateManagedWidget("fig_offset_y", asciiTextWidgetClass,
 					     file_panel, Args, ArgCount);
 	FirstArg(XtNfromHoriz, fig_offset_y);
@@ -892,12 +1031,14 @@ void create_file_panel(void)
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
 	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
+	NextArg(XtNinternational, False);
 	file_yoff_unit_panel = XtCreateManagedWidget(offset_unit_items[appres.INCHES? 0: 1],
 				menuButtonWidgetClass, file_panel, Args, ArgCount);
 	make_pulldown_menu(offset_unit_items, XtNumber(offset_unit_items),
 				   -1, "", file_yoff_unit_panel, file_yoff_unit_select);
 
 	FirstArg(XtNlabel, " Current File");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, file_yoff_unit_panel);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
@@ -909,6 +1050,7 @@ void create_file_panel(void)
 					  file_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, cur_filename);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNfromVert, file_yoff_unit_panel);
 	NextArg(XtNfromHoriz, cfile_lab);
 	NextArg(XtNjustify, XtJustifyLeft);
@@ -922,6 +1064,7 @@ void create_file_panel(void)
 					   file_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, "     Filename");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, cfile_lab);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -939,6 +1082,7 @@ void create_file_panel(void)
 	NextArg(XtNheight, max_char_height(temp_font) * 2 + 4);
 	NextArg(XtNeditType, XawtextEdit);
 	NextArg(XtNstring, cur_filename);
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNinsertPosition, strlen(cur_filename));
 	NextArg(XtNfromHoriz, file);
 	NextArg(XtNfromVert, cfile_lab);
@@ -980,12 +1124,14 @@ void create_file_panel(void)
 
 	/* a title */
 	FirstArg(XtNlabel, "Preview   ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	preview_label = XtCreateManagedWidget("preview_label", labelWidgetClass,
 					    preview_form, Args, ArgCount);
 
 	/* make a label for the figure size */
 	FirstArg(XtNlabel, " ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, preview_label);
 	NextArg(XtNborderWidth, 0);
 	preview_size = XtCreateManagedWidget("preview_size", labelWidgetClass,
@@ -993,6 +1139,7 @@ void create_file_panel(void)
 
 	/* and one for the figure name in the preview */
 	FirstArg(XtNlabel, " ");
+	NextArg(XtNinternational, appres.international);
 	NextArg(XtNfromVert, preview_label);
 	NextArg(XtNborderWidth, 0);
 	preview_name = XtCreateManagedWidget("preview_name", labelWidgetClass,
@@ -1021,6 +1168,7 @@ void create_file_panel(void)
 
 	/* make a dummy label to fill the space until we put up the preview */
 	FirstArg(XtNlabel, "");
+	NextArg(XtNinternational, False);
 	NextArg(XtNwidth, PREVIEW_CANVAS_W);
 	NextArg(XtNheight, PREVIEW_CANVAS_W);
 	dummy_label = XtCreateManagedWidget("dummy_label", labelWidgetClass,
@@ -1030,6 +1178,7 @@ void create_file_panel(void)
 	/* don't manage it yet until the panel is popped up */
 
 	FirstArg(XtNlabel, "");
+	NextArg(XtNinternational, False);
 	NextArg(XtNbackgroundPixmap, preview_port_pixmap);
 	NextArg(XtNhorizDistance, (PREVIEW_CANVAS_W-PREVIEW_CANVAS_H)/2+4); /* center */
 	NextArg(XtNvertDistance, 4);
@@ -1047,6 +1196,7 @@ void create_file_panel(void)
 	/* label for the figure comments */
 
 	FirstArg(XtNlabel, "Figure comments:");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, preview_widget_form);
 	below = XtCreateManagedWidget("comments_label", labelWidgetClass,
 					  preview_form, Args, ArgCount);
@@ -1065,11 +1215,13 @@ void create_file_panel(void)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, appres.international);
 	comments_widget = XtCreateManagedWidget("comments", asciiTextWidgetClass,
 					     preview_form, Args, ArgCount);
 
 	/* make cancel preview button */
 	FirstArg(XtNlabel, "Stop Preview");
+	NextArg(XtNinternational, False);
 	NextArg(XtNsensitive, False);
 	NextArg(XtNfromVert, comments_widget);
 	preview_stop = XtCreateManagedWidget("preview_stop", commandWidgetClass,
@@ -1082,6 +1234,7 @@ void create_file_panel(void)
 	/* now make buttons along the bottom */
 
 	FirstArg(XtNlabel, "Cancel");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNfromVert, butbelow);
@@ -1101,6 +1254,7 @@ void create_file_panel(void)
 	   done in popup_saveas_panel, popup_open_panel, and popup_merge_panel */
 
 	FirstArg(XtNlabel, " Save ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNvertDistance, 15);
 	NextArg(XtNfromVert, butbelow);
 	NextArg(XtNfromHoriz, beside);
@@ -1117,6 +1271,7 @@ void create_file_panel(void)
 			  (XtEventHandler) do_save, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, " Open ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNvertDistance, 15);
 	NextArg(XtNfromVert, butbelow);
@@ -1133,6 +1288,7 @@ void create_file_panel(void)
 			  (XtEventHandler)load_request, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Merge ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNhorizDistance, 20);
 	NextArg(XtNfromVert, butbelow);
 	NextArg(XtNfromHoriz, beside);
@@ -1149,6 +1305,7 @@ void create_file_panel(void)
 			  (XtEventHandler)merge_request, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "New xfig...");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNvertDistance, 15);
 	NextArg(XtNfromVert, butbelow);
@@ -1194,7 +1351,7 @@ clear_preview(void)
     SetValues(preview_size);
 
     /* clear both port and land pixmap */
-    XSetForeground(tool_d, gccache[ERASE], x_bg_color.pixel);
+    XSetForeground(tool_d, gccache[ERASE], getpixel(CANVAS_BG));
     XFillRectangle(tool_d, preview_land_pixmap, gccache[ERASE], 0, 0,
 		   PREVIEW_CANVAS_W, PREVIEW_CANVAS_H);
     XFillRectangle(tool_d, preview_port_pixmap, gccache[ERASE], 0, 0,
@@ -1210,6 +1367,8 @@ clear_preview(void)
 static void
 file_preview_stop(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     /* set the cancel flag */
     cancel_preview = True;
     /* make the cancel button insensitive */
@@ -1217,7 +1376,7 @@ file_preview_stop(Widget w, XButtonEvent *ev)
     process_pending();
 }
 
-void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_widget, Pixmap port_pixmap, Pixmap land_pixmap)
+void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_widget)
 {
     fig_settings    settings;
     int		save_objmask;
@@ -1226,7 +1385,7 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
     Boolean	save_shownums;
     F_compound	*figure;
     int		xmin, xmax, ymin, ymax;
-    float	width, height, size;
+    float	width, height;
     int		pixwidth, pixheight;
     int		i, status;
     char	figsize[50];
@@ -1276,7 +1435,7 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
     FirstArg(XtNbackground, &pb);
     GetValues(preview_label);
     /* set it to red while previewing */
-    FirstArg(XtNbackground, x_color(YELLOW));
+    FirstArg(XtNbackground, getpixel(YELLOW));
     SetValues(preview_label);
 
     /* give filename a chance to appear in the Filename field */
@@ -1307,9 +1466,9 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 	image_colors_are_saved = True;
 	/* save current canvas image colors */
 	for (i=0; i<avail_image_cols; i++) {
-	    save_image_cells[i].red   = image_cells[i].red;
-	    save_image_cells[i].green = image_cells[i].green;
-	    save_image_cells[i].blue  = image_cells[i].blue;
+	    save_image_cells[i].color.red   = image_cells[i].color.red;
+	    save_image_cells[i].color.green = image_cells[i].color.green;
+	    save_image_cells[i].color.blue  = image_cells[i].color.blue;
 	}
 	save_avail_image_cols = avail_image_cols;
     }
@@ -1359,7 +1518,6 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 	ymax = figure->secorner.y;
 	width = xmax - xmin;
 	height = ymax - ymin;
-	size = max2(width,height) / ZOOM_FACTOR;
 	/* calculate size in current units */
 	if (width < PIX_PER_INCH || height < PIX_PER_INCH) {
 	    /* for small figures, show more decimal places */
@@ -1383,14 +1541,15 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 
 	/* now switch the drawing canvas to our preview window and set width/height */
 	if (settings.landscape) {
-	    canvas_win = (Window) land_pixmap;
+	    canvas_win = (Window) preview_land_pixmap;
 	    pixwidth = PREVIEW_CANVAS_W;
 	    pixheight = PREVIEW_CANVAS_H;
 	} else {
-	    canvas_win = (Window) port_pixmap;
+	    canvas_win = (Window) preview_port_pixmap;
 	    pixwidth = PREVIEW_CANVAS_H;
 	    pixheight = PREVIEW_CANVAS_W;
 	}
+	XftDrawChange(canvas_draw, canvas_win);
 
 	/* scale to fit the preview canvas */
 	display_zoomscale =
@@ -1414,7 +1573,7 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 	cur_objmask = 0;
 
 	/* clear the pixmap with the canvas background color */
-	XSetForeground(tool_d, gccache[ERASE], x_bg_color.pixel);
+	XSetForeground(tool_d, gccache[ERASE], getpixel(CANVAS_BG));
 	XFillRectangle(tool_d, canvas_win, gccache[ERASE], 0, 0, pixwidth, pixheight);
 
 	/* if he pressed "cancel preview" while reading file, skip displaying figure */
@@ -1446,6 +1605,7 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 
     /* switch canvas back */
     canvas_win = main_canvas;
+    XftDrawChange(canvas_draw, main_canvas);
 
     /* restore active layer array */
     restore_active_layers();
@@ -1512,3 +1672,127 @@ void preview_figure(char *filename, Widget parent, Widget canvas, Widget size_wi
 	request_redraw = False;
     }
 }
+
+/* save user colors into temp vars */
+void save_user_colors(void)
+{
+	int	i;
+
+	if (appres.DEBUG)
+		fprintf(stderr,
+			"** Saving user colors. Before: bool user_colors_saved = %d\n",
+		user_colors_saved);
+
+	user_colors_saved = True;
+
+	/* first save the current colors because del_color_cell destroys them */
+	for (i = 0; i < num_usr_cols; ++i) {
+		saved_user_colors[i] = user_color[i];
+		/* and save Free entries */
+		saved_userFree[i] = colorFree[i];
+		/* now free any previously defined user colors */
+		del_color_cell(i);	/* remove widget and colormap entry */
+	}
+
+	saved_user_num = num_usr_cols;
+}
+
+/* save n_user colors into temp vars */
+void save_nuser_colors(void)
+{
+	int		i;
+
+	if (appres.DEBUG)
+		fprintf(stderr,
+			"** Saving n_user colors. Before: bool nuser_colors_saved = %d\n",
+			user_colors_saved);
+
+	nuser_colors_saved = True;
+
+	/* first save the current colors because del_color_cell destroys them */
+	for (i = 0; i < n_num_usr_cols; ++i) {
+		saved_nuser_colors[i] = n_user_colors[i];
+		/* and save Free entries */
+		saved_nuserFree[i] = n_colorFree[i];
+	}
+	saved_nuser_num = n_num_usr_cols;
+}
+
+/* restore user colors from temp vars */
+void restore_user_colors(void)
+{
+	int	i, num;
+
+	if (!user_colors_saved)
+		return;
+
+	if (appres.DEBUG)
+		fprintf(stderr,
+			"** Restoring user colors. Before: bool user_colors_saved = %d\n",
+			user_colors_saved);
+
+	user_colors_saved = False;
+
+	/* first free any previously defined user colors */
+	for (i = 0; i < num_usr_cols; ++i) {
+		del_color_cell(i);	/* remove widget and colormap entry */
+	}
+
+	num_usr_cols = saved_user_num;
+
+	/* now restore the orig user colors */
+	for (i = 0; i < num_usr_cols; ++i) {
+		user_color[i] = saved_user_colors[i];
+		/* and Free entries */
+		colorFree[i] = saved_userFree[i];
+	}
+
+	/* now try to allocate those colors */
+	if (num_usr_cols > 0) {
+		num = num_usr_cols;
+		num_usr_cols = 0;
+		/* fill the colormap and the color memories */
+		for (i = 0; i < num; ++i) {
+			if (colorFree[i]) {
+				colorUsed[i] = False;
+				continue;
+			}
+			/* and add a widget and colormap entry */
+			if (add_color_cell(USE_EXISTING_COLOR, i,
+						user_color[i].color.red/256,
+						user_color[i].color.green/256,
+						user_color[i].color.blue/256)
+					== -1) {
+				file_msg("Can't allocate more than %d user colors, not enough colormap entries",
+						num_usr_cols);
+				return;
+			}
+			colorUsed[i] = True;
+		}
+	}
+}
+
+/* Restore user colors from temp vars into n_user_...  */
+void restore_nuser_colors(void)
+{
+	int	i;
+
+	if (!nuser_colors_saved)
+		return;
+
+	if (appres.DEBUG)
+		fprintf(stderr,
+			"** Restoring user colors into n_user colors\n");
+
+	nuser_colors_saved = False;
+
+	n_num_usr_cols = saved_nuser_num;
+
+	/* now restore the orig user colors */
+	for (i = 0; i < n_num_usr_cols; ++i) {
+		n_user_colors[i] = saved_nuser_colors[i];
+		/* and Free entries */
+		n_colorFree[i] = saved_nuserFree[i];
+	}
+}
+
