@@ -1,7 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,6 +16,11 @@
  *
  */
 
+#ifndef W_FILE_H
+#define W_FILE_H
+
+#include <X11/Intrinsic.h>	/* includes X11/Xlib.h */
+
 extern Boolean	colors_are_swapped;
 extern void	load_request(Widget w, XButtonEvent *ev);
 extern void	do_save(Widget w, XButtonEvent *ev);
@@ -23,7 +30,6 @@ extern void	popup_saveas_panel(void);
 extern Boolean	query_save(char *msg);
 
 extern Widget	preview_size, preview_widget, preview_name;
-extern Pixmap	preview_land_pixmap, preview_port_pixmap;
 
 extern Widget	file_panel;
 extern Boolean	file_up;
@@ -38,6 +44,8 @@ extern Widget	file_popup;
 extern Boolean	check_cancel(void);
 extern Boolean	cancel_preview;
 extern Boolean	preview_in_progress;
-extern void preview_figure (char *filename, Widget parent, Widget canvas, Widget size_widget, Pixmap port_pixmap, Pixmap land_pixmap);
+extern void preview_figure (char *filename, Widget parent, Widget canvas, Widget size_widget);
 extern int renamefile (char *file);
 extern void file_panel_dismiss (void);
+
+#endif
