@@ -1,6 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -13,17 +16,33 @@
  *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "w_fontpanel.h"
+
+#include <stdint.h>
+#include <sys/types.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Xatom.h>		/* XA_STRING */
+#include <locale.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif /* HAVE_STRINGS_H */
+
 #include "figx.h"
 #include "resources.h"
 #include "u_fonts.h"		/* printer font names */
+#include "w_cmdpanel.h"		/* refresh_character_panel() */
+#include "w_color.h"
 #include "w_fontbits.h"
 #include "w_indpanel.h"
 #include "w_msgpanel.h"
-#include "w_fontpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
-#include "w_color.h"
+
 
 /********************  local variables	***************************/
 
@@ -70,14 +89,14 @@ init_fontmenu(Widget tool)
     register MenuItemRec *mi;
     XtTranslations  pane_actions;
 
-    DeclareArgs(8);
+    DeclareArgs(9);
 
     first_fontmenu = True;
 
     FirstArg(XtNborderWidth, POPUP_BW);
     NextArg(XtNmappedWhenManaged, False);
     NextArg(XtNtitle, "Xfig: Font menu");
-
+    NextArg(XtNtitleEncoding, XA_STRING);
     ps_fontmenu = XtCreatePopupShell("ps_font_menu",
 				     transientShellWidgetClass, tool,
 				     Args, ArgCount);
@@ -164,6 +183,7 @@ init_fontmenu(Widget tool)
     NextArg(XtNinternalHeight, 0);
     NextArg(XtNborderWidth, INTERNAL_BW);
     NextArg(XtNresize, False);	/* don't allow resize */
+    NextArg(XtNinternational, False);
 
     for (i = 0; i < NUM_FONTS + 1; ++i) {
 	mi = &ps_fontmenu_items[i];
@@ -174,6 +194,7 @@ init_fontmenu(Widget tool)
     }
     /* Cancel */
     FirstArg(XtNlabel, "Cancel");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, ps_fontpanes);
     NextArg(XtNhorizDistance, 4);
     NextArg(XtNtop, XtChainTop);
@@ -187,6 +208,7 @@ init_fontmenu(Widget tool)
 
     /* button to switch to the LaTeX fonts */
     FirstArg(XtNlabel, " Use LaTex Fonts ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, ps_fontpanes);
     NextArg(XtNfromHoriz, ps_cancel);
     NextArg(XtNhorizDistance, 10);
@@ -208,6 +230,7 @@ init_fontmenu(Widget tool)
     NextArg(XtNinternalHeight, 0);
     NextArg(XtNborderWidth, INTERNAL_BW);
     NextArg(XtNresize, False);	/* don't allow resize */
+    NextArg(XtNinternational, False);
 
     for (i = 0; i < NUM_LATEX_FONTS; ++i) {
 	mi = &latex_fontmenu_items[i];
@@ -218,6 +241,7 @@ init_fontmenu(Widget tool)
     }
     /* Cancel */
     FirstArg(XtNlabel, "Cancel");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, latex_fontpanes);
     NextArg(XtNhorizDistance, 4);
     NextArg(XtNtop, XtChainTop);
@@ -231,6 +255,7 @@ init_fontmenu(Widget tool)
 
     /* button to switch to the PostScript fonts */
     FirstArg(XtNlabel, " Use PostScript Fonts ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, latex_fontpanes);
     NextArg(XtNfromHoriz, latex_cancel);
     NextArg(XtNhorizDistance, 10);
@@ -265,7 +290,6 @@ setup_fontmenu(void)
 
     /* Create the bitmaps */
 
-#ifdef I18N
     if (appres.international) {
       char *lang;
       lang = appres.font_menu_language;
@@ -286,7 +310,6 @@ setup_fontmenu(void)
 	latexfont_menu_bits[2] = Korean_Bold_bits;
       }
     }
-#endif  /* I18N */
     for (i = 0; i < NUM_FONTS + 1; i++)
 	psfont_menu_bitmaps[i] = XCreatePixmapFromBitmapData(tool_d,
 				   XtWindow(ind_panel), (char *) psfont_menu_bits[i],
@@ -348,19 +371,32 @@ fontpane_popup(int *psfont_adr, int *latexfont_adr, int *psflag_adr, void (*show
 static void
 fontpane_select(Widget w, XtPointer closure, XtPointer call_data)
 {
-    MenuItemRec	   *mi = (MenuItemRec *) closure;
-    char	   *font_name = mi->label;
-    ptr_int	data = {mi->info};
+	(void)w;
+	(void)call_data;
+	Bool		refresh = False;
+	MenuItemRec	*mi = (MenuItemRec *) closure;
+	char		*font_name = mi->label;
+	ptr_int		data = {mi->info};
 
-    if (*flag_sel)
-	*font_ps_sel = data.val;
-    else
-	*font_latex_sel = data.val;
+	if (*flag_sel) {
+		if (*font_ps_sel != data.val) {
+			*font_ps_sel = data.val;
+			refresh = True;
+		}
+	} else {
+		if (*font_latex_sel != data.val) {
+			*font_latex_sel = data.val;
+			refresh = True;
+		}
+	}
 
-    put_msg("Font: %s", font_name);
-    /* put image of font in indicator window */
-    (*font_setimage) (font_widget);
-    XtPopdown(*flag_sel ? ps_fontmenu : latex_fontmenu);
+	if (refresh) {
+		refresh_character_panel(*flag_sel, data.val);
+		put_msg("Font: %s", font_name);
+		/* put image of font in indicator window */
+		(*font_setimage) (font_widget);
+	}
+	XtPopdown(*flag_sel ? ps_fontmenu : latex_fontmenu);
 }
 
 static void
