@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2020 by Thomas Loimer
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -20,11 +20,18 @@
  * Copyright (c) 1995-2002 by T. Sato
  */
 
-#include "fig.h"
-#include "figx.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdlib.h>
+#include <string.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+
 #include "resources.h"
 #include "u_fonts.h"
-#include "object.h"  /* for spell_canvas() */
+#include "xfig_math.h"
+
 
 /* replacement of Times_Roman_bits etc, for Japanese */
 unsigned char Japanese_Times_Roman_bits[] = {
@@ -389,327 +396,6 @@ unsigned char Korean_Bold_bits[] = {
    0x00,0x00,0x00,0x00};
 
 
-/* check if str include i18n character */
-static Boolean is_i18n_text(str)
-     char *str;
-{
-  int i;
-  if (appres.always_use_fontset) return TRUE;
-  for (i = 0; str[i] != '\0'; i++) {
-    if (str[i] & 0x80) return TRUE;
-  }
-  return FALSE;
-}
-
-/* get fontset and size corresponding to font specified by fid */
-/* return FALSE if the font is not for i18n */
-static Boolean seek_fontset(fid, fontset, size_ret)
-     Font fid;
-     XFontSet *fontset;
-     int *size_ret;
-{
-  extern struct _xfstruct x_fontinfo[]; /* X11 fontnames */
-  struct xfont *nf;
-  int i;
-  if (!appres.international) return FALSE;
-  for (i = 0; i < NUM_FONTS; i++) {
-    nf = x_fontinfo[i].xfontlist;
-    while (nf != NULL && (nf->fset == NULL || nf->fstruct->fid != fid))
-      nf = nf->next;
-    if (nf != NULL && nf->fset != NULL) {
-      *fontset=nf->fset;
-      *size_ret = nf->size;
-      return TRUE;
-    }
-  }
-  return FALSE;
-}
-
-/* check if the font is for i18n */
-Boolean is_i18n_font(font)
-     XFontStruct *font;
-{
-  XFontSet fontset;
-  int font_size;
-  return seek_fontset(font->fid, &fontset, &font_size);
-}
-
-int
-i18n_fontset_height(fontset)
-     XFontSet fontset;
-{
-  XFontSetExtents *extents;
-  extents = XExtentsOfFontSet(fontset);
-  return extents->max_logical_extent.height;
-}
-
-
-/* get extents of the text */
-void
-i18n_text_extents(font, str, len, dir, asc, des, overall)
-     XFontStruct *font;
-     char *str;
-     int len;
-     int *dir, *asc, *des;  /* assume these return values are not used */
-     XCharStruct *overall;
-{
-  XFontSet fontset;
-  int font_size;
-  XRectangle ink, logical;
-  double scale;
-  if (!appres.international || !is_i18n_text(str)
-      || !seek_fontset(font->fid, &fontset, &font_size)) {
-    XTextExtents(font, str, len, dir, asc, des, overall);
-  } else {
-    XmbTextExtents(fontset, str, len, &ink, &logical);
-    if (appres.fontset_size <= 0)
-      scale = (double)font_size / (double)i18n_fontset_height(fontset);
-    else
-      scale = (double)font_size / (double)appres.fontset_size;
-    overall->width = max2(logical.width, ink.width + ink.x) * scale + 0.5;
-    overall->lbearing = -ink.x * scale;
-    overall->rbearing = overall->width;
-    overall->ascent = -logical.y * scale;
-    overall->descent = logical.height * scale - overall->ascent;
-  }
-}
-
-/* functions for bitmap scaling (AllocTextBitmapAndScaleBuf, ScaleUp, */
-/* and ScaleDown) are derived from tgif-2.13 */
-
-static int text_bitmap_width = -1;
-static int text_bitmap_height = -1;
-static Pixmap text_bitmap = None;
-static char *scale_buf = NULL;
-static GC depth_one_gc = None;
-
-static void AllocTextBitmapAndScaleBuf(display, d, width, height)
-     Display *display;
-     Drawable d;
-     int width;
-     int height;
-{
-  if (height > text_bitmap_height || width > text_bitmap_width) {
-    if (0 < text_bitmap_height && 0 < text_bitmap_width) {
-      XFreePixmap(display, text_bitmap);
-      free(scale_buf);
-    }
-
-    if (height < 1) height = 1;
-    if (width < 1) width = 1;
-    text_bitmap_height = height;
-    text_bitmap_width = width;
-    text_bitmap = XCreatePixmap(display, d,
-		       text_bitmap_width, text_bitmap_height, 1);
-    scale_buf = (char *)malloc(text_bitmap_height * sizeof(char));
-  }
-}
-
-static void ScaleUp(from_image, to_image, width, height, scale)
-     XImage *from_image;
-     XImage *to_image;
-     int width;
-     int height;
-     double scale;
-{
-  int wd = width * scale;
-  int ht = height * scale;
-  register int i, j;
-  register int x1, y1, x2, y2;
-
-  i = -1;
-  for (x2 = 0; x2 < wd; x2++) {
-    x1 = x2 / scale;
-    if (x1 != i) {
-      j = -1;
-      for (y2 = 0; y2 < ht; y2++) {
-	y1 = y2 / scale;
-	if (y1 != j) {
-	  if (scale_buf[y2] = XGetPixel(from_image, x1, y1))
-	    XPutPixel(to_image, x2, y2, 1);
-	  j = y1;
-	} else {
-	  if (scale_buf[y2] = scale_buf[y2 - 1])
-	    XPutPixel(to_image, x2, y2, 1);
-	}
-      }
-      i = x1;
-    } else {
-      for (y2 = 0; y2 < ht; y2++) {
-	if (scale_buf[y2] == 1)
-	  XPutPixel(to_image, x2, y2, 1);
-      }
-    }
-  }
-}
-
-static void ScaleDown(from_image, to_image, width, height, scale)
-     XImage *from_image;
-     XImage *to_image;
-     int width;
-     int height;
-     double scale;
-{
-  int y1, y2;
-  register int x1, x2, last_x2;
-
-  for (y1 = 0; y1 < height; y1++) {
-    y2 = y1 * scale;
-    last_x2 = -1;
-    for (x1 = 0; x1 < width; x1++) {
-      x2 = x1 * scale;
-      if (x2 != last_x2 && XGetPixel(from_image, x1, y1)) {
-	XPutPixel(to_image, x2, y2, 1);
-	last_x2 = x2;
-      }
-    }
-  }
-}
-
-void i18n_draw_text();
-
-/* draw text to drawable WITH background color */
-
-void i18n_draw_image_string(dpy, drawable, gc, x, y, str, len)
-     Display *dpy;
-     Drawable drawable;
-     GC gc;
-     int x, y;
-     char *str;
-     int len;
-{
-  i18n_draw_text(dpy, drawable, gc, x, y, str, len, True);
-}
-
-/* draw text to drawable with no background */
-
-void i18n_draw_string(dpy, drawable, gc, x, y, str, len)
-     Display *dpy;
-     Drawable drawable;
-     GC gc;
-     int x, y;
-     char *str;
-     int len;
-{
-  i18n_draw_text(dpy, drawable, gc, x, y, str, len, False);
-}
-
-void i18n_draw_text(dpy, drawable, gc, x, y, str, len, drawbg)
-     Display	*dpy;
-     Drawable	 drawable;
-     GC		 gc;
-     int	 x, y;
-     char	 *str;
-     int	 len;
-     Boolean	 drawbg;
-{
-  static Pixmap paint_bitmap = None;
-  static int paint_bitmap_width = -1;
-  static int paint_bitmap_height = -1;
-  static XImage *to_image = NULL;
-  static char *to_image_data = NULL;
-  static int to_image_size = -1;
-  static GC my_gc = None;
-  static Drawable last_drawable = None;
-  static Font last_font = None;
-  static char last_str[400] = "";
-  static int width, lbearing, height, ascent;
-  XImage *from_image;
-  XFontSet fontset;
-  int font_size;
-  int base_width, base_height;
-  XGCValues values;
-  XRectangle ink, logical;
-  double scale;
-
-  XGetGCValues(dpy, gc, GCFont, &values);
-  if (!appres.international || !is_i18n_text(str)
-      || !seek_fontset(values.font, &fontset, &font_size)) {
-    if (drawbg)
-	XDrawImageString(dpy, drawable, gc, x, y, str, len);
-    else
-	XDrawString(dpy, drawable, gc, x, y, str, len);
-  } else {
-    if (drawable != last_drawable || values.font != last_font
-	|| strncmp(str, last_str, sizeof(last_str)) != 0) {
-      /* generate paint_bitmap only if it is needed */
-      last_drawable = drawable;
-      last_font = values.font;
-      strncpy(last_str, str, sizeof(last_str));
-
-      XmbTextExtents(fontset, str, len, &ink, &logical);
-      base_width = ink.width + abs(ink.x) + 1;
-      base_height = ink.height + 1;
-
-      if (appres.fontset_size <= 0)
-	scale = (double)font_size / (double)i18n_fontset_height(fontset);
-      else
-	scale = (double)font_size / (double)appres.fontset_size;
-      width = base_width * scale + 0.5;
-      lbearing = max2(-ink.x, 0) * scale + 0.5;
-      height = base_height * scale + 0.5;
-      ascent = -ink.y * scale + 0.5;
-      AllocTextBitmapAndScaleBuf(dpy, drawable,
-			 max2(base_width, width), max2(base_height, height));
-      if (depth_one_gc == None) {
-	depth_one_gc = XCreateGC(dpy, text_bitmap, (unsigned long)0, 0);
-	XSetBackground(dpy, depth_one_gc, 0);
-	XSetFillStyle(dpy, depth_one_gc, FillSolid);
-	XSetFunction(dpy, depth_one_gc, GXcopy);
-      }
-      if (my_gc != None) XFreeGC(dpy, my_gc);
-      my_gc = XCreateGC(dpy, drawable, (unsigned long)0, 0);
-      if (to_image == NULL
-	  || to_image->width < text_bitmap_width
-	  || to_image->height < text_bitmap_height) {
-	if (to_image != NULL) XDestroyImage(to_image);
-	to_image_size = (unsigned)(text_bitmap_width/8 + 1) * text_bitmap_height;
-	to_image_data = malloc(to_image_size);
-	to_image = XCreateImage(dpy, DefaultVisual(dpy, DefaultScreen(dpy)),
-				1, XYBitmap, 0, to_image_data,
-				text_bitmap_width, text_bitmap_height, 8, 0);
-      }
-      if (paint_bitmap_width < width || paint_bitmap_height < height) {
-	if (paint_bitmap != None) XFreePixmap(dpy, paint_bitmap);
-	paint_bitmap = XCreatePixmap(dpy, drawable, width, height, 1);
-	paint_bitmap_width = width;
-	paint_bitmap_height = height;
-      }
-
-      XSetForeground(dpy, depth_one_gc, 0);
-      XFillRectangle(dpy, text_bitmap, depth_one_gc, 0, 0,
-		     text_bitmap_width, text_bitmap_height);
-      XSetForeground(dpy, depth_one_gc, 1);
-      if (drawbg)
-          XmbDrawImageString(dpy, text_bitmap, fontset, depth_one_gc,
-		    max2(-ink.x, 0), -ink.y, str, len);
-      else
-          XmbDrawString(dpy, text_bitmap, fontset, depth_one_gc,
-		    max2(-ink.x, 0), -ink.y, str, len);
-
-      from_image = XGetImage(dpy, text_bitmap, 0, 0,
-		     text_bitmap_width, text_bitmap_height, 1, ZPixmap);
-
-      memset(to_image_data, 0, (size_t)to_image_size);
-      if (1.0 < scale) {  /* scale up */
-	ScaleUp(from_image, to_image, base_width, base_height, scale);
-      } else {  /* scale down */
-	ScaleDown(from_image, to_image, base_width, base_height, scale);
-      }
-      XDestroyImage(from_image);
-
-      XPutImage(dpy, paint_bitmap, depth_one_gc, to_image, 0, 0, 0, 0, width, height);
-    }
-
-    /* paint text using stipple technique */
-    XCopyGC(dpy, gc, GCForeground|GCBackground|GCFunction|GCPlaneMask, my_gc);
-    XSetFillStyle(dpy, my_gc, FillStippled);
-    XSetStipple(dpy, my_gc, paint_bitmap);
-    XSetTSOrigin(dpy, my_gc, x - lbearing, y - ascent);
-    XFillRectangle(dpy, drawable, my_gc, x - lbearing, y - ascent, width, height);
-  }
-}
-
 #ifndef HAVE_SETLOCALE
 
 char *setlocale(category, locale)
