@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,26 +16,35 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "w_layers.h"
+
+#include <stdint.h>
+#include <stdio.h>
+#include <string.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
 
-#include "fig.h"
 #include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
 #include "f_read.h"
+#include "u_redraw.h"
 #include "w_drawprim.h"
-#include "w_dir.h"
+#include "w_canvas.h"
+#include "w_cmdpanel.h"
 #include "w_file.h"
 #include "w_indpanel.h"
 #include "w_layers.h"
-#include "w_util.h"
-#include "w_setup.h"
-#include "u_redraw.h"
-#include "w_snap.h"
-
-#include "w_cmdpanel.h"
 #include "w_msgpanel.h"
-#include "w_canvas.h"
+#include "w_setup.h"
+#include "w_snap.h"
+#include "w_util.h"
+
 
 /* EXPORTS */
 
@@ -67,13 +77,6 @@ static	void	all_inactive(Widget w, XtPointer closure, XtPointer call_data);
 static	void	toggle_all(Widget w, XtPointer closure, XtPointer call_data);
 static	void	switch_layer_mode(Widget w, XtPointer closure, XtPointer call_data);
 static	void	layer_exposed(Widget w, XExposeEvent *event, String *params, Cardinal *nparams);
-
-#ifndef XAW3D1_5E
-/* popup message over command button when mouse enters it */
-static void     layer_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
-static void     layer_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
-#endif /* XAW3D1_5E */
-
 static  void	draw_layer_button(Window win, int but);
 static  void	draw_layer_buttons(void);
 
@@ -137,6 +140,7 @@ init_depth_panel(Widget parent)
 
     /* a label */
     FirstArg(XtNlabel, "Depths   ");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
@@ -147,6 +151,7 @@ init_depth_panel(Widget parent)
     /* buttons to make all active, all inactive or toggle all */
 
     FirstArg(XtNlabel, "All On  ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, label);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -155,15 +160,9 @@ init_depth_panel(Widget parent)
     below = all_active_but = XtCreateManagedWidget("all_active", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)all_active, (XtPointer) NULL);
-#ifndef XAW3D1_5E
-    /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, False,
-			  (XtEventHandler)layer_balloon_trigger, (XtPointer) "Display all depths");
-    XtAddEventHandler(below, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
 
     FirstArg(XtNlabel, "All Off ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -172,15 +171,9 @@ init_depth_panel(Widget parent)
     below = all_inactive_but = XtCreateManagedWidget("all_inactive", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)all_inactive, (XtPointer) NULL);
-#ifndef XAW3D1_5E
-    /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, False,
-			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Hide all depths");
-    XtAddEventHandler(below, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
 
     FirstArg(XtNlabel, "Toggle  ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -189,13 +182,6 @@ init_depth_panel(Widget parent)
     below = toggle_all_but = XtCreateManagedWidget("toggle_all", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)toggle_all, (XtPointer) NULL);
-#ifndef XAW3D1_5E
-    /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, False,
-			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Toggle displayed/hidden depths");
-    XtAddEventHandler(below, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
 
     /* radio buttons to switch between graying out inactive layers or blanking altogether */
 
@@ -210,8 +196,8 @@ init_depth_panel(Widget parent)
     NextArg(XtNright, XtChainLeft);
     NextArg(XtNinternalWidth, 1);
     NextArg(XtNinternalHeight, 1);
-    NextArg(XtNfont, bold_font);
-    NextArg(XtNlabel, "  ");
+    NextArg(XtNlabel, " ");
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, (gray_layers? False : True));	/* make opposite button sens */
     NextArg(XtNstate, gray_layers);	/* initial state */
     NextArg(XtNradioData, 1);		/* when this is pressed the value is 1 */
@@ -230,15 +216,9 @@ init_depth_panel(Widget parent)
     NextArg(XtNleft, XtChainRight);
     NextArg(XtNright, XtChainRight);
     NextArg(XtNlabel, "Gray  ");
+    NextArg(XtNinternational, False);
     graylabel = below = XtCreateManagedWidget("graylabel", labelWidgetClass,
 				layer_form, Args, ArgCount);
-#ifndef XAW3D1_5E
-    /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, False,
-			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Display inactive layers in gray");
-    XtAddEventHandler(below, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
 
     /* radio for blanking */
     FirstArg(XtNwidth, sm_check_width+6);
@@ -250,8 +230,8 @@ init_depth_panel(Widget parent)
     NextArg(XtNright, XtChainLeft);
     NextArg(XtNinternalWidth, 1);
     NextArg(XtNinternalHeight, 1);
-    NextArg(XtNfont, bold_font);
-    NextArg(XtNlabel, "  ");
+    NextArg(XtNlabel, " ");
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, (gray_layers? True : False));	/* make opposite button sens */
     NextArg(XtNstate, !gray_layers);	/* initial state */
     NextArg(XtNradioData, 2);		/* when this is pressed the value is 2 */
@@ -267,19 +247,13 @@ init_depth_panel(Widget parent)
     NextArg(XtNfromVert, graytoggle);
     NextArg(XtNfromHoriz, blanktoggle);
     NextArg(XtNlabel, "Blank ");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainRight);
     NextArg(XtNright, XtChainRight);
     blanklabel = below = XtCreateManagedWidget("blanklabel", labelWidgetClass,
 				layer_form, Args, ArgCount);
-#ifndef XAW3D1_5E
-    /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, False,
-			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Blank inactive layers");
-    XtAddEventHandler(below, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
 
     /* a form to hold the "Front" and "Back" labels and the viewport widget */
 
@@ -297,6 +271,7 @@ init_depth_panel(Widget parent)
 
     FirstArg(XtNborderWidth, 0);
     NextArg(XtNlabel, " Front ");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainRight);
@@ -319,12 +294,6 @@ init_depth_panel(Widget parent)
     /* popup when mouse passes over button */
 
     /* pass null pointer for trigger message so we'll use the two-line one */
-#ifndef XAW3D1_5E
-    XtAddEventHandler(layer_viewp, EnterWindowMask, False,
-			   (XtEventHandler)layer_balloon_trigger, (XtPointer) NULL);
-    XtAddEventHandler(layer_viewp, LeaveWindowMask, False,
-			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
-#endif /* XAW3D1_5E */
     XtAddEventHandler(layer_viewp, StructureNotifyMask, False,
 			   (XtEventHandler)update_layers, (XtPointer) NULL);
 
@@ -334,6 +303,7 @@ init_depth_panel(Widget parent)
     FirstArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
     NextArg(XtNlabel, "");
+    NextArg(XtNinternational, False);
     layer_canvas = XtCreateManagedWidget("layer_canvas", labelWidgetClass,
 				layer_viewp, Args, ArgCount);
     /* label - "Back" */
@@ -341,6 +311,7 @@ init_depth_panel(Widget parent)
     FirstArg(XtNborderWidth, 0);
     NextArg(XtNfromVert, layer_viewp);
     NextArg(XtNlabel, " Back  ");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainBottom);
     NextArg(XtNbottom, XtChainBottom);
     NextArg(XtNleft, XtChainRight);
@@ -354,9 +325,7 @@ init_depth_panel(Widget parent)
     XtOverrideTranslations(layer_canvas,
 			   XtParseTranslationTable(layer_translations));
 
-#ifdef XAW3D1_5E
     update_layerpanel();
-#endif /* XAW3D1_5E */
 }
 
 /* now that the mouse function and indicator panels have been sized
@@ -486,6 +455,7 @@ draw_layer_button(Window win, int but)
 static void
 layer_exposed(Widget w, XExposeEvent *event, String *params, Cardinal *nparams)
 {
+	(void)params; (void)nparams;
     int		    y, i;
 
     if (min_depth < 0) return;  /* if no objects return */
@@ -524,6 +494,7 @@ calculate_pressed_depth(int y)
 static void
 set_depth_to_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)w; (void)params; (void)nparams;
     int		    but;
 
     but = calculate_pressed_depth(event->y);
@@ -536,6 +507,7 @@ set_depth_to_layer(Widget w, XButtonEvent *event, String *params, Cardinal *npar
 static void
 toggle_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)w; (void)params; (void)nparams;
     Window	    win = XtWindow(layer_canvas);
     int		    but, i;
     Boolean	    obscure;
@@ -572,6 +544,7 @@ toggle_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 static void
 sweep_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)w; (void)params; (void)nparams;
     Window	    win = XtWindow(layer_canvas);
     int		    but;
     int		    i;
@@ -611,12 +584,14 @@ sweep_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 static void
 leave_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)w; (void)event; (void)params; (void)nparams;
     pressed_but = -1;
 }
 
 static void
 all_active(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)w; (void)closure; (void)call_data;
     int i;
     Boolean changed = False;
 
@@ -638,6 +613,7 @@ all_active(Widget w, XtPointer closure, XtPointer call_data)
 static void
 all_inactive(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)w; (void)closure; (void)call_data;
     int i;
     Boolean changed = False;
 
@@ -659,6 +635,7 @@ all_inactive(Widget w, XtPointer closure, XtPointer call_data)
 static void
 toggle_all(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)w; (void)closure; (void)call_data;
     int i;
 
     if (min_depth < 0) return;
@@ -675,6 +652,7 @@ toggle_all(Widget w, XtPointer closure, XtPointer call_data)
 static void
 switch_layer_mode(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure; (void)call_data;
     Boolean	    state;
     intptr_t	    which;
 
@@ -746,154 +724,51 @@ any_active_in_compound(F_compound *cmpnd)
 	return False;
 }
 
-#ifdef XAW3D1_5E
-void update_layerpanel()
+void
+update_layerpanel()
 {
     /*
      * We must test for the widgets, as this is called by
      * w_cmdpanel.c:refresh_view_menu().
      */
 
-    if (all_active_but)
+    if (all_active_but) {
 	if (appres.showballoons)
-	    XawTipEnable(all_active_but, "Display all depths");
+	    XAWTIPENABLE(all_active_but, "Display all depths");
 	else
 	    XawTipDisable(all_active_but);
-    if (all_inactive_but)
+    }
+    if (all_inactive_but) {
 	if (appres.showballoons)
-	    XawTipEnable(all_inactive_but, "Hide all depths");
+	    XAWTIPENABLE(all_inactive_but, "Hide all depths");
 	else
 	    XawTipDisable(all_inactive_but);
-    if (toggle_all_but)
+    }
+    if (toggle_all_but) {
 	if (appres.showballoons)
-	    XawTipEnable(toggle_all_but, "Toggle displayed/hidden depths");
+	    XAWTIPENABLE(toggle_all_but, "Toggle displayed/hidden depths");
 	else
 	    XawTipDisable(toggle_all_but);
-    if (layer_viewp)
+    }
+    if (layer_viewp) {
 	if (appres.showballoons)
-	    XawTipEnable(layer_viewp, "Display or hide any depth");
+	    XAWTIPENABLE(layer_viewp, "Display or hide any depth");
 	else
 	    XawTipDisable(layer_viewp);
-    if (graylabel)
+    }
+    if (graylabel) {
 	if (appres.showballoons)
-	    XawTipEnable(graylabel, "Display inactive layers in gray");
+	    XAWTIPENABLE(graylabel, "Display inactive layers in gray");
 	else
 	    XawTipDisable(graylabel);
-    if (blanklabel)
+    }
+    if (blanklabel) {
 	if (appres.showballoons)
-	    XawTipEnable(blanklabel, "Blank inactive layers");
+	    XAWTIPENABLE(blanklabel, "Blank inactive layers");
 	else
 	    XawTipDisable(blanklabel);
-}
-#else
-static	Widget layer_balloon_popup = (Widget) 0;
-static	XtIntervalId balloon_id = (XtIntervalId) 0;
-static	Widget balloon_w;
-static	char *popmsg;
-
-static	void layer_balloon(Widget w, XtPointer closure, XtPointer call_data);
-
-/* come here when the mouse passes over a button in the depths panel */
-
-static void
-layer_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
-{
-	if (!appres.showballoons)
-		return;
-	balloon_w = widget;
-	/* save the message to popup */
-	popmsg = (char *) closure;
-	/* if an old balloon is still up, destroy it */
-	if ((balloon_id != 0) || (layer_balloon_popup != (Widget) 0)) {
-		layer_unballoon((Widget) 0, (XtPointer) 0, (XEvent*) 0, (Boolean*) 0);
-	}
-	balloon_id = XtAppAddTimeOut(tool_app, appres.balloon_delay,
-			(XtTimerCallbackProc) layer_balloon, (XtPointer) widget);
-}
-
-/* come here when the timer times out (and the mouse is still over the button) */
-
-static void
-layer_balloon(Widget w, XtPointer closure, XtPointer call_data)
-{
-	Position  x, y;
-	Dimension wid, ht;
-	Widget box, balloon_label;
-	XtWidgetGeometry xtgeom,comp;
-
-	XtTranslateCoords(w, 0, 0, &x, &y);
-	FirstArg(XtNx, x);
-	NextArg(XtNy, y);
-	layer_balloon_popup = XtCreatePopupShell("layer_balloon_popup",
-				overrideShellWidgetClass, tool, Args, ArgCount);
-	FirstArg(XtNborderWidth, 0);
-	NextArg(XtNhSpace, 0);
-	NextArg(XtNvSpace, 0);
-	NextArg(XtNorientation, XtorientVertical);
-	box = XtCreateManagedWidget("box", boxWidgetClass,
-				layer_balloon_popup, Args, ArgCount);
-
-	/* make label for mouse message */
-	/* if the message was passed via the callback */
-	if (popmsg) {
-		FirstArg(XtNborderWidth, 0);
-		NextArg(XtNlabel, popmsg);
-		balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
-					    box, Args, ArgCount);
-	} else {
-		/* otherwise it is the two-line message with mouse indicators */
-		FirstArg(XtNborderWidth, 0);
-		NextArg(XtNleftBitmap, mouse_l);	/* bitmap of mouse with left button pushed */
-		NextArg(XtNlabel, "Display or hide depth    ");
-		balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
-					    box, Args, ArgCount);
-		FirstArg(XtNborderWidth, 0);
-		NextArg(XtNleftBitmap, mouse_r);	/* bitmap of mouse with right button pushed */
-		NextArg(XtNlabel, "Set current depth to this");
-		(void) XtCreateManagedWidget("r_label", labelWidgetClass,
-					box, Args, ArgCount);
-	}
-
-	/* realize the popup so we can get its size */
-	XtRealizeWidget(layer_balloon_popup);
-
-	/* get width/height */
-	FirstArg(XtNwidth, &wid);
-	NextArg(XtNheight, &ht);
-	GetValues(balloon_label);
-	/* only change X position of widget */
-	xtgeom.request_mode = CWX;
-	/* shift popup left */
-	xtgeom.x = x-wid-5;
-	/* if the mouse is in the depth button area, position the y there */
-	if (w == layer_viewp) {
-	    int wx, wy;	/* XTranslateCoordinates wants int, not Position */
-	    get_pointer_win_xy(&wx, &wy);
-	    xtgeom.request_mode |= CWY;
-	    xtgeom.y = y+wy - (int) ht - 60;
-	}
-	(void) XtMakeGeometryRequest(layer_balloon_popup, &xtgeom, &comp);
-
-	XtPopup(layer_balloon_popup,XtGrabNone);
-	XtRemoveTimeOut(balloon_id);
-	balloon_id = (XtIntervalId) 0;
-}
-
-/* come here when the mouse leaves a button in the layer panel */
-
-static void
-layer_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
-{
-    if (balloon_id) {
-	XtRemoveTimeOut(balloon_id);
-    }
-    balloon_id = (XtIntervalId) 0;
-    if (layer_balloon_popup != (Widget) 0) {
-	XtDestroyWidget(layer_balloon_popup);
-	layer_balloon_popup = (Widget) 0;
     }
 }
-#endif /* XAW3D1_5E */
 
 /* map/unmap the depth manager on the right side */
 
