@@ -1,7 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1998 by Stephane Mancini
- * Parts Copyright (c) 1999-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 1998 by Stephane Mancini
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,45 +19,62 @@
 
 /* This is where the library popup is created */
 
-#include "fig.h"
-#include "figx.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "w_library.h"
+
+#include <dirent.h>
 #include <stdarg.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#include <sys/stat.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Xatom.h>		/* XA_STRING */
+#include <X11/Xft/Xft.h>
+
+#include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
-#include "e_placelib.h"
+#include "paintop.h"
+#include "dirstruct.h"
 #include "e_placelib.h"
 #include "f_read.h"
-#include "u_create.h"
-#include "u_redraw.h"
-#include "w_canvas.h"
-#include "w_drawprim.h"		/* for max_char_height */
-#include "w_dir.h"
-#include "w_file.h"
-#include "w_indpanel.h"
-#include "w_library.h"
-#include "w_listwidget.h"
-#include "w_layers.h"
-#include "w_msgpanel.h"
-#include "w_util.h"
-#include "w_setup.h"
-#include "w_icons.h"
-#include "w_zoom.h"
-
 #include "f_util.h"
+#include "u_colors.h"
+#include "u_create.h"
 #include "u_free.h"
 #include "u_list.h"
 #include "u_markers.h"
+#include "u_redraw.h"
 #include "u_translate.h"
+#include "w_canvas.h"
 #include "w_color.h"
 #include "w_cursor.h"
+#include "w_drawprim.h"		/* for max_char_height */
+#include "w_dir.h"
+#include "w_file.h"
+#include "w_listwidget.h"
+#include "w_layers.h"
 #include "w_mousefun.h"
-#include "dirstruct.h"
-
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_util.h"
+#include "w_zoom.h"
+#include "xfig_math.h"
 #ifndef XAW3D1_5E
 #include "SmeCascade.h"
 #endif /* XAW3D1_5E */
 
+
 #define N_LIB_MAX	  100		/* max number of libraries */
 #define N_LIB_OBJECT_MAX  400		/* max number of objects in a library */
 #define N_LIB_NAME_MAX	  81		/* max length of library name + 1 */
@@ -157,13 +177,19 @@ static Position xposn, yposn;
 /* comparison function for filename sorting using qsort() */
 
 
-static Boolean	MakeObjectLibrary(char *library_dir, char **objects_names, F_libobject **libobjects),MakeLibraryFileList(char *dir_name, char **obj_list);
+static Boolean	MakeObjectLibrary(char *library_dir, char **objects_names,
+				F_libobject **libobjects);
+static Boolean	MakeLibraryFileList(char *dir_name, char **obj_list);
 static int	MakeLibrary(void);
-static Boolean	ScanLibraryDirectory(Boolean at_top, struct lib_rec **librec, char *path, char *longname, char *name, Boolean *figs_at_top, int *nentries);
-static Boolean	PutLibraryEntry(struct lib_rec *librec, char *path, char *lname, char *name);
+static Boolean	ScanLibraryDirectory(Boolean at_top, struct lib_rec **librec,
+				char *path, char *longname,
+				Boolean *figs_at_top, int *nentries);
+static Boolean	PutLibraryEntry(struct lib_rec *librec, char *path, char *lname,
+			       char *name);
 static Boolean	lib_just_loaded, icons_made;
 static Boolean	load_lib_obj(int obj);
-static Widget	make_library_menu(Widget parent, char *name, struct lib_rec **librec, int num);
+static Widget	make_library_menu(Widget parent, char *name,
+				struct lib_rec **librec, int num);
 
 
 static int
@@ -201,6 +227,9 @@ void put_selected_request(void)
 static void
 library_stop(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
+
      library_stop_request = True;
      XtSetSensitive(stop, False);
 }
@@ -215,6 +244,9 @@ library_dismiss(void)
 static void
 library_cancel(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
+
     /* unhighlight the selected item */
     if (appres.icon_view)
 	unsel_icon(cur_library_object);
@@ -272,6 +304,8 @@ library_cancel(Widget w, XButtonEvent *ev)
 static void
 load_library(Widget w, XtPointer new_library, XtPointer garbage)
 {
+	(void)w;
+	(void)garbage;
     struct lib_rec *librec = (struct lib_rec *) new_library;
     Dimension	    vwidth, vheight;
     Dimension	    vawidth, vaheight;
@@ -365,6 +399,8 @@ load_library(Widget w, XtPointer new_library, XtPointer garbage)
 static void
 put_object_sel(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     int	    obj;
 
     /* if there is no library loaded, ignore select */
@@ -473,6 +509,8 @@ load_lib_obj(int obj)
 static void
 NewObjectSel(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)w;
+	(void)closure;
     int		    new_obj;
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
@@ -530,7 +568,7 @@ popup_library_panel(void)
     FirstArg(XtNbackground, &unsel_color);
     GetValues(icon_box);
     /* use black for "selected" border color */
-    sel_color = x_color(BLACK);
+    sel_color = getpixel(BLACK);
 
     /* if the file message window is up add it to the grab */
     file_msg_add_grab();
@@ -592,6 +630,7 @@ create_library_panel(void)
     FirstArg(XtNx, xposn);
     NextArg(XtNy, yposn + 50);
     NextArg(XtNtitle, "Select an object or library");
+    NextArg(XtNtitleEncoding, XA_STRING);
     NextArg(XtNcolormap, tool_cm);
     library_popup = XtCreatePopupShell("library_menu",
 				     transientShellWidgetClass,
@@ -603,6 +642,7 @@ create_library_panel(void)
 			 XtParseTranslationTable(library_translations));
 
     FirstArg(XtNlabel, "Load a Library");
+    NextArg(XtNinternational, False);
     NextArg(XtNwidth, MAIN_WIDTH-4);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNtop, XtChainTop);
@@ -612,6 +652,7 @@ create_library_panel(void)
 
     /* label for Library */
     FirstArg(XtNlabel, "Library:");
+    NextArg(XtNinternational, False);
     NextArg(XtNjustify, XtJustifyLeft);
     NextArg(XtNfromVert, title);
     NextArg(XtNborderWidth, 0);
@@ -632,6 +673,7 @@ create_library_panel(void)
 	/* make long label to fill out panel */
 	FirstArg(XtNlabel, "000000000000000000000000000000000000000000000000");
     }
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, library_label);
     NextArg(XtNfromVert, title);
     NextArg(XtNborderWidth, INTERNAL_BW);
@@ -654,6 +696,7 @@ create_library_panel(void)
 
     /* Status indicator label */
     FirstArg(XtNlabel, " Status:");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, library_menu_button);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNtop, XtChainTop);
@@ -667,6 +710,7 @@ create_library_panel(void)
 
     FirstArg(XtNwidth, MAIN_WIDTH-77);
     NextArg(XtNstring, "None loaded");
+    NextArg(XtNinternational, False);
     NextArg(XtNeditType, XawtextRead);	/* read-only */
     NextArg(XtNdisplayCaret, FALSE);	/* don't want to see the ^ cursor */
     NextArg(XtNheight, 30);		/* make room for scrollbar if necessary */
@@ -684,6 +728,7 @@ create_library_panel(void)
     /* button to stop loading library */
 
     FirstArg(XtNlabel,"  Stop  ");
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, False);	/* disabled to start */
     NextArg(XtNfromVert, status_label);
     NextArg(XtNborderWidth, 0);
@@ -697,6 +742,7 @@ create_library_panel(void)
 		    (XtEventHandler) library_stop, (XtPointer) NULL);
 
     FirstArg(XtNlabel,"Selected object:");
+    NextArg(XtNinternational, False);
     NextArg(XtNresize, False);
     NextArg(XtNfromVert, stop);
     NextArg(XtNborderWidth, 0);
@@ -708,6 +754,7 @@ create_library_panel(void)
 			      library_form, Args, ArgCount);
 
     FirstArg(XtNlabel, " ");
+    NextArg(XtNinternational, False);
     NextArg(XtNresize, False);
     NextArg(XtNwidth, MAIN_WIDTH-130);
     NextArg(XtNfromHoriz, beside);
@@ -723,6 +770,7 @@ create_library_panel(void)
 
     /* make a label and pulldown to choose list or icon view */
     FirstArg(XtNlabel, "View:");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, put_cur_obj);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNtop, XtChainTop);
@@ -734,6 +782,7 @@ create_library_panel(void)
 
 
     FirstArg(XtNlabel, appres.icon_view? "Icon View":"List View");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, beside);
     NextArg(XtNfromVert, put_cur_obj);
     NextArg(XtNtop, XtChainTop);
@@ -753,6 +802,7 @@ create_library_panel(void)
     /* label and pulldown to choose size of icons */
 
     FirstArg(XtNlabel, "Icon size:");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, sel_view_button);
     NextArg(XtNhorizDistance, 10);
     NextArg(XtNfromVert, put_cur_obj);
@@ -766,6 +816,7 @@ create_library_panel(void)
 
     sprintf(buf, "%4d", appres.library_icon_size);
     FirstArg(XtNlabel, buf);
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, appres.icon_view? True: False); /* sensitive only if icon view */
     NextArg(XtNfromHoriz, beside);
     NextArg(XtNfromVert, put_cur_obj);
@@ -891,6 +942,7 @@ create_library_panel(void)
     NextArg(XtNbottom, XtChainBottom);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
+    NextArg(XtNinternational, False);
     object_list = XtCreateManagedWidget("object_list", figListWidgetClass,
 					list_viewport, Args, ArgCount);
     /* install the empty list */
@@ -904,6 +956,7 @@ create_library_panel(void)
     /* make a label for the preview canvas */
 
     FirstArg(XtNlabel, "Object preview");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, list_viewport);
     NextArg(XtNvertDistance, 40);
     NextArg(XtNwidth, LIB_PREVIEW_SIZE+2);
@@ -918,6 +971,7 @@ create_library_panel(void)
     /* and a label for the object name */
 
     FirstArg(XtNlabel," ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, list_viewport);
     NextArg(XtNfromVert,lib_preview_label);
     NextArg(XtNwidth, LIB_PREVIEW_SIZE+2);
@@ -945,6 +999,7 @@ create_library_panel(void)
     FirstArg(XtNfromHoriz, list_viewport);
     NextArg(XtNfromVert, lib_obj_label);
     NextArg(XtNlabel, "");
+    NextArg(XtNinternational, False);
     NextArg(XtNborderWidth, 1);
     NextArg(XtNbackgroundPixmap, preview_lib_pixmap);
     NextArg(XtNwidth, LIB_PREVIEW_SIZE);
@@ -960,6 +1015,7 @@ create_library_panel(void)
 
     /* now a place for the library object comments */
     FirstArg(XtNlabel, "Object comments:");
+    NextArg(XtNinternational, False);
     NextArg(XtNwidth, MAIN_WIDTH-6);
     NextArg(XtNfromVert, object_form);
     NextArg(XtNtop, XtChainBottom);
@@ -978,10 +1034,12 @@ create_library_panel(void)
     NextArg(XtNheight, 50);
     NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
     NextArg(XtNscrollVertical, XawtextScrollWhenNeeded);
+    NextArg(XtNinternational, appres.international);
     object_comments = XtCreateManagedWidget("object_comments", asciiTextWidgetClass,
 					  library_form, Args, ArgCount);
 
     FirstArg(XtNlabel, "   Cancel    ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, object_comments);
     NextArg(XtNvertDistance, 10);
     NextArg(XtNheight, 25);
@@ -997,6 +1055,7 @@ create_library_panel(void)
 		    (XtEventHandler) library_cancel, (XtPointer) NULL);
 
     FirstArg(XtNlabel, "Select object");
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, False);			/* no library loaded yet */
     NextArg(XtNfromVert, object_comments);
     NextArg(XtNvertDistance, 10);
@@ -1069,6 +1128,7 @@ make_library_menu(Widget parent, char *name, struct lib_rec **librec, int num)
 #endif /* XAW3D1_5E */
 		NextArg(XtNleftBitmap, diamond_pixmap);
 	    }
+	    NextArg(XtNinternational, False);
 #ifndef XAW3D1_5E
 	    entry = XtCreateManagedWidget(menu_name, smeCascadeObjectClass,
 					  menu, Args, ArgCount);
@@ -1153,6 +1213,8 @@ sel_view(Widget w, XtPointer client_data, XtPointer garbage)
 static void
 change_icon_size(Widget w, XtPointer menu_entry, XtPointer garbage)
 {
+	(void)w;
+	(void)garbage;
     char	   *new_size = (char*) icon_sizes[(intptr_t) menu_entry];
     int		    i;
 
@@ -1200,7 +1262,7 @@ erase_pixmap(Pixmap pixmap)
 static Boolean
 PutLibraryEntry(struct lib_rec *librec, char *path, char *lname, char *name)
 {
-    int i;
+    size_t i;
 
     /* strip any trailing whitespace */
     for (i=strlen(name)-1; i>0; i--) {
@@ -1231,14 +1293,14 @@ PutLibraryEntry(struct lib_rec *librec, char *path, char *lname, char *name)
 	librec	- pointer to lib_rec record to put entry
 	path	- absolute path of library entry
 	longname - name with parents prepended (e.g. Electrical / Physical)
-	name	- name of library
    Outputs:
 	figs_at_top - whether or not there are Fig files in toplevel
 	nentries - number of directories found (including subdirectories)
 */
 
 static Boolean
-ScanLibraryDirectory(Boolean at_top, struct lib_rec **librec, char *path, char *longname, char *name, Boolean *figs_at_top, int *nentries)
+ScanLibraryDirectory(Boolean at_top, struct lib_rec **librec, char *path,
+			char *longname, Boolean *figs_at_top, int *nentries)
 {
     DIR		*dirp;
     DIRSTRUCT	*dp;
@@ -1314,7 +1376,7 @@ ScanLibraryDirectory(Boolean at_top, struct lib_rec **librec, char *path, char *
 		    PutLibraryEntry(lp, path2, lname, dp->d_name);
 		    recnum++;
 		    /* and recurse */
-		    if (!ScanLibraryDirectory(False, lp->subdirs, path2, lname, dp->d_name,
+		    if (!ScanLibraryDirectory(False, lp->subdirs, path2, lname,
 				&lp->figs_at_top, &lp->nsubs)) {
 			/* close the directory */
 			closedir(dirp);
@@ -1366,7 +1428,7 @@ MakeLibrary(void)
 	return 0;
     } else if (S_ISDIR(st.st_mode)) {
 	/* if it is directory, scan the sub-directories and search libraries */
-	(void) ScanLibraryDirectory(True, library_rec, path, "", "", &dum, &numlibs);
+	(void) ScanLibraryDirectory(True, library_rec, path, "", &dum, &numlibs);
 	return numlibs;
     } else {
 	/* if it is a file, it must contain list of libraries */
@@ -1382,7 +1444,7 @@ MakeLibrary(void)
 			strcpy(name, strrchr(path2, '/') + 1);
 		    else {
 			/* use the last dir in the path for the name */
-			if (c=strrchr(path2,'/'))
+			if ((c=strrchr(path2,'/')))
 			    strcpy(name,c);
 			else
 			    strcpy(name, path2);
@@ -1396,8 +1458,9 @@ MakeLibrary(void)
 		}
 		PutLibraryEntry(library_rec[numlibs], path2, name, name);
 		/* and attach its subdirectories */
-		(void) ScanLibraryDirectory(True, library_rec[numlibs]->subdirs, path2, name, "",
-					&library_rec[numlibs]->figs_at_top, &num);
+		(void) ScanLibraryDirectory(True, library_rec[numlibs]->subdirs,
+				path2, name, &library_rec[numlibs]->figs_at_top,
+				&num);
 		library_rec[numlibs]->nsubs = num;
 		numlibs++;
 	    }
@@ -1648,6 +1711,7 @@ preview_libobj(int objnum, Pixmap pixmap, int pixsize, int margin)
 
     /* now switch the drawing canvas to our pixmap */
     canvas_win = (Window) pixmap;
+    XftDrawChange(canvas_draw, pixmap);
 
     /* make wait cursor */
     XDefineCursor(tool_d, XtWindow(library_form), wait_cursor);
@@ -1690,6 +1754,7 @@ preview_libobj(int objnum, Pixmap pixmap, int pixsize, int margin)
 
     /* switch canvas back */
     canvas_win = main_canvas;
+    XftDrawChange(canvas_draw, main_canvas);
 
     /* restore active layer array */
     restore_active_layers();
@@ -1753,6 +1818,7 @@ static	int	    old_item = -1;
 static void
 sel_item_icon(Widget w, XButtonEvent *ev)
 {
+	(void)ev;
     int		    i;
     F_compound	   *compound;
 
