@@ -1,6 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,21 +18,23 @@
 
 /* FigList widget public header file */
 
-#ifndef _FigList_h
-#define _FigList_h
+#ifndef W_LISTWIDGET_H
+#define W_LISTWIDGET_H
+
+#if defined HAVE_CONFIG_H && !defined VERSION
+#include "config.h"
+#endif
 
 #ifdef XAW3D
 #include <X11/Xaw3d/Simple.h>
 #else /* XAW3D */
 #include <X11/Xaw/Simple.h>
 #endif /* XAW3D */
-#include <X11/Xfuncproto.h>
+#include <X11/Intrinsic.h>
 
 extern WidgetClass figListWidgetClass;
 
 typedef struct	_FigListClassRec *FigListWidgetClass;
 typedef struct	_FigListRec	 *FigListWidget;
 
-#endif /* _FigList_h */
-
-/* DON'T ADD ANYTHING AFTER THIS #endif */
+#endif
