@@ -1,7 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -19,8 +21,10 @@
 #endif
 #include "w_modepanel.h"
 
-#include <stddef.h>
 #include <math.h>
+#include <stddef.h>
+#include <X11/StringDefs.h>
+#include <X11/Intrinsic.h>	/* includes X11/Xlib.h */
 
 #include "figx.h"
 #include "resources.h"
@@ -66,6 +70,7 @@
 #include "w_msgpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
+#include "w_zoom.h"
 
 /* EXPORTS */
 
@@ -303,6 +308,7 @@ static Arg      button_args[] =
 						 * a multiple of three of
 						 * them */
      /* 6 */ {XtNbackgroundPixmap, (XtArgVal) NULL},
+     /* 7 */ {XtNinternational, (XtArgVal) False},
 };
 
 
@@ -410,6 +416,7 @@ init_mode_panel(Widget tool)
 		     INTERNAL_BW * (SW_PER_ROW - 1));
 	    NextArg(XtNborderWidth, 0);
 	    NextArg(XtNresize, False);
+	    NextArg(XtNinternational, False);
 	    NextArg(XtNheight, (MODEPANEL_SPACE + 1) / 2);
 	    NextArg(XtNlabel, "Drawing");
 	    d_label = XtCreateManagedWidget("label", labelWidgetClass,
@@ -454,8 +461,9 @@ init_mode_panel(Widget tool)
 
 void setup_mode_panel(void)
 {
-    register int    i;
-    register mode_sw_info *msw;
+    int			i;
+    mode_sw_info	*msw;
+    unsigned long	but_fg, but_bg;
 
     blank_gc = XCreateGC(tool_d, XtWindow(mode_panel), (unsigned long) 0, NULL);
     FirstArg(XtNforeground, &but_fg);
@@ -556,6 +564,7 @@ mode_balloon(void)
 				Args, ArgCount);
 	FirstArg(XtNborderWidth, 0);
 	NextArg(XtNlabel, msw->modemsg);
+	NextArg(XtNinternational, False);
 	balloon_label = XtCreateManagedWidget("label", labelWidgetClass,
 				    box, Args, ArgCount);
 
@@ -607,6 +616,8 @@ sel_mode_but(Widget widget, XtPointer closure, XEvent *event,
     XButtonEvent    xbutton;
     mode_sw_info    *msw = (mode_sw_info *) closure;
 
+    if (zoom_in_progress)
+	cancel_zoom();
     /* erase any existing anchor for flips */
     if (setanchor)
 	center_marker(setanchor_x, setanchor_y);
