@@ -1,7 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,14 +16,20 @@
  *
  */
 
-#include "fig.h"
+#if defined HAVE_CONFIG_H && !defined VERSION
+#include "config.h"
+#endif
+#include "w_mousefun.h"
+
+#include <string.h>
+#include <X11/keysym.h>
+#include <X11/StringDefs.h>
+#include <X11/IntrinsicP.h>
+
 #include "figx.h"
 #include "resources.h"
-#include <X11/keysym.h>
 #include "w_icons.h"
 #include "w_drawprim.h"
-#include "w_indpanel.h"
-#include "w_mousefun.h"
 #include "w_setup.h"
 #include "w_util.h"
 
@@ -50,7 +58,7 @@ static char	mousefun_sh_r[MOUSEFUN_MAX];
 static char	lr_blank[]  = "                  ";
 /* give the middle button label the same */
 static char	mid_blank[] = "                  ";
-
+static unsigned long	mouse_but_fg, mouse_but_bg;
 static Pixmap	mousefun_pm;
 static Pixmap	keybd_pm;
 
@@ -78,6 +86,7 @@ init_mousefun(Widget tool)
     NextArg(XtNbackgroundPixmap, NULL);
     NextArg(XtNmappedWhenManaged, False);
     NextArg(XtNlabel, "");
+    NextArg(XtNinternational, False);
     NextArg(XtNleft, XtChainLeft);	/* chain so doesn't resize */
     NextArg(XtNright, XtChainLeft);
     NextArg(XtNtop, XtChainTop);
@@ -116,12 +125,13 @@ setup_mousefun(void)
 #ifdef XAW3D1_5E
 void update_mousepanel()
 {
-    if (mousefun)
+    if (mousefun) {
 	if (appres.showballoons)
 	    XawTipEnable(mousefun,
 			 "Shows which mouse buttons\nare active in each mode");
 	else
 	    XawTipDisable(mousefun);
+    }
 }
 #else
 /* come here when the mouse passes over a button in the mouse indicator panel */
@@ -163,6 +173,7 @@ mouse_balloon(void)
 					Args, ArgCount);
 	FirstArg(XtNborderWidth, 0);
 	NextArg(XtNlabel, "Shows which mouse buttons\nare active in each mode");
+	NextArg(XtNinternational, False);
 	balloon_label = XtCreateManagedWidget("label", labelWidgetClass,
 				box, Args, ArgCount);
 
@@ -343,6 +354,8 @@ draw_mousefun_unitbox(void)
 void
 draw_mousefun_topruler(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+	(void)w; (void)params; (void)num_params;
+
     if (event->type == KeyPress) {
 	KeySym	    key;
 	XKeyEvent  *xkey = (XKeyEvent *)event;
@@ -356,6 +369,8 @@ draw_mousefun_topruler(Widget w, XEvent *event, String *params, Cardinal *num_pa
 void
 draw_mousefun_sideruler(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+	(void)w; (void)params; (void)num_params;
+
     if (event->type == KeyPress) {
 	KeySym	    key;
 	XKeyEvent *xkey = (XKeyEvent *)event;
