@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2021 by Thomas Loimer
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,13 +16,23 @@
  *
  */
 
-
 /* This is for the message window below the command panel */
 /* The popup message window is handled in the second part of this file */
 
-#include "fig.h"
-#include "figx.h"
+#include "w_msgpanel.h"
+
+#include <math.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/IntrinsicP.h>
+#include <X11/Xatom.h>			/* XA_STRING */
+
+#include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
@@ -30,18 +40,17 @@
 #include "f_read.h"
 #include "f_util.h"
 #include "paintop.h"
+#include "u_colors.h"
 #include "u_elastic.h"
-#include "w_canvas.h"
-#include "w_drawprim.h"
-#include "w_file.h"
-#include "w_indpanel.h"
-#include "w_msgpanel.h"
-#include "w_util.h"
-#include "w_setup.h"
-#include "w_zoom.h"
-
 #include "u_geom.h"
+#include "w_canvas.h"
 #include "w_color.h"
+#include "w_drawprim.h"
+#include "w_setup.h"
+#include "w_util.h"
+#include "w_zoom.h"
+#include "xfig_math.h"
+
 
 /********************* EXPORTS *******************/
 
@@ -56,7 +65,7 @@ Boolean		first_lenmsg = True;
 #define		BUF_SIZE		500
 static char	prompt[BUF_SIZE];
 
-DeclareStaticArgs(12);
+DeclareStaticArgs(13);
 
 /* for the popup message (file_msg) window */
 
@@ -88,6 +97,7 @@ init_msg(Widget tool)
     NextArg(XtNwidth, MSGPANEL_WD);
     NextArg(XtNheight, MSGPANEL_HT);
     NextArg(XtNstring, "\0");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, cmd_form);
     NextArg(XtNvertDistance, -INTERNAL_BW);
     NextArg(XtNborderWidth, INTERNAL_BW);
@@ -184,9 +194,9 @@ boxsize_msg(int fact)
 
 	/* erase old text */
 	if (!first_lenmsg) {
-	    pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufx,
+	    pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, bufx,
 			RED, COLOR_NONE);
-	    pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufy,
+	    pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, bufy,
 			RED, COLOR_NONE);
 	}
 	first_lenmsg = False;
@@ -207,7 +217,7 @@ boxsize_msg(int fact)
 	    t1y = cur_y + sdy - 5.0/zoomscale;			/* above the line */
 	else
 	    t1y = cur_y + sdy + sizey.ascent + 5.0/zoomscale;	/* below the line */
-	pw_text(canvas_win, t1x, t1y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufx,
+	pw_text(canvas_win, t1x, t1y, INV_PAINT, MAX_DEPTH+1, roman_font, bufx,
 			RED, COLOR_NONE);
 
 	/* then dy */
@@ -216,7 +226,7 @@ boxsize_msg(int fact)
 	    t2x = fix_x + sdx + 5.0/zoomscale;			/* right of the line */
 	else
 	    t2x = fix_x + sdx - sizex.length - 4.0/zoomscale;	/* left of the line */
-	pw_text(canvas_win, t2x, t2y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufy,
+	pw_text(canvas_win, t2x, t2y, INV_PAINT, MAX_DEPTH+1, roman_font, bufy,
 			RED, COLOR_NONE);
 
 	/* now save new values */
@@ -257,9 +267,9 @@ void erase_box_lengths(void)
 {
     if (!first_lenmsg && appres.showlengths && !freehand_line) {
 	/* erase old text */
-	pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufx,
+	pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, bufx,
 			RED, COLOR_NONE);
-	pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufy,
+	pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, bufy,
 			RED, COLOR_NONE);
     }
     first_lenmsg = False;
@@ -273,11 +283,11 @@ void erase_lengths(void)
 	pw_vector(canvas_win, ofx, oy, ofx, ofy, INV_PAINT, 1, RUBBER_LINE, 0.0, RED);
 
 	/* erase old text */
-	pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufx,
+	pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font, bufx,
 			RED, COLOR_NONE);
-	pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufy,
+	pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font, bufy,
 			RED, COLOR_NONE);
-	pw_text(canvas_win, ot3x, ot3y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0, bufhyp,
+	pw_text(canvas_win, ot3x, ot3y, INV_PAINT, MAX_DEPTH+1, roman_font, bufhyp,
 			RED, COLOR_NONE);
     }
     first_lenmsg = False;
@@ -302,9 +312,9 @@ altlength_msg(int type, int fx, int fy)
   float		udx, udy;
   float		ang;
   int		sdx, sdy;
-  int		t1x, t1y, t2x, t2y, t3x, t3y;
+  int		t1x = 0, t1y = 0, t2x = 0, t2y = 0, t3x = 0, t3y = 0;
   PR_SIZE	sizex, sizey, sizehyp;
-  char		lenstr[80],dxstr[80],dystr[80];
+  char		lenstr[80], dxstr[80], dystr[80];
 
   dx = (cur_x - fx);
   dy = (cur_y - fy);
@@ -389,11 +399,11 @@ altlength_msg(int type, int fx, int fy)
 
 		    /* erase old text */
 		    pw_text(canvas_win, ot1x, ot1y, INV_PAINT, MAX_DEPTH+1, roman_font,
-					0.0, bufx, RED, COLOR_NONE);
+					bufx, RED, COLOR_NONE);
 		    pw_text(canvas_win, ot2x, ot2y, INV_PAINT, MAX_DEPTH+1, roman_font,
-					0.0, bufy, RED, COLOR_NONE);
+					bufy, RED, COLOR_NONE);
 		    pw_text(canvas_win, ot3x, ot3y, INV_PAINT, MAX_DEPTH+1, roman_font,
-					0.0, bufhyp, RED, COLOR_NONE);
+					bufhyp, RED, COLOR_NONE);
 		}
 
 		/* draw new lines */
@@ -420,7 +430,7 @@ altlength_msg(int type, int fx, int fy)
 		    t1y = cur_y + sdy - 3.0/zoomscale;			/* above the line */
 		else
 		    t1y = cur_y + sdy + sizey.ascent + 3.0/zoomscale;	/* below the line */
-		pw_text(canvas_win, t1x, t1y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0,
+		pw_text(canvas_win, t1x, t1y, INV_PAINT, MAX_DEPTH+1, roman_font,
 				bufx, RED, COLOR_NONE);
 
 		t2y = (cur_y+fy)/2+sdy;
@@ -429,7 +439,7 @@ altlength_msg(int type, int fx, int fy)
 		    t2x = fx + sdx + 4.0/zoomscale;			/* right of the line */
 		else
 		    t2x = fx + sdx - sizex.length - 4.0/zoomscale;	/* left of the line */
-		pw_text(canvas_win, t2x, t2y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0,
+		pw_text(canvas_win, t2x, t2y, INV_PAINT, MAX_DEPTH+1, roman_font,
 				bufy, RED, COLOR_NONE);
 
 		/* finally, the hypotenuse */
@@ -441,7 +451,7 @@ altlength_msg(int type, int fx, int fy)
 		    t3y = t2y + sizehyp.ascent + 3.0/zoomscale;	/* below the hyp */
 		else
 		    t3y = t2y - 3.0/zoomscale;			/* above the hyp */
-		pw_text(canvas_win, t3x, t3y, INV_PAINT, MAX_DEPTH+1, roman_font, 0.0,
+		pw_text(canvas_win, t3x, t3y, INV_PAINT, MAX_DEPTH+1, roman_font,
 				bufhyp, RED, COLOR_NONE);
 
 		break;
@@ -484,12 +494,19 @@ length_msg2(int x1, int y1, int x2, int y2, int x3, int y3)
 	    dx1 = x3 - x1;
 	    dy1 = y3 - y1;
 	    len1 = (float)(sqrt(dx1*dx1 + dy1*dy1));
+    } else {
+	    dx1 = 0.0;
+	    dy1 = 0.0;
     }
     if (x2 != -999) {
 	    dx2 = x3 - x2;
 	    dy2 = y3 - y2;
 	    len2 = (float)(sqrt(dx2*dx2 + dy2*dy2));
+    } else {
+	    dx2 = 0.0;
+	    dy2 = 0.0;
     }
+
     make_dimension_string(len1, len1str, False);
     make_dimension_string(len2, len2str, False);
     make_dimension_string(dx1, dx1str, False);
@@ -615,6 +632,7 @@ file_msg(char *format,...)
 static void
 clear_file_message(Widget w, XButtonEvent *ev)
 {
+	(void)w; (void)ev;
     XawTextBlock	block;
     int			replcode;
 
@@ -648,6 +666,8 @@ clear_file_message(Widget w, XButtonEvent *ev)
 static void
 file_msg_panel_dismiss(Widget w, XButtonEvent *ev)
 {
+	(void)w; (void)ev;
+
 	XtPopdown(file_msg_popup);
 	file_msg_is_popped=False;
 }
@@ -671,6 +691,7 @@ popup_file_msg(void)
 	NextArg(XtNy, 0);
 	NextArg(XtNcolormap, tool_cm);
 	NextArg(XtNtitle, "Xfig: Error messages");
+	NextArg(XtNtitleEncoding, XA_STRING);
 	file_msg_popup = XtCreatePopupShell("file_msg",
 					transientShellWidgetClass,
 					tool, Args, ArgCount);
@@ -692,12 +713,14 @@ popup_file_msg(void)
 	NextArg(XtNscrollVertical, XawtextScrollAlways);
 	NextArg(XtNright, XtChainRight);
 	NextArg(XtNbottom, XtChainBottom);
+	NextArg(XtNinternational, appres.international);
 	file_msg_win = XtCreateManagedWidget("file_msg_win", asciiTextWidgetClass,
 					     file_msg_panel, Args, ArgCount);
 	XtOverrideTranslations(file_msg_win,
 			   XtParseTranslationTable(file_msg2_translations));
 
 	FirstArg(XtNlabel, "Dismiss");
+	NextArg(XtNinternational, False);
 	NextArg(XtNheight, 25);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNfromVert, file_msg_win);
@@ -711,6 +734,7 @@ popup_file_msg(void)
 			  (XtEventHandler)file_msg_panel_dismiss, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Clear");
+	NextArg(XtNinternational, False);
 	NextArg(XtNheight, 25);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNfromVert, file_msg_win);
