@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2018 by Thomas Loimer
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,29 +16,40 @@
  *
  */
 
-#include "fig.h"
+#if defined HAVE_CONFIG_H && !defined VERSION
+#include "config.h"
+#endif
+#include "w_print.h"
+
+#include <errno.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <X11/StringDefs.h>
+#include <X11/Shell.h>
+#include <X11/Xatom.h>			/* XA_STRING */
+
 #include "figx.h"
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
-#include "e_edit.h"
-#include "u_create.h"
-#include "u_print.h"
-#include "w_export.h"
-#include "w_print.h"
-#include "w_icons.h"
-#include "w_indpanel.h"
-#include "w_msgpanel.h"
-#include "w_setup.h"
-#include "w_util.h"
-
 #include "f_util.h"
 #include "u_bound.h"
+#include "u_create.h"
+#include "u_print.h"
 #include "u_redraw.h"
 #include "w_canvas.h"
 #include "w_cmdpanel.h"
 #include "w_color.h"
 #include "w_cursor.h"
+#include "w_export.h"
+#include "w_icons.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_util.h"
+
 
 /* EXPORTS */
 
@@ -65,12 +76,14 @@ void	do_print(Widget w), do_print_batch(Widget w);
 DeclareStaticArgs(15);
 
 static char	print_msg[] = "PRINT";
+static int	fdbatch;
 static int	num_batch_figures=0;
 static Boolean	writing_batch=False;
 #define MAX_PRINTERS 1000		/* for those systems using lprng :-) */
 static char    *printer_names[MAX_PRINTERS];
 static int	parse_printcap(char **names);
 static int	numprinters;
+static int	print_command = 0;
 
 static Widget	beside;
 
@@ -90,7 +103,7 @@ static void	printer_select(Widget w, XtPointer new, XtPointer garbage);
 static void	switch_print_layers(Widget, XtPointer, XtPointer);
 
 static Widget	dismiss, print,
-		printer_text, param_text,
+		print_command_panel, printer_text, param_text,
 		clear_batch, print_batch,
 		num_batch,
 		printalltoggle, printactivetoggle, boundactivetoggle;
@@ -128,12 +141,18 @@ static XtActionsRec     prn_actions[] =
     {"UpdateMag", (XtActionProc) update_mag},
 };
 
+static const char	*print_command_items[] = {
+	" lp",	/* print_command == 0 */
+	"lpr"	/* print_command == 1 */
+};
 static void create_print_panel(Widget w);
 static void update_batch_count(void);
 
+
 static void
 print_panel_dismiss(Widget w, XButtonEvent *ev)
 {
+	(void)w; (void)ev;
     /* first get magnification in case it changed */
     /* the other things like paper size, justification, etc. are already
        updated because they are from menus */
@@ -175,16 +194,18 @@ do_print(Widget w)
 	get_grid_spec(grid, print_grid_minor_text, print_grid_major_text);
 
 	if (batch_exists) {
-	    gen_print_cmd(cmd,batch_file,printer_val,param_val);
-	    if (system(cmd) != 0)
-		file_msg("Error during PRINT");
+		if (print_spawn_printcmd(print_command, batch_file,
+						printer_val, param_val))
+			file_msg("Error during PRINT");
+	    put_msg("Printed batch file %s", batch_file);
 	    /* clear the batch file and the count */
 	    do_clear_batch(w);
+	    app_flush();
 	} else {
 	    strcpy(cmd, param_val);
 	    /* see if the user wants the filename in the param list (%f) */
 	    if (!strstr(cur_filename,"%f")) {	/* don't substitute if the filename has a %f */
-		while (c1=strstr(cmd,"%f")) {
+		while ((c1=strstr(cmd,"%f"))) {
 		    strcpy(cmd2, c1+2);		/* save tail */
 		    strcpy(c1, cur_filename);	/* change %f to filename */
 		    strcat(c1, cmd2);		/* append tail */
@@ -192,8 +213,9 @@ do_print(Widget w)
 	    }
 	    /* make a #rrggbb string from the background color */
 	    make_rgb_string(export_background_color, backgrnd);
-	    print_to_printer(printer_val, backgrnd, appres.magnification,
-				print_all_layers, bound_active_layers, grid, cmd);
+	    print_to_printer(print_command, printer_val, backgrnd,
+			    appres.magnification, print_all_layers,
+			    bound_active_layers, grid, cmd);
 	}
 }
 
@@ -266,6 +288,7 @@ get_magnif(void)
 static void
 update_mag(Widget widget, XtPointer item, XtPointer event)
 {
+	(void)widget; (void)item; (void)event;
     char	   *buf;
 
     buf = panel_get_value(print_mag_text);
@@ -291,12 +314,7 @@ update_figure_size(void)
 void
 do_print_batch(Widget w)
 {
-	FILE	*infp,*outfp;
-	char	tmp_exp_file[PATH_MAX];
-	char	str[255];
 	char	backgrnd[10], grid[80];
-	int	fd;
-	int	save_exp_lang;
 
 	if (writing_batch || emptyfigure_msg(print_msg))
 		return;
@@ -305,18 +323,25 @@ do_print_batch(Widget w)
 	/* this could happen if the user presses the button too fast */
 	writing_batch = True;
 
-	/* make a temporary name to write this figure to */
-	snprintf(tmp_exp_file, sizeof(tmp_exp_file), "%s/xfig-exp.XXXXXX",
-		TMPDIR);
-
 	if (batch_exists != True) {
+		int	flags;
 		/* make a temporary name to write the batch stuff to */
 		sprintf(batch_file, "%s/xfig-batch.XXXXXX", TMPDIR);
-		if ((fd = mkstemp(batch_file)) == -1) {
-			file_msg("Error creating temporary file");
+		if ((fdbatch = mkstemp(batch_file)) == -1) {
+			file_msg("Error creating temporary file %s: %s",
+					batch_file, strerror(errno));
 			return;
 		}
-		close(fd);
+		if ((flags = fcntl(fdbatch, F_GETFL)) == -1 ||
+				fcntl(fdbatch, F_SETFL, flags & O_APPEND)==-1) {
+			flags = errno;
+			(void)close(fdbatch);
+			(void)unlink(batch_file);
+			file_msg("Cannot append to batch file %s: %s",
+					batch_file, strerror(flags));
+			return;
+		}
+
 		batch_exists = True;
 	}
 	if (!print_popup)
@@ -331,37 +356,19 @@ do_print_batch(Widget w)
 	/* make a #rrggbb string from the background color */
 	make_rgb_string(export_background_color, backgrnd);
 
-	if ((fd = mkstemp(tmp_exp_file)) == -1) {
-		file_msg("Error creating temporary file");
-		return;
-	}
-	close(fd);
-
 	/* get grid params and assemble into fig2dev parm */
 	get_grid_spec(grid, print_grid_minor_text, print_grid_major_text);
 
-	save_exp_lang = cur_exp_lang;
-	cur_exp_lang = LANG_PS;
-	print_to_file(tmp_exp_file, 0,0, backgrnd, NULL, False, 0, grid);
-	cur_exp_lang = save_exp_lang;
-	put_msg("Appending to batch file \"%s\" (%s mode) ... done",
-		    batch_file, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
-	app_flush();		/* make sure message gets displayed */
-
-	/* now append that to the batch file */
-	if ((infp = fopen(tmp_exp_file, "rb")) == NULL) {
-		file_msg("Error during PRINT - cannot open temporary file to read");
-		return;
+	if (!print_to_batchfile(fdbatch, backgrnd, grid)) {
+		(void)fsync(fdbatch);
+		put_msg("Appending to batch file %s (%s mode) ... done",
+				batch_file,
+				appres.landscape ? "LANDSCAPE" : "PORTRAIT");
+		app_flush();		/* make sure message gets displayed */
+	} else {
+		file_msg("Error during PRINT - cannot append to batch file %s",
+				batch_file);
 	}
-	if ((outfp = fopen(batch_file, "ab")) == NULL) {
-		file_msg("Error during PRINT - cannot open print file to append");
-		return;
-	}
-	while (fgets(str,255,infp) != NULL)
-		(void) fputs(str,outfp);
-	fclose(infp);
-	fclose(outfp);
-	remove(tmp_exp_file);
 	/* count this batch figure */
 	num_batch_figures++ ;
 	/* and update the label widget */
@@ -373,7 +380,14 @@ do_print_batch(Widget w)
 static void
 do_clear_batch(Widget w)
 {
-	unlink(batch_file);
+	(void)w;
+
+	if (close(fdbatch))
+		file_msg("Error closing batch file %s: %s", batch_file,
+				strerror(errno));
+	if (unlink(batch_file))
+		file_msg("Error removing batch file %s: %s", batch_file,
+				strerror(errno));
 	batch_exists = False;
 	num_batch_figures = 0;
 	/* update the label widget */
@@ -403,6 +417,7 @@ void update_batch_count(void)
 static void
 orient_select(Widget w, XtPointer data, XtPointer garbage)
 {
+	(void)w; (void)garbage;
 	ptr_int		tmp = {data};
 	int new_orient = tmp.val;
 
@@ -418,6 +433,7 @@ orient_select(Widget w, XtPointer data, XtPointer garbage)
 static void
 just_select(Widget w, XtPointer new_just, XtPointer garbage)
 {
+	(void)garbage;
 
     FirstArg(XtNlabel, XtName(w));
     SetValues(print_just_panel);
@@ -430,6 +446,7 @@ just_select(Widget w, XtPointer new_just, XtPointer garbage)
 static void
 papersize_select(Widget w, XtPointer new_papersize, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     intptr_t papersize = (intptr_t) new_papersize;
 
     FirstArg(XtNlabel, paper_sizes[papersize].fname);
@@ -445,6 +462,7 @@ papersize_select(Widget w, XtPointer new_papersize, XtPointer garbage)
 static void
 multiple_select(Widget w, XtPointer new_multiple, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     intptr_t multiple = (intptr_t) new_multiple;
 
     FirstArg(XtNlabel, multiple_pages[multiple]);
@@ -479,6 +497,7 @@ multiple_select(Widget w, XtPointer new_multiple, XtPointer garbage)
 static void
 overlap_select(Widget w, XtPointer new_overlap, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     intptr_t overlap = (intptr_t) new_overlap;
 
     FirstArg(XtNlabel, overlap_pages[overlap]);
@@ -494,6 +513,7 @@ overlap_select(Widget w, XtPointer new_overlap, XtPointer garbage)
 static void
 background_select(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	(void)call_data;
     Pixel	    bgcolor, fgcolor;
     ptr_int	    closure = {client_data};
 
@@ -521,6 +541,7 @@ background_select(Widget w, XtPointer client_data, XtPointer call_data)
 void
 print_grid_minor_select(Widget w, XtPointer new_grid_choice, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     char	buf[MAX_GRID_STRLEN];
     char	*val;
     ptr_int	grid = {new_grid_choice};
@@ -545,6 +566,7 @@ print_grid_minor_select(Widget w, XtPointer new_grid_choice, XtPointer garbage)
 void
 print_grid_major_select(Widget w, XtPointer new_grid_choice, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     char	buf[MAX_GRID_STRLEN];
     char	*val;
     ptr_int	grid = {new_grid_choice};
@@ -564,9 +586,21 @@ print_grid_major_select(Widget w, XtPointer new_grid_choice, XtPointer garbage)
     SetValues(print_grid_major_text);
 }
 
+static void
+print_command_select(Widget w, XtPointer data, XtPointer garbage)
+{
+	(void)w; (void)garbage;
+	ptr_int		cmd = {data};
+
+	FirstArg(XtNlabel, XtName(w));
+	SetValues(print_command_panel);
+	print_command = cmd.val;
+}
+
 static void
 printer_select(Widget w, XtPointer new_printer, XtPointer garbage)
 {
+	(void)w; (void)garbage;
     intptr_t printer = (intptr_t) new_printer;
 
     /* put selected printer in the menu button */
@@ -643,6 +677,7 @@ popup_print_panel(Widget w)
 
 void create_print_panel(Widget w)
 {
+	(void)w;
 	Widget	    image;
 	Widget	    entry,mag_spinner, below, fitpage;
 	Pixmap	    p;
@@ -660,6 +695,7 @@ void create_print_panel(Widget w)
 	FirstArg(XtNx, xposn+50);
 	NextArg(XtNy, yposn+50);
 	NextArg(XtNtitle, "Xfig: Print menu");
+	NextArg(XtNtitleEncoding, XA_STRING);
 	NextArg(XtNcolormap, tool_cm);
 	print_popup = XtCreatePopupShell("print_popup",
 					 transientShellWidgetClass,
@@ -674,6 +710,7 @@ void create_print_panel(Widget w)
 	/* start with the picture of the printer */
 
 	FirstArg(XtNlabel, "   ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNwidth, printer_ic.width);
 	NextArg(XtNheight, printer_ic.height);
 	NextArg(XtNborderWidth, 0);
@@ -695,6 +732,7 @@ void create_print_panel(Widget w)
 	SetValues(image);
 
 	FirstArg(XtNlabel, "Print to PostScript Printer");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, image);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -705,6 +743,7 @@ void create_print_panel(Widget w)
 					print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, " Magnification %");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, image);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -728,6 +767,7 @@ void create_print_panel(Widget w)
 	/* Fit Page to the right of the magnification window */
 
 	FirstArg(XtNlabel, "Fit to Page");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, image);
 	NextArg(XtNfromHoriz, mag_spinner);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -750,6 +790,7 @@ void create_print_panel(Widget w)
 		(float)(ux-lx)/mult*appres.magnification/100.0,unit,
 		(float)(uy-ly)/mult*appres.magnification/100.0,unit);
 	FirstArg(XtNlabel, buf);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, image);
 	NextArg(XtNfromHoriz, fitpage);
 	NextArg(XtNhorizDistance, 5);
@@ -764,6 +805,7 @@ void create_print_panel(Widget w)
 	/* paper size */
 
 	FirstArg(XtNlabel, "      Paper Size");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, fitpage);
 	NextArg(XtNtop, XtChainTop);
@@ -774,6 +816,7 @@ void create_print_panel(Widget w)
 					 print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, paper_sizes[appres.papersize].fname);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNfromVert, fitpage);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -799,6 +842,7 @@ void create_print_panel(Widget w)
 	/* Orientation */
 
 	FirstArg(XtNlabel, "     Orientation");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, print_papersize_panel);
 	NextArg(XtNtop, XtChainTop);
@@ -816,13 +860,15 @@ void create_print_panel(Widget w)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
-	print_orient_panel = XtCreateManagedWidget(orient_items[appres.landscape],
-					     menuButtonWidgetClass,
-					     print_panel, Args, ArgCount);
+	NextArg(XtNinternational, False);
+	print_orient_panel = XtCreateManagedWidget(
+			orient_items[(int)appres.landscape],
+			menuButtonWidgetClass, print_panel, Args, ArgCount);
 	make_pulldown_menu(orient_items, XtNumber(orient_items), -1, "",
 				      print_orient_panel, orient_select);
 
 	FirstArg(XtNlabel, "Justification");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromHoriz, print_orient_panel);
 	NextArg(XtNfromVert, print_papersize_panel);
@@ -834,6 +880,7 @@ void create_print_panel(Widget w)
 					 print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, just_items[appres.flushleft? 1 : 0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, just_lab);
 	NextArg(XtNfromVert, print_papersize_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -851,6 +898,7 @@ void create_print_panel(Widget w)
 	/* multiple/single page */
 
 	FirstArg(XtNlabel, "           Pages");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, print_just_panel);
 	NextArg(XtNtop, XtChainTop);
@@ -861,6 +909,7 @@ void create_print_panel(Widget w)
 					 print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, multiple_pages[appres.multiple? 1:0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNfromVert, print_just_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -876,6 +925,7 @@ void create_print_panel(Widget w)
 				    print_multiple_panel, multiple_select);
 
 	FirstArg(XtNlabel, overlap_pages[appres.overlap? 1:0]);
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, print_multiple_panel);
 	NextArg(XtNfromVert, print_just_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -893,6 +943,7 @@ void create_print_panel(Widget w)
 	/* background color */
 
 	FirstArg(XtNlabel, "Background");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromHoriz, print_overlap_panel);
 	NextArg(XtNfromVert, print_just_panel);
 	NextArg(XtNborderWidth, 0);
@@ -912,6 +963,7 @@ void create_print_panel(Widget w)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, False);
 	print_background_panel = XtCreateManagedWidget("background",
 					   menuButtonWidgetClass,
 					   print_panel, Args, ArgCount);
@@ -925,6 +977,7 @@ void create_print_panel(Widget w)
 					NO_TRANSP, INCL_BACKG);
 	/* grid options */
 	FirstArg(XtNlabel, "            Grid");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, print_background_panel);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -940,9 +993,40 @@ void create_print_panel(Widget w)
 				&print_grid_unit_label,
 				print_grid_major_select, print_grid_minor_select);
 
+	/* print command */
+
+	FirstArg(XtNlabel, "   Print Command");
+	NextArg(XtNinternational, False);
+	NextArg(XtNfromVert, below);
+	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	beside = XtCreateManagedWidget("printcommand_label", labelWidgetClass,
+					    print_panel, Args, ArgCount);
+
+	FirstArg(XtNlabel, print_command_items[print_command]);
+	NextArg(XtNinternational, False);
+	NextArg(XtNfromHoriz, beside);
+	NextArg(XtNfromVert, below);
+	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNleftBitmap, menu_arrow);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	print_command_panel = XtCreateManagedWidget("printcommand",
+			menuButtonWidgetClass, print_panel, Args, ArgCount);
+	make_pulldown_menu((char **)print_command_items,
+			XtNumber(print_command_items), -1, "",
+			print_command_panel, print_command_select);
+
 	/* printer name */
 
-	FirstArg(XtNlabel, "         Printer");
+	FirstArg(XtNlabel, "Printer");
+	NextArg(XtNinternational, False);
+	NextArg(XtNfromHoriz, print_command_panel);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -967,6 +1051,7 @@ void create_print_panel(Widget w)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, False);
 	printer_text = XtCreateManagedWidget("printer", asciiTextWidgetClass,
 					     print_panel, Args, ArgCount);
 
@@ -1016,6 +1101,7 @@ void create_print_panel(Widget w)
 	    strcat(buf," ");
 	if (numprinters > 0) {
 	    FirstArg(XtNlabel, buf);
+	    NextArg(XtNinternational, False);
 	    NextArg(XtNfromHoriz, printer_text);
 	    NextArg(XtNfromVert, below);
 	    NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1032,6 +1118,7 @@ void create_print_panel(Widget w)
 	}
 
 	FirstArg(XtNlabel, "Print Job Params");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, printer_text);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1045,7 +1132,7 @@ void create_print_panel(Widget w)
 	 * job parameters in a resource, e.g.:	 *param*string: -K2
 	 */
 
-	FirstArg(XtNwidth, 200);
+	FirstArg(XtNwidth, 320);
 	NextArg(XtNfromVert, printer_text);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNeditType, XawtextEdit);
@@ -1055,6 +1142,7 @@ void create_print_panel(Widget w)
 	NextArg(XtNbottom, XtChainTop);
 	NextArg(XtNleft, XtChainLeft);
 	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNinternational, False);
 	param_text = XtCreateManagedWidget("job_params", asciiTextWidgetClass,
 					     print_panel, Args, ArgCount);
 
@@ -1062,6 +1150,7 @@ void create_print_panel(Widget w)
 			       XtParseTranslationTable(print_translations));
 
 	FirstArg(XtNlabel, "Figures in batch");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, param_text);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNtop, XtChainTop);
@@ -1072,6 +1161,7 @@ void create_print_panel(Widget w)
 					    print_panel, Args, ArgCount);
 	FirstArg(XtNwidth, 30);
 	NextArg(XtNlabel, "  0");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, param_text);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNborderWidth, INTERNAL_BW);
@@ -1083,6 +1173,7 @@ void create_print_panel(Widget w)
 					     print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, "Dismiss");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNhorizDistance, 6);
@@ -1105,6 +1196,7 @@ void create_print_panel(Widget w)
 	/* print buttons */
 
 	FirstArg(XtNlabel, "Print FIGURE\nto Printer");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, beside);
 	NextArg(XtNresize, False);	/* must not allow resize because the label changes */
@@ -1122,6 +1214,7 @@ void create_print_panel(Widget w)
 			  (XtEventHandler)do_print, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Print FIGURE\nto Batch");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, print);
 	NextArg(XtNheight, 35);
@@ -1138,6 +1231,7 @@ void create_print_panel(Widget w)
 			  (XtEventHandler)do_print_batch, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Clear\nBatch");
+	NextArg(XtNinternational, False);
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, print_batch);
 	NextArg(XtNheight, 35);
@@ -1183,6 +1277,7 @@ void create_print_panel(Widget w)
 static void /* XtCallbackProc */
 switch_print_layers(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure; (void)call_data;
     Boolean	    state;
     intptr_t	    which;
 
@@ -1225,6 +1320,7 @@ switch_print_layers(Widget w, XtPointer closure, XtPointer call_data)
 static void /* XtCallbackProc */
 switch_bound(Widget w, XtPointer closure, XtPointer call_data)
 {
+	(void)closure; (void)call_data;
     Boolean	    state;
 
     /* check state of the toggle and set/remove checkmark */
@@ -1355,6 +1451,7 @@ make_layer_choice(char *label_all, char *label_active, Widget parent, Widget bel
 	NextArg(XtNinternalWidth, 1);
 	NextArg(XtNinternalHeight, 1);
 	NextArg(XtNlabel, "  ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNsensitive, (print_all_layers? False : True)); /* make opposite button sens */
 	NextArg(XtNstate, print_all_layers);	/* initial state */
 	NextArg(XtNradioData, 1);		/* when this is pressed the value is 1 */
@@ -1366,6 +1463,7 @@ make_layer_choice(char *label_all, char *label_active, Widget parent, Widget bel
 	/* label - " XXXX all layers" */
 
 	FirstArg(XtNlabel, label_all);
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromHoriz, printalltoggle);
 	NextArg(XtNtop, XtChainTop);
@@ -1386,6 +1484,7 @@ make_layer_choice(char *label_all, char *label_active, Widget parent, Widget bel
 	NextArg(XtNinternalWidth, 1);
 	NextArg(XtNinternalHeight, 1);
 	NextArg(XtNlabel, "  ");
+	NextArg(XtNinternational, False);
 	NextArg(XtNsensitive, (print_all_layers? True : False)); /* make opposite button sens */
 	NextArg(XtNstate, !print_all_layers);	/* initial state */
 	NextArg(XtNradioData, 2);		/* when this is pressed the value is 2 */
@@ -1398,6 +1497,7 @@ make_layer_choice(char *label_all, char *label_active, Widget parent, Widget bel
 	/* label - "XXXX only active" */
 
 	FirstArg(XtNlabel, label_active);
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, printalltoggle);
 	NextArg(XtNfromHoriz, printactivetoggle);
@@ -1427,6 +1527,7 @@ make_layer_choice(char *label_all, char *label_active, Widget parent, Widget bel
 					(XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Boundary only active layers");
+	NextArg(XtNinternational, False);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, printalltoggle);
 	NextArg(XtNfromHoriz, boundactivetoggle);
