@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2018 by Thomas Loimer
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,6 +16,11 @@
  *
  */
 
+#ifndef W_PRINT_H
+#define W_PRINT_H
+
+#include <X11/Intrinsic.h>
+
 extern Widget	print_popup;	/* the main print popup */
 extern Widget	print_panel;	/* the form it's in */
 extern Widget	print_orient_panel;
@@ -41,3 +46,5 @@ extern void	print_grid_minor_select(Widget w, XtPointer new_grid_choice,
 					XtPointer garbage);
 extern void	print_grid_major_select(Widget w, XtPointer new_grid_choice,
 					XtPointer garbage);
+
+#endif
