@@ -1,45 +1,58 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and documentation
  * files (the "Software"), including without limitation the rights to use,
- * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies of
  * the Software, and to permit persons who receive copies from any such
  * party to do so, with the only requirement being that the above copyright
  * and this permission notice remain intact.
  *
  */
 
-#include "fig.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "w_rulers.h"
+
+#include <math.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Intrinsic.h>
+#include <X11/Xatom.h>		/* XA_STRING */
+
 #include "figx.h"
 #include "resources.h"
-#include "mode.h"
-#include "paintop.h"
-#include "u_pan.h"
-#include "w_drawprim.h"
-#include "w_icons.h"
-#include "w_indpanel.h"
-#include "w_mousefun.h"
-#include "w_msgpanel.h"
-#include "w_rulers.h"
-#include "w_setup.h"
-#include "w_util.h"
-#include "w_zoom.h"
 #include "object.h"
+#include "mode.h"
 
 #include "f_util.h"
+#include "u_pan.h"
 #include "u_redraw.h"
 #include "u_scale.h"
 #include "w_canvas.h"
 #include "w_color.h"
+#include "w_drawprim.h"
 #include "w_export.h"
 #include "w_grid.h"
+#include "w_icons.h"
+#include "w_mousefun.h"
+#include "w_msgpanel.h"
 #include "w_print.h"
+#include "w_setup.h"
+#include "w_util.h"
+#include "w_zoom.h"
+#include "xfig_math.h"
 
 /*
  * The following will create rulers the same size as the initial screen size.
@@ -356,6 +369,7 @@ init_unitbox(Widget tool)
     }
 
     FirstArg(XtNlabel, buf);
+    NextArg(XtNinternational, False);
     NextArg(XtNwidth, UNITBOX_WD);
     NextArg(XtNheight, RULER_WD);
     NextArg(XtNfromHoriz, topruler_sw);
@@ -405,11 +419,12 @@ void update_rulerpanel()
     else
 	sprintf(msg + strlen(msg), "(right button)");
 
-    if (unitbox_sw)
+    if (unitbox_sw) {
 	if (appres.showballoons)
 	    XawTipEnable(unitbox_sw, msg);
 	else
 	    XawTipDisable(unitbox_sw);
+    }
 }
 #else
 /* come here when the mouse passes over the unit box */
@@ -460,6 +475,7 @@ unit_balloon(void)
 	    NextArg(XtNleftBitmap, mouse_l);	/* bitmap of mouse with left button pushed */
 	}
 	NextArg(XtNlabel, "Pan to (0,0)   ");
+	NextArg(XtNinternational, False);
 	balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
 				    box, Args, ArgCount);
 	FirstArg(XtNborderWidth, 0);
@@ -469,6 +485,7 @@ unit_balloon(void)
 	    NextArg(XtNleftBitmap, mouse_r);	/* bitmap of mouse with right button pushed */
 	}
 	NextArg(XtNlabel, "Set Units/Scale");
+	NextArg(XtNinternational, False);
 	balloon_label = XtCreateManagedWidget("r_label", labelWidgetClass,
 				box, Args, ArgCount);
 	XtRealizeWidget(unit_balloon_popup);
@@ -521,12 +538,17 @@ unit_panel_dismiss(void)
 static void
 unit_panel_cancel(Widget w, XButtonEvent *ev)
 {
-    unit_panel_dismiss();
+	(void)w;
+	(void)ev;
+
+	unit_panel_dismiss();
 }
 
 static void
 unit_panel_set(Widget w, XButtonEvent *ev)
 {
+	(void)w;
+	(void)ev;
     int		    old_rul_unit, pix;
     float	    scale_factor;
 
@@ -620,6 +642,8 @@ set_unit_indicator(Boolean use_userscale)
 static void
 fig_unit_select(Widget w, XtPointer new_unit, XtPointer garbage)
 {
+	(void)garbage;
+
     FirstArg(XtNlabel, XtName(w));
     SetValues(fig_unit_panel);
     fig_unit_setting = (Boolean) (intptr_t) new_unit;
@@ -640,6 +664,8 @@ fig_unit_select(Widget w, XtPointer new_unit, XtPointer garbage)
 static void
 fig_scale_select(Widget w, XtPointer new_scale, XtPointer garbage)
 {
+	(void)garbage;
+
     FirstArg(XtNlabel, XtName(w));
     SetValues(fig_scale_panel);
     fig_scale_setting = (Boolean) (intptr_t) new_scale;
@@ -660,6 +686,7 @@ fig_scale_select(Widget w, XtPointer new_scale, XtPointer garbage)
 static void
 rul_unit_select(Widget w, XtPointer closure, XtPointer garbage)
 {
+	(void)garbage;
     intptr_t	    new_unit = (intptr_t) closure;
 
     /* return if no change */
@@ -734,6 +761,7 @@ popup_unit_panel(void)
     NextArg(XtNy, y_val);
     NextArg(XtNcolormap, tool_cm);
     NextArg(XtNtitle, "Xfig: Unit menu");
+    NextArg(XtNtitleEncoding, XA_STRING);
     unit_popup = XtCreatePopupShell("unit_popup",
 				    transientShellWidgetClass, tool,
 				    Args, ArgCount);
@@ -747,6 +775,7 @@ popup_unit_panel(void)
     unit_panel = XtCreateManagedWidget("unit_panel", formWidgetClass, unit_popup, NULL, 0);
 
     FirstArg(XtNborderWidth, 0);
+    NextArg(XtNinternational, False);
     label = XtCreateManagedWidget("         Unit/Scale settings          ",
 		labelWidgetClass, unit_panel, Args, ArgCount);
 
@@ -755,12 +784,14 @@ popup_unit_panel(void)
     rul_unit_setting = appres.INCHES ? True : False;
     FirstArg(XtNfromVert, label);
     NextArg(XtNborderWidth, 0);
+    NextArg(XtNinternational, False);
     beside = XtCreateManagedWidget(" Ruler units", labelWidgetClass,
                                    unit_panel, Args, ArgCount);
 
     FirstArg(XtNfromVert, label);
     NextArg(XtNfromHoriz, beside);
     NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
+    NextArg(XtNinternational, False);
     rul_unit_panel = XtCreateManagedWidget(rul_unit_items[cur_gridunit],
 				menuButtonWidgetClass, unit_panel, Args, ArgCount);
     below = rul_unit_panel;
@@ -771,12 +802,14 @@ popup_unit_panel(void)
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
+    NextArg(XtNinternational, False);
     beside = XtCreateManagedWidget("Figure units", labelWidgetClass,
                                    unit_panel, Args, ArgCount);
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, beside);
     NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
+    NextArg(XtNinternational, False);
     fig_unit_panel = XtCreateManagedWidget(fig_unit_items[(int) fig_unit_setting],
 				menuButtonWidgetClass, unit_panel, Args, ArgCount);
     below = fig_unit_panel;
@@ -788,6 +821,7 @@ popup_unit_panel(void)
     FirstArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNlabel, "   Unit name");
+    NextArg(XtNinternational, False);
     user_unit_lab = XtCreateManagedWidget("user_units",
                                 labelWidgetClass, unit_panel, Args, ArgCount);
 
@@ -797,6 +831,7 @@ popup_unit_panel(void)
     NextArg(XtNstring, cur_fig_units);
     NextArg(XtNeditType, XawtextEdit);
     NextArg(XtNwidth, 50);
+    NextArg(XtNinternational, False);
     user_unit_entry = XtCreateManagedWidget("unit_entry", asciiTextWidgetClass,
 					unit_panel, Args, ArgCount);
     XtOverrideTranslations(user_unit_entry,
@@ -814,14 +849,17 @@ popup_unit_panel(void)
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
+    NextArg(XtNinternational, False);
     beside = XtCreateManagedWidget("Figure scale", labelWidgetClass,
                                    unit_panel, Args, ArgCount);
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, beside);
     NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
-    fig_scale_panel = XtCreateManagedWidget(fig_scale_items[fig_scale_setting],
-				menuButtonWidgetClass, unit_panel, Args, ArgCount);
+    NextArg(XtNinternational, False);
+    fig_scale_panel = XtCreateManagedWidget(
+		    fig_scale_items[(int)fig_scale_setting],
+		    menuButtonWidgetClass, unit_panel, Args, ArgCount);
     below = fig_scale_panel;
     make_pulldown_menu(fig_scale_items, XtNumber(fig_scale_items), -1, "",
                                      fig_scale_panel, fig_scale_select);
@@ -831,6 +869,7 @@ popup_unit_panel(void)
     FirstArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNlabel, "Scale factor");
+    NextArg(XtNinternational, False);
     scale_factor_lab = XtCreateManagedWidget("scale_factor",
                                 labelWidgetClass, unit_panel, Args, ArgCount);
 
@@ -839,6 +878,7 @@ popup_unit_panel(void)
     NextArg(XtNborderWidth, INTERNAL_BW);
     NextArg(XtNfromHoriz, scale_factor_lab);
     NextArg(XtNstring, buf);
+    NextArg(XtNinternational, False);
     NextArg(XtNeditType, XawtextEdit);
     NextArg(XtNwidth, 50);
     scale_factor_entry = XtCreateManagedWidget("factor_entry", asciiTextWidgetClass,
@@ -850,6 +890,7 @@ popup_unit_panel(void)
     /* standard set/cancel buttons */
 
     FirstArg(XtNlabel, "Cancel");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNborderWidth, INTERNAL_BW);
     cancel = XtCreateManagedWidget("cancel", commandWidgetClass,
@@ -858,6 +899,7 @@ popup_unit_panel(void)
 		      (XtEventHandler)unit_panel_cancel, (XtPointer) NULL);
 
     FirstArg(XtNlabel, " Set  ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, cancel);
     NextArg(XtNborderWidth, INTERNAL_BW);
@@ -929,6 +971,7 @@ static String	topruler_translations =
 static void
 topruler_selected(Widget tool, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)tool; (void)params; (void)nparams;
     XButtonEvent   *be = (XButtonEvent *) event;
 
     /* see if wheel mouse */
@@ -1013,6 +1056,8 @@ void set_toprulermark(int x)
 static void
 topruler_exposed(Widget tool, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)tool; (void)params; (void)nparams;
+
     if (((XExposeEvent *) event)->count > 0)
 	return;
     redisplay_topruler();
@@ -1029,6 +1074,7 @@ init_topruler(Widget tool)
     FirstArg(XtNwidth, TOPRULER_WD);
     NextArg(XtNheight, TOPRULER_HT);
     NextArg(XtNlabel, "");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, mode_panel);
     NextArg(XtNhorizDistance, -INTERNAL_BW);
     NextArg(XtNfromVert, msg_panel);
@@ -1256,6 +1302,7 @@ static String	sideruler_translations =
 static void
 sideruler_selected(Widget tool, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)tool; (void)params; (void)nparams;
     XButtonEvent   *be = (XButtonEvent *) event;
 
     /* see if wheel mouse */
@@ -1322,6 +1369,8 @@ sideruler_selected(Widget tool, XButtonEvent *event, String *params, Cardinal *n
 static void
 sideruler_exposed(Widget tool, XButtonEvent *event, String *params, Cardinal *nparams)
 {
+	(void)tool; (void)params; (void)nparams;
+
     if (((XExposeEvent *) event)->count > 0)
 	return;
     redisplay_sideruler();
@@ -1333,6 +1382,7 @@ init_sideruler(Widget tool)
     FirstArg(XtNwidth, SIDERULER_WD);
     NextArg(XtNheight, SIDERULER_HT);
     NextArg(XtNlabel, "");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, canvas_sw);
     NextArg(XtNfromVert, topruler_sw);
     NextArg(XtNvertDistance, -INTERNAL_BW);
