@@ -1,8 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,13 +16,14 @@
  *
  */
 
-#include "fig.h"
+#include "w_setup.h"
+
+#include <math.h>
+
 #include "resources.h"
 #include "w_cmdpanel.h"
 #include "w_icons.h"
-#include "w_indpanel.h"
-#include "w_setup.h"
-#include "w_util.h"
+
 
 int		TOOL_WD, TOOL_HT;
 int		CMDFORM_WD, CMDFORM_HT = CMD_BUT_HT;
