@@ -1,7 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1997 by T. Sato
- * Parts Copyright (c) 1997-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 1997 by T. Sato
+ * Parts Copyright (c) 2016-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -40,30 +43,39 @@ There is currently no way to undo replace/update operations.
 
 ****************************************************************/
 
-#include "fig.h"
+#include "w_srchrepl.h"
+
+#include <errno.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#include <unistd.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/IntrinsicP.h>    /* includes X11/Xlib.h, which includes X11/X.h */
+#include <X11/Xatom.h>		/* XA_STRING */
+
 #include "figx.h"
 #include "resources.h"
 #include "object.h"
-#include "d_text.h"
+#include "mode.h"
 #include "e_update.h"
 #include "f_util.h"
-#include "w_drawprim.h"
-#include "w_indpanel.h"
-#include "w_listwidget.h"
-#include "w_msgpanel.h"
-#include "w_srchrepl.h"
-#include "w_setup.h"
-#include "w_util.h"
-#include "u_create.h"
-
-#include "mode.h"
 #include "u_bound.h"
+#include "u_create.h"
 #include "u_fonts.h"
 #include "u_redraw.h"
 #include "w_canvas.h"
 #include "w_color.h"
+#include "w_listwidget.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
+#include "w_util.h"
 
-#include <stdarg.h>
 
 #define MAX_MISSPELLED_WORDS	200
 #define	SEARCH_WIDTH		496	/* width of search message and results */
@@ -154,6 +166,7 @@ static Boolean compare_string(char *str, char *pattern)
 static void
 do_replace(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
   int cnt;
 
   if (found_text_cnt > 0) {
@@ -180,9 +193,8 @@ replace_text_in_compound(F_compound *com, char *pattern, char *dst)
 {
   F_compound	*c;
   F_text	*t;
-  PR_SIZE	 size;
   Boolean	 replaced, processed;
-  int		 pat_len, i, j;
+  size_t	 pat_len, i, j;
   char		 str[300];
 
   pat_len = strlen(pattern);
@@ -225,11 +237,7 @@ replace_text_in_compound(F_compound *com, char *pattern, char *dst)
           t->cstring = new_string(strlen(str));
         }
         strcpy(t->cstring, str);
-        size = textsize(lookfont(x_fontnum(psfont_text(t), t->font),
-				t->size), strlen(t->cstring), t->cstring);
-        t->ascent = size.ascent;
-        t->descent = size.descent;
-        t->length = size.length;
+	font_setbbox_offset(t);
         processed = True;
       }
     }
@@ -243,6 +251,8 @@ replace_text_in_compound(F_compound *com, char *pattern, char *dst)
 static void
 do_update(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
+
   if (found_text_cnt > 0) {
     search_text_in_compound(&objects,
             panel_get_value(search_text_widget), update_text);
@@ -332,6 +342,7 @@ show_text_object(F_text *t)
 static void
 search_and_replace_text(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
   char	*string;
 
   show_search_msg("Searching text...");
@@ -370,8 +381,9 @@ search_text_in_compound(F_compound *com, char *pattern, void (*proc) (/* ??? */)
   F_compound *c;
   F_text *t;
   Boolean match, processed;
-  int pat_len, i;
+  size_t pat_len, i;
   processed = False;
+
   for (c = com->compounds; c != NULL; c = c->next) {
     if (search_text_in_compound(c, pattern, proc))
 	processed = True;
@@ -402,6 +414,7 @@ search_text_in_compound(F_compound *com, char *pattern, void (*proc) (/* ??? */)
 static void
 search_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
   found_text_panel_dismiss();
   if (search_panel != None)
 	XtDestroyWidget(search_panel);
@@ -438,6 +451,8 @@ popup_search_panel(void)
     NextArg(XtNy, (Position) ry);
     NextArg(XtNcolormap, tool_cm);
     NextArg(XtNtitle, "Xfig: Search & Replace");
+    NextArg(XtNtitleEncoding, XA_STRING);
+    NextArg(XtNinput, True);
 
     search_panel = XtCreatePopupShell("search_panel",
 				transientShellWidgetClass, tool,
@@ -452,6 +467,7 @@ popup_search_panel(void)
     form = XtCreateManagedWidget("form", formWidgetClass, search_panel, NULL, 0) ;
 
     FirstArg(XtNlabel, "  Search for:");
+    NextArg(XtNinternational, False);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -467,6 +483,7 @@ popup_search_panel(void)
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternational, appres.international);
     search_text_widget = XtCreateManagedWidget("search_text", asciiTextWidgetClass,
 				form, Args, ArgCount);
     XtOverrideTranslations(search_text_widget,
@@ -475,6 +492,7 @@ popup_search_panel(void)
     /* search button */
 
     FirstArg(XtNlabel, "Search ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromHoriz, search_text_widget);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -493,6 +511,7 @@ popup_search_panel(void)
     NextArg(XtNvertDistance, 6);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNlabel, "Replace with:");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
@@ -509,6 +528,7 @@ popup_search_panel(void)
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternational, appres.international);
     replace_text_widget = XtCreateManagedWidget("replace_text", asciiTextWidgetClass,
 				form, Args, ArgCount);
     XtOverrideTranslations(replace_text_widget,
@@ -517,6 +537,7 @@ popup_search_panel(void)
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, replace_text_widget);
     NextArg(XtNlabel, "Replace");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
@@ -529,6 +550,7 @@ popup_search_panel(void)
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, do_replace_button);
     NextArg(XtNlabel, "UPDATE  settings");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
@@ -543,6 +565,7 @@ popup_search_panel(void)
     /* make a label to report if no match for search */
 
     FirstArg(XtNlabel, "Enter search string and press \"Search\"");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, below);
     NextArg(XtNjustify, XtJustifyLeft);
     NextArg(XtNwidth, SEARCH_WIDTH);
@@ -569,6 +592,7 @@ popup_search_panel(void)
     NextArg(XtNbottom, XtChainBottom);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternational, appres.international);
     search_results_win = XtCreateManagedWidget("search_results_win", asciiTextWidgetClass,
 			form, Args, ArgCount);
     XtOverrideTranslations(search_results_win,
@@ -580,6 +604,7 @@ popup_search_panel(void)
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNlabel, "Dismiss");
+    NextArg(XtNinternational, False);
     NextArg(XtNtop, XtChainBottom);
     NextArg(XtNbottom, XtChainBottom);
     NextArg(XtNleft, XtChainLeft);
@@ -622,6 +647,8 @@ popup_spell_check_panel(char **list, int nitems)
     NextArg(XtNy, (Position) y_val);
     NextArg(XtNcolormap, tool_cm);
     NextArg(XtNtitle, "Xfig: Misspelled words");
+    NextArg(XtNtitleEncoding, XA_STRING);
+    NextArg(XtNinput, True);
     spell_check_panel = XtCreatePopupShell("spell_check_panel",
 				transientShellWidgetClass,
 				tool, Args, ArgCount);
@@ -638,6 +665,7 @@ popup_spell_check_panel(char **list, int nitems)
     /* make a label to report either "No misspelled words" or "Misspelled words:" */
 
     FirstArg(XtNlabel, "Spell checker");
+    NextArg(XtNinternational, False);
     NextArg(XtNjustify, XtJustifyLeft);
     NextArg(XtNwidth, 375);
     NextArg(XtNheight, 20);
@@ -650,6 +678,7 @@ popup_spell_check_panel(char **list, int nitems)
 
     /* labels for list and correct word entry */
     FirstArg(XtNlabel, "Misspelled words    ");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, spell_msg_win);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
@@ -660,6 +689,7 @@ popup_spell_check_panel(char **list, int nitems)
     below = label;
 
     FirstArg(XtNlabel, "Correction");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, spell_msg_win);
     NextArg(XtNfromHoriz, label);
     NextArg(XtNtop, XtChainTop);
@@ -685,6 +715,7 @@ popup_spell_check_panel(char **list, int nitems)
 
     /* now make the list widget */
     FirstArg(XtNlist, list);
+    NextArg(XtNinternational, appres.international);
     NextArg(XtNnumberStrings, nitems);
     NextArg(XtNforceColumns, True);		/* force to one column */
     NextArg(XtNdefaultColumns, 1);		/* ditto */
@@ -705,6 +736,7 @@ popup_spell_check_panel(char **list, int nitems)
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternational, appres.international);
     correct_word = XtCreateManagedWidget("correct_word", asciiTextWidgetClass,
 				     form, Args, ArgCount);
 
@@ -717,6 +749,7 @@ popup_spell_check_panel(char **list, int nitems)
 
     /* now "Correct" button to the right */
     FirstArg(XtNlabel, "Correct");
+    NextArg(XtNinternational, False);
     NextArg(XtNsensitive, False);		/* start insensitive */
     NextArg(XtNfromVert, below);
     NextArg(XtNvertDistance, 1);
@@ -733,6 +766,7 @@ popup_spell_check_panel(char **list, int nitems)
     /* make a re-check spelling button at bottom of whole panel */
 
     FirstArg(XtNlabel, "Recheck");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, spell_viewport);
     NextArg(XtNsensitive, False);		/* insensitive to start */
     NextArg(XtNtop, XtChainBottom);
@@ -747,6 +781,7 @@ popup_spell_check_panel(char **list, int nitems)
     /* make dismiss button to the right of the recheck button */
 
     FirstArg(XtNlabel, "Dismiss");
+    NextArg(XtNinternational, False);
     NextArg(XtNfromVert, spell_viewport);
     NextArg(XtNfromHoriz, recheck_button);
     NextArg(XtNtop, XtChainBottom);
@@ -774,6 +809,8 @@ popup_spell_check_panel(char **list, int nitems)
 static void
 spell_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
+
   if (spell_check_panel != None)
 	XtDestroyWidget(spell_check_panel);
   spell_check_panel = None;
@@ -881,6 +918,7 @@ write_text_from_compound(FILE *fp, F_compound *com)
 static void
 spell_select_word(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure;
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
     /* make correct button and correction entry sensitive */
@@ -901,6 +939,7 @@ spell_select_word(Widget widget, XtPointer closure, XtPointer call_data)
 static void
 spell_correct_word(Widget widget, XtPointer closure, XtPointer call_data)
 {
+	(void)widget; (void)closure; (void)call_data;
     char	   *corrected_word;
 
     /* get the correct word from the ascii widget */
@@ -925,4 +964,3 @@ show_spell_msg(char *format,...)
   FirstArg(XtNlabel, tmpstr);
   SetValues(spell_msg_win);
 }
-
