@@ -1,7 +1,9 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * This file Copyright (c) 2002 Stephane Mancini
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,15 +16,19 @@
  *
  */
 
+#ifndef W_STYLE_H
+#define W_STYLE_H
+
+#include <X11/Intrinsic.h>
+
 extern void	init_manage_style_panel(void);
 extern void	setup_manage_style_panel(void);
 extern void	add_style_actions(void);
 extern void	popup_manage_style_panel(void);
 extern int	confirm_close_style(void);
-extern void add_style_actions(void);
-extern void init_manage_style_panel (void);
-extern void setup_manage_style_panel (void);
-
+extern void	add_style_actions(void);
+extern void	init_manage_style_panel(void);
+extern void	setup_manage_style_panel(void);
 
 extern Boolean	style_dirty_flag;
 
@@ -50,3 +56,5 @@ typedef struct Style_family {
   char *name;
   Style style[MAX_STYLE_FAMILY];
 } Style_family;
+
+#endif
