@@ -1,14 +1,15 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 2016-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and documentation
  * files (the "Software"), including without limitation the rights to use,
- * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies of
  * the Software, and to permit persons who receive copies from any such
  * party to do so, with the only requirement being that the above copyright
  * and this permission notice remain intact.
@@ -18,8 +19,12 @@
 #ifndef W_UTIL_H
 #define W_UTIL_H
 
+#include <assert.h>
+#include <X11/Intrinsic.h>     /* includes X11/Xlib.h, which includes X11/X.h */
+
 #include "w_indpanel.h"
 
+
 /* constant values used for popup_query */
 
 #define QUERY_YESCAN	0
@@ -60,9 +65,10 @@
 #define	LARGE_CHK	True	/* use large checkmark */
 #define	SMALL_CHK	False	/* use small checkmark (don't use large) */
 
-/* number of arrow types */
 
-#define NUM_ARROW_TYPES		30
+#define NUM_ARROW_TYPES		30	/* number of arrow types */
+
+#define MAX_GRID_STRLEN		8	/* max strlen in grid_xxx_choices[] */
 
 /* EXPORTS */
 
@@ -76,12 +82,11 @@ extern	void	reset_grid_menus(Boolean inches);
 
 extern	Boolean	check_action_on(void);
 extern	void	check_for_resize(Widget tool, XButtonEvent *event, String *params, Cardinal *nparams);
-extern	void	check_colors(void);
 
 extern	Widget	make_pulldown_menu(char **entries, Cardinal nent, int divide_line, char *divide_message, Widget parent, XtCallbackProc callback);
 extern	Widget	make_color_popup_menu(Widget parent, char *name, XtCallbackProc callback, Boolean include_transp, Boolean include_backg);
 extern	void	set_color_name(int color, char *buf);
-extern	void	set_but_col(Widget widget, Pixel color);
+extern	void	set_but_col(Widget widget, int color);
 extern	Widget	MakeIntSpinnerEntry(Widget parent, Widget *text, char *name, Widget below, Widget beside, XtCallbackProc callback, char *string, int min, int max, int inc, int width);
 extern	Widget	MakeFloatSpinnerEntry(Widget parent, Widget *text, char *name, Widget below, Widget beside, XtCallbackProc callback, char *string, float min, float max, float inc, int width);
 extern	Widget	CreateCheckbutton(char *label, char *widget_name, Widget parent, Widget below, Widget beside, Boolean manage, Boolean large, Boolean *value, XtCallbackProc user_callback, Widget *togwidg);
@@ -94,26 +99,26 @@ extern	Pixmap	sm_check_pm, sm_null_check_pm;
 #define check_height 16
 #define sm_check_width 10
 #define sm_check_height 10
-extern	Pixmap	balloons_on_bitmap;
-extern	Pixmap	balloons_off_bitmap;
 extern	Pixmap	menu_arrow, menu_cascade_arrow;
 extern	Pixmap	arrow_pixmaps[NUM_ARROW_TYPES+1];
 extern	Pixmap	diamond_pixmap;
 extern	Pixmap	linestyle_pixmaps[NUM_LINESTYLE_TYPES];
 extern	char    *panel_get_value(Widget widg);
 extern	void	panel_set_value(Widget widg, char *val);
-extern	void	panel_set_int(Widget widg, int intval), panel_set_float(Widget widg, float floatval, char *format);
+extern	void	panel_set_int(Widget widg, int intval);
+extern	void	panel_set_float(Widget widg, float floatval, char *format);
 extern	void	update_wm_title(char *name);
 extern	void	get_pointer_win_xy(int *xposn, int *yposn);
 extern	void	get_pointer_root_xy(int *xposn, int *yposn);
-extern	void	spinner_up_down(Widget w, XButtonEvent *ev, String *params, Cardinal *num_params);
+extern	void	spinner_up_down(Widget w, XButtonEvent *ev, String *params,
+				Cardinal *num_params);
 extern	void	clear_splash(void);
 extern	void	InstallScroll(Widget widget);
 extern	void	InstallScrollParent(Widget widget);
-extern  void fix_converters(void);
+extern  void	fix_converters(void);
+extern void	XSyncOn(void);
+extern void	XSyncOff(void);
 
-extern	Boolean	user_colors_saved;
-extern	Boolean	nuser_colors_saved;
 
 /*
  * Author:	Doyle C. Davidson
@@ -137,21 +142,33 @@ extern	Boolean	nuser_colors_saved;
  *	}
  */
 
-#include <assert.h>
 
 #define ArgCount	_ArgCount
 #define Args		_ArgList
+
+#ifdef NDEBUG
+#define DeclareArgs(n)		Arg Args[n]; int ArgCount
+#define DeclareStaticArgs(n)	static Arg Args[n]; static int ArgCount
+#define NextArg(name, val)						 \
+			do {	XtSetArg(Args[ArgCount], (name), (val)); \
+				++ArgCount;				 \
+			} while (0)
+#else /* NDEBUG */
+
 #define ArgCountMax	_ArgCountMax
+#define DeclareArgs(n)		Arg Args[n]; int ArgCountMax = n; int ArgCount
+#define DeclareStaticArgs(n)	static Arg Args[n]; static int ArgCountMax = n;\
+				static int ArgCount
+#define NextArg(name, val)						\
+			do {	assert(ArgCount < ArgCountMax);		\
+				XtSetArg(Args[ArgCount], (name), (val));\
+				ArgCount++;				\
+			} while (0)
 
-#define DeclareArgs(n)	Arg Args[n]; int ArgCountMax = n; int ArgCount
+#endif	/* NDEBUG */
 
-#define DeclareStaticArgs(n)  static Arg Args[n]; static int ArgCountMax = n; static int ArgCount
-
-#define FirstArg(name, val) \
+#define FirstArg(name, val)	\
 	do { XtSetArg(Args[0], (name), (val)); ArgCount=1;} while (0)
-#define NextArg(name, val) \
-	do { assert(ArgCount < ArgCountMax); \
-		XtSetArg(Args[ArgCount], (name), (val)); ArgCount++;} while (0)
 #define GetValues(n)	XtGetValues(n, Args, ArgCount)
 #define SetValues(n)	XtSetValues(n, Args, ArgCount)
 
@@ -163,6 +180,12 @@ typedef struct {
 	float	inc;		/* how much to inc/dec spinner with each click */
 } spin_struct;
 
+/* A clumsy exercise to avoid -Wpointer-to-int-cast warnings.	*/
+typedef union {
+	XtPointer	ptr;
+	int		val;
+} ptr_int;
+
 extern void app_flush (void);
 extern void file_msg_add_grab (void);
 extern void process_pending (void);
@@ -174,6 +197,5 @@ extern void save_user_colors (void);
 extern int popup_query(int query_type, char *message);
 extern void create_bitmaps(void);
 extern void splash_screen(void);
-extern int xallncol(char *name, XColor *color, XColor *exact);
 
 #endif /* W_UTIL_H */
