@@ -1,8 +1,10 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1991 by Henning Spruth
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
+ * Parts Copyright (c) 1991 by Henning Spruth
+ * Parts Copyright (c) 2016-2022 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,30 +17,29 @@
  *
  */
 
-#include <X11/keysym.h>
-#include "fig.h"
+#include "w_zoom.h"
+
+#include <stdlib.h>
+#include <X11/Xlib.h>
+
 #include "resources.h"
 #include "mode.h"
-#include "object.h"
-#include "paintop.h"
-#include "u_create.h"
 #include "u_elastic.h"
 #include "u_pan.h"
 #include "w_canvas.h"
+#include "w_cursor.h"
 #include "w_file.h"
 #include "w_indpanel.h"
-#include "w_msgpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
-#include "w_zoom.h"
+#include "xfig_math.h"
 
-#include "w_cursor.h"
 
 /* global for w_canvas.c */
 
 Boolean	zoom_in_progress = False;
 
-static void	do_zoom(int x, int y), cancel_zoom(void);
+static void	do_zoom(int x, int y);
 static void	init_zoombox_drawing(int x, int y);
 
 static void	(*save_kbd_proc) ();
@@ -168,14 +169,7 @@ do_zoom(int x, int y)
     if (dimx && dimy) {
 	scalex = ZOOM_FACTOR * CANVAS_WD / (float) dimx;
 	scaley = ZOOM_FACTOR * CANVAS_HT / (float) dimy;
-
 	display_zoomscale = (scalex > scaley ? scaley : scalex);
-	if (display_zoomscale <= 1.0)	/* keep to 0.1 increments */
-	    display_zoomscale = (int)((display_zoomscale+0.09)*10.0)/10.0 - 0.1;
-
-	/* round if integral zoom is on (indicator panel in zoom popup) */
-	if (integral_zoom && display_zoomscale > 1.0)
-	    display_zoomscale = round(display_zoomscale);
 	show_zoom(&ind_switches[ZOOM_SWITCH_INDEX]);
     }
     /* restore state */
@@ -190,7 +184,7 @@ do_zoom(int x, int y)
     zoom_in_progress = False;
 }
 
-static void
+void
 cancel_zoom(void)
 {
     elastic_box(my_fix_x, my_fix_y, my_cur_x, my_cur_y);
