@@ -35,7 +35,6 @@
 #include <limits.h>
 #include <math.h>
 
-#include "fig.h"
 #include "resources.h"	/* must come before mode.h; struct appres */
 #include "mode.h"
 
@@ -77,8 +76,10 @@ compare(int xc, int yc, int n, int pts[n][2])
 }
 
 int
-main(void)
+main(int argc, char *argv[])
 {
+	(void)	argc;
+	(void)	argv;
 	int x, y, yspacing, xc, yc, dist, xdist;
 	int error = 0;
 	int pts_square[2][2];
