@@ -3,7 +3,7 @@
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 2016-2020 by Thomas Loimer
+ * Parts Copyright (c) 2016-2021 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -39,6 +39,9 @@ main(int argc, char *argv[])
 	int	llx, lly, urx, ury;
 
 	appres.ghostscript = GSEXE;
+	/* On error, gs_mediabox calls file_msg(), which tries to open a Widget
+	   if update_figs is False. Set update_figs to True. */
+	update_figs = 1;
 
 	if (gs_mediabox(argv[1], &llx, &lly, &urx, &ury) == -3)
 		return 77;
